#include "datetime.h"
#include "ui_datetime.h"
#include "dateManager.h"
#include "dateobject.h"
#include <KAuth/KAuthAction>
#include <KAuth/KAuthExecuteJob>

DateTime::DateTime(QWidget *parent) :
    Page(parent),
    ui(new Ui::DateTime)
{
    ui->setupUi(this);

    QFile f(":/qss/datetime.qss");
    if(f.open(QIODevice::ReadOnly))
    {
        QString style = f.readAll().replace("\n", "");
        this->setStyleSheet(style);
    }

    QHBoxLayout * lay_timemode = new QHBoxLayout();
    lay_timemode->setContentsMargins(0,0,0,0);
    lay_timemode->setSpacing(0);
    ui->w_timemode->setLayout(lay_timemode);
    toggle_time = new NdeWidget::QMaterialToggle(ui->w_timemode);
    connect(toggle_time, &NdeWidget::QMaterialToggle::toggled, this, &DateTime::onTimeModeClick);
    lay_timemode->addWidget(toggle_time);

    QHBoxLayout * lay_auto = new QHBoxLayout();
    lay_auto->setContentsMargins(0,0,0,0);
    lay_auto->setSpacing(0);
    ui->w_autotime->setLayout(lay_auto);
    toggle_auto = new NdeWidget::QMaterialToggle(ui->w_autotime);
    connect(toggle_auto, &NdeWidget::QMaterialToggle::toggled, this, &DateTime::onAutotimeClick);
    lay_auto->addWidget(toggle_auto);

    InitDateTime();
    connect(&dateface,&NsdlDateTime::ChangeNTPResult,this,&DateTime::onAutotimeResult);
    connect(&dateface,&NsdlDateTime::timemodeChanged,this,&DateTime::onTimeModeChanged);

    connect(ui->frame_zone, &ClickedFrame::clicked,this,&DateTime::onChangezoneClick);
    connect(ui->frame_time, &ClickedFrame::clicked,this,&DateTime::onChangetimeClick);
}

DateTime::~DateTime()
{
    delete ui;
}

void DateTime::refreshPage()
{

}

void DateTime::onTimeModeClick(){

    if(toggle_time->isChecked()){
        dateface.setHourMode(2);
        format_time = "yyyy-MM-dd hh:mm";
    } else {
        dateface.setHourMode(1);
        format_time = "yyyy-MM-dd AP hh:mm ";
    }
}

void DateTime::onAutotimeClick(){
    if(bclick){
        toggle_auto->setChecked(!toggle_auto->isChecked());
        return ;
    }
    bclick = true;
    bool autotime = toggle_auto->isChecked();
    KAuth::Action action("org.nde.auth.helper.setntp");
    if (!action.isValid()) {
        return ;
    }
    action.setHelperId(QStringLiteral("org.nde.auth.helper"));

    action.addArgument(QStringLiteral("enable"), autotime);

    KAuth::ExecuteJob *job = action.execute();
    if(job->exec()) {
    } else {
    }
    QTimer r;
    r.singleShot(100,[this](){
        this->bclick=false;
    });
    onsetAutotimeMode();

}

void DateTime::onsetAutotimeMode(){
    bool autotime = dateface.automaticTimeUpdate();
    toggle_auto->setChecked(autotime);
    if(autotime){
        hideautotime();
    } else {
        showautotime();
    }

}

void DateTime::onAutotimeResult(bool result){
    QTimer r;
    r.singleShot(100,[this](){
        this->bclick=false;
    });
    QTimer::singleShot(10,this,&DateTime::onsetAutotimeMode);

}

void DateTime::showEvent(QShowEvent *event){
    InitDateTime();
}

void DateTime::onChangezoneClick(){
    Q_EMIT nextPage(Zone::pageKey());

}
void DateTime::onChangetimeClick(){
    Q_EMIT nextPage(TimeSpliter::pageKey());
}

void DateTime::onTimeModeChanged() {
    InitTimemode();
}

void DateTime::hideautotime(){
    disconnect(ui->frame_zone, &ClickedFrame::clicked,this,&DateTime::onChangezoneClick);
    disconnect(ui->frame_time, &ClickedFrame::clicked,this,&DateTime::onChangetimeClick);
    ui->label_next->hide();
    ui->line_2->hide();
    ui->frame_time->hide();
}
void DateTime::showautotime(){
    if(!ui->label_next->isVisible())
    {
        connect(ui->frame_zone, &ClickedFrame::clicked,this,&DateTime::onChangezoneClick);
        connect(ui->frame_time, &ClickedFrame::clicked,this,&DateTime::onChangetimeClick);
        ui->label_next->show();
        ui->line_2->show();
        ui->frame_time->show();
    }
}

void DateTime::InitZone(){
    //toggle_time->setChecked(true);
    bool autotime = dateface.automaticTimeUpdate();
    toggle_auto->setChecked(autotime);
    QString zonename = dateface.timezone();
    QList<QObject*> dataList = DateManager().getDateModel();
    int size = dataList.size();
    for (int i=0; i < size; i++) {
        QObject * t = dataList[i];
        DateObject* tob = static_cast<DateObject*>(t);
        if(tob->englishZoneName().compare(dateface.timezone()) == 0){
            zonename = tob->zoneName();
            break;
        }
    }
    ui->label_zone->setText(zonename);

    if(autotime){
        hideautotime();
    }
}

void DateTime::InitTime(){
    if(!CurrentTimer){
        CurrentTimer = new QTimer(this);
    }
    //设置时间显示的字体
    //定时器
    if(CurrentTimer->isActive()) CurrentTimer->stop();
    CurrentTimer->start(500);
    //使用定时器信号槽，尽快更新时间的显示
    connect(CurrentTimer,&QTimer::timeout,[=](){
        QDateTime current_time = QDateTime::currentDateTime();
        //显示时间，格式为：年-月-日 时：分：秒 周几
        QString StrCurrentTime = current_time.toString(format_time);
        //时间显示格式可自由设定，如hh:mm ddd 显示 时：分 周几
        //具体格式见函数QString QDateTime::​toString(const QString & format) const
        ui->label_showtime->setText(StrCurrentTime);
    });
}

void DateTime::InitTimemode(){
    if(dateface.getHourMode()){
        toggle_time->setChecked(true);
        format_time = "yyyy-MM-dd hh:mm";
    } else {
        toggle_time->setChecked(false);
        format_time = "yyyy-MM-dd AP hh:mm";
    }
}

void DateTime::InitDateTime()
{
    InitTimemode();
    InitZone();
    InitTime();
}
