#include "slidingselector.h"
#include <QDate>
#include <QDebug>
SlidingSelector::SlidingSelector(Type t,QWidget *parent) : type(t), QWidget(parent)
{
    frame_color=QColor(54,54,54);
    partingLine_color=QColor("#e5e5e7");
    currentNum_color=QColor(255,140,0);
    besideNum_color=QColor(130,130,130);

    space = 0;
    Current_num = 6;
    MAX_NUM = 12;
    MIN_NUM = 1;
    num_size = 6;
    isDragging = 0;
    mouse_move_length=0;
    detal = 0;
    propertyAnimation  = new QPropertyAnimation(this,"mouseMoveLength");
    propertyAnimation->setDuration(500);
    propertyAnimation->setEasingCurve(QEasingCurve::InOutQuad);

    s_strWeekl.append({
            tr("Mon"), tr("Tue"),tr("Wed"),
            tr("Thu"), tr("Fri"),tr("Sat"),tr("Sun"),
        });
}

void SlidingSelector::mousePressEvent(QMouseEvent *event)
{
    propertyAnimation->stop();
    isDragging = 1;
    if(mode == Horizontal)
        mouse_Press=event->x();
    else
        mouse_Press=event->y();
    mouse_move_length = 0;
}

void SlidingSelector::mouseMoveEvent(QMouseEvent *event)
{
    if(isDragging)
    {
        if(mode == Horizontal)
        {
            //数值到边界时，阻止继续往对应方向移动
            if((Current_num == MIN_NUM && event->pos().x() >= mouse_Press)||
            (Current_num == MAX_NUM && event->pos().x() <= mouse_Press))
                return;

            mouse_move_length = event->pos().x() - mouse_Press;

            //若移动速度过快时进行限制
            if(mouse_move_length > (width()-1)/4)
                mouse_move_length = (width()-1)/4;
            else if(mouse_move_length < -(width()-1)/4)
                mouse_move_length = -(width()-1)/4;

            repaint();
            Center_num();
        }
        else
        {
            //数值到边界时，阻止继续往对应方向移动
            if(!m_loop)
            {
                if((Current_num == MIN_NUM && event->pos().y() >= mouse_Press)||
                (Current_num == MAX_NUM && event->pos().y() <= mouse_Press))
                    return;

            }

            int tmp_length =  event->pos().y() - mouse_Press;
            qInfo()<<"tmp_length" << tmp_length << "/t" << "mouse_move_length "<< mouse_move_length;
            if((tmp_length - mouse_move_length < (height()-1)/8 ) && (tmp_length - mouse_move_length > -(height()-1)/8 ))
            {
                return;
            }
            qInfo()<<"tmp_length" << tmp_length << "/t" << "mouse_move_length "<< mouse_move_length;
            mouse_move_length = event->pos().y() - mouse_Press;

            if(mouse_move_length > 0){
                detal = 1;
            }
            else if(mouse_move_length < 0) {
                detal = -1;
            }

            //若移动速度过快时进行限制
            if(mouse_move_length > (height()-1)/4)
                mouse_move_length = (height()-1)/4;
            else if(mouse_move_length < -(height()-1)/4)
                mouse_move_length = -(height()-1)/4;

            repaint();
            Center_num(false,true);
        }

    }
}

void SlidingSelector::mouseReleaseEvent(QMouseEvent *)
{
    if(isDragging)
    {
        isDragging = 0;
        detal = 0;
        Center_num();
    }
}

void SlidingSelector::enterEvent(QEvent *event){
    propertyAnimation->stop();
    setFocus();
}

void SlidingSelector::wheelEvent(QWheelEvent *event){
    if(mode == Horizontal)
    {
        //数值到边界时，阻止继续往对应方向移动
        if((Current_num == MIN_NUM && event->delta()>0)||
        (Current_num == MAX_NUM && event->delta() <0))
            return;
        mouse_move_length =  event->delta();
           //若移动速度过快时进行限制
        if(mouse_move_length > (width()-1)/4)
            mouse_move_length = (width()-1)/4;
        else if(mouse_move_length < -(width()-1)/4)
            mouse_move_length = -(width()-1)/4;
        repaint();
        Center_num();

    }
    else
    {
        //数值到边界时，阻止继续往对应方向移动
        if(!m_loop)
        {
            if((Current_num == MIN_NUM && event->delta()>0)||
            (Current_num == MAX_NUM && event->delta()<0))
                return;
        }

        mouse_move_length = event->delta();
        //若移动速度过快时进行限制
        if(mouse_move_length > (height()-1)/4)
            mouse_move_length = (height()-1)/4;
        else if(mouse_move_length < -(height()-1)/4)
            mouse_move_length = -(height()-1)/4;
        repaint();
        Center_num(true,true);

    }
}

void SlidingSelector::paintEvent(QPaintEvent *)
{
    QPainter painter(this);
    painter.setRenderHint(QPainter::Antialiasing, true);

    paint_frame(&painter);
    paint_parting_line(&painter);

    int Width = width()-1;
    int Height = height()-1;

    if(mode == Horizontal)
    {
        if(mouse_move_length >= Width/4 && Current_num > MIN_NUM)
        {
            mouse_Press += Width/4;
            mouse_move_length -= Width/4;
            Current_num -= 1;
        }
        if(mouse_move_length <= -Width/4 && Current_num < MAX_NUM)
        {
            mouse_Press -= Width/4;
            mouse_move_length += Width/4;
            Current_num += 1;
        }

        //中间数字
        paint_num(&painter,Current_num,mouse_move_length,1);

        //两侧数字1
        if(Current_num != MIN_NUM)
            paint_num(&painter,Current_num-1,mouse_move_length-Width/4+8,0);
        if(Current_num != MAX_NUM)
            paint_num(&painter,Current_num+1,mouse_move_length+Width/4,0);

        //两侧数字2,超出则不显示
        if(mouse_move_length >= 0 && Current_num-2 >= MIN_NUM)
            paint_num(&painter,Current_num-2,mouse_move_length-Width/2,0);
        if(mouse_move_length <= 0 && Current_num+2 <= MAX_NUM)
            paint_num(&painter,Current_num+2,mouse_move_length+Width/2,0);
    }
    else
    {
        if(mouse_move_length >= Height/3 && (m_loop || Current_num > MIN_NUM))
        {
            mouse_Press += Height/3;
            mouse_move_length -= Height/3;
            Current_num -= 1;
        }
        if(mouse_move_length <= -Height/3 && (m_loop || Current_num < MAX_NUM))
        {
            mouse_Press -= Height/3;
            mouse_move_length += Height/3;
            Current_num += 1;
        }

        if(Current_num < MIN_NUM)
        {
            Current_num = Current_num + MAX_NUM;
        }
        if(Current_num > MAX_NUM)
        {
            Current_num = Current_num - MAX_NUM;
        }

        //中间数字
        if(detal == 0) {
             paint_num(&painter,Current_num,mouse_move_length + 2,1);
        } else {
            paint_num(&painter,Current_num,mouse_move_length + 2,0);
        }


        //上下数字1
        if(m_loop || Current_num != MIN_NUM){
            if(detal > 0){
                paint_num(&painter,Current_num-1,mouse_move_length-Height/4 +10,1);
            } else {
                paint_num(&painter,Current_num-1,mouse_move_length-Height/4 +10,0);
            }
        }
        if(m_loop || Current_num != MAX_NUM) {
            if(detal < 0 ){
                paint_num(&painter,Current_num+1,mouse_move_length+Height/4 -10,1);
            } else {
                paint_num(&painter,Current_num+1,mouse_move_length+Height/4 -10,0);
            }
        }

        //上下数字2,超出则不显示
        if(mouse_move_length >= 0 && (m_loop || Current_num-2 >= MIN_NUM))
            paint_num(&painter,Current_num-2,mouse_move_length-Height * 5/12,0);
        if(mouse_move_length <= 0 && (m_loop || Current_num+2 <= MAX_NUM))
            paint_num(&painter,Current_num+2,mouse_move_length+Height * 5/12,0);
    }

}

void SlidingSelector::paint_frame(QPainter *painter)
{
    painter->save();

    painter->setPen(Qt::NoPen);
    painter->setBrush(frame_color);

    painter->drawRect(space,space,width()-space*2,height()-space*2);

    painter->restore();
}

QString SlidingSelector::datetostring(QDate d){

    int year = d.year();
    int day = d.dayOfWeek();
    int month = d.month();
    int date = d.day();
    QString datestr = QString::number(month) + " " + tr("mth")  + " " + QString::number(date) + " " + tr("day");
    QString week = s_strWeekl[day-1];
    datestr = datestr + " " +week;
    return datestr;
}

void SlidingSelector::paint_num(QPainter *painter,int num,int mouse_move_length,int mid)
{
    painter->save();

    if(num < MIN_NUM)
    {
        num = num + MAX_NUM + 1;
    }
    if(num > MAX_NUM)
    {
        num = num - MAX_NUM - 1;
    }


    int Width = width()-1;
    int Height = height()-1;
    QString text = QString::number(num);

    switch (type) {
    case YEAR:
        break;
    case DATE:{
        QDate date;
        date.setDate(year,1,1);
        date = date.addDays(num-1);
        text = datetostring(date);
        break;
    }
    case MON:
        if(num==1){
            text = tr("AM");
        } else{
            text = tr("PM");
        }
        break;
    case HOUR:
        break;
    case MIN:
        break;
    }

    if(mode == Horizontal)
    {
        int size = (Width - mouse_move_length)/num_size;//像素尺寸
        int width = Width/2-3*(mouse_move_length);//数字框宽度
        int x = Width/2+mouse_move_length-width/2;//数字框x坐标

//        QFont font;
//        font.setPixelSize(size);
//        painter->setFont(font);
        if(mid == 1)
            painter->setPen(currentNum_color);
        else
            painter->setPen(besideNum_color);

        painter->drawText(QRectF(x,0,width,Height),Qt::AlignCenter,text);
    }
    else
    {
        int size = 14;//像素尺寸
        int height = Height/5;//数字框宽度
        int y = Height/2+mouse_move_length-height/2;//数字框x坐标

//        QFont font;
//        font.setPixelSize(size);
//        painter->setFont(font);
        if(mid == 1)
            painter->setPen(currentNum_color);
        else
            painter->setPen(besideNum_color);

        painter->drawText(QRectF(14,y,Width,height),Qt::AlignCenter,text);
    }

    painter->restore();

}

void SlidingSelector::paint_parting_line(QPainter *painter)
{
    painter->save();

    QPen pen;
    pen.setBrush(partingLine_color);
    pen.setWidth(1);
    pen.setCapStyle(Qt::RoundCap);
    pen.setStyle(Qt::SolidLine);
    painter->setPen(pen);

    if(mode == Horizontal)
    {
        int up_down_space=height()/10;
        QPoint line_left_up=QPoint(space+(width()-space*2)/3,space+up_down_space);
        QPoint line_left_down=QPoint(space+(width()-space*2)/3,height()-space-up_down_space);
        QPoint line_right_up=QPoint(space+((width()-space*2)/3)*2,space+up_down_space);
        QPoint line_right_down=QPoint(space+((width()-space*2)/3)*2,height()-space-up_down_space);
        painter->drawLine(line_left_up,line_left_down);//绘制左侧分割线
        painter->drawLine(line_right_up,line_right_down);//绘制右侧分割线
    }
    else
    {
        int up_down_space=0;
        QPoint line_left_up=QPoint(space+up_down_space,space+(height()-space*2)/5);
        QPoint line_left_down=QPoint(space+(width()-space*2)-up_down_space,space+(height()-space*2)/5);
        painter->drawLine(line_left_up,line_left_down);//绘制左侧分割线
        QPoint line_right_up=QPoint(space+up_down_space,space+((height()-space*2)/5)*2);
        QPoint line_right_down=QPoint(space+(width()-space*2)-up_down_space,space+((height()-space*2)/5)*2);
        painter->drawLine(line_right_up,line_right_down);//绘制右侧分割线
        QPoint line_right_up1=QPoint(space+up_down_space,space+height()*3/5);
        QPoint line_right_down1=QPoint(space+(width()-space*2)-up_down_space,space+height()*3/5);
        painter->drawLine(line_right_up1,line_right_down1);//绘制右侧分割线
        QPoint line_right_up2=QPoint(space+up_down_space,space+height()*4/5);
        QPoint line_right_down2=QPoint(space+(width()-space*2)-up_down_space,space+height()*4/5);
        painter->drawLine(line_right_up2,line_right_down2);//绘制右侧分割线
    }

    painter->restore();

}

void SlidingSelector::Center_num(bool move,bool add)
{
    if(Current_num < 0)
        return;
    if(mode == Horizontal)
    {
        //将数字矫正到中心
        if(mouse_move_length > width()/8)
        {
            propertyAnimation->setStartValue((width()-1)/8-mouse_move_length);
            propertyAnimation->setEndValue(0);
            Current_num--;
        }
        else if(mouse_move_length > -width()/8)
        {
            propertyAnimation->setStartValue(mouse_move_length);
            propertyAnimation->setEndValue(0);
        }
        else if(mouse_move_length < -width()/8)
        {
            propertyAnimation->setStartValue(-(width()-1)/8-mouse_move_length);
            propertyAnimation->setEndValue(0);
            Current_num++;
        }

        Q_EMIT currentValueChanged(Current_num);
        propertyAnimation->start();
    }
    else
    {
        //将数字矫正到中心
        if(mouse_move_length > height()/8)
        {
            if(move){
                propertyAnimation->setStartValue((height()-1)/8-mouse_move_length);
                propertyAnimation->setEndValue(0);
            }
            if(add)
                Current_num--;
        }
        else if(mouse_move_length > -height()/8)
        {
            if(move){
                propertyAnimation->setStartValue(mouse_move_length);
                propertyAnimation->setEndValue(0);
            }

        }
        else
        {
            if(move){
                propertyAnimation->setStartValue(-(height()-1)/8-mouse_move_length);
                propertyAnimation->setEndValue(0);
            }
            if(add)
                Current_num++;
        }

        if(Current_num < MIN_NUM)
        {
            Current_num = Current_num + MAX_NUM + 1;
        }
        if(Current_num >  MAX_NUM)
        {
            Current_num = Current_num - MAX_NUM - 1;
        }
        if(move){
            Q_EMIT currentValueChanged(Current_num);
            propertyAnimation->start();
        }

    }  
}

int SlidingSelector::readMouseMoveLength()
{
    return mouse_move_length;
}

void SlidingSelector::setMouseMoveLength(int n)
{
    mouse_move_length = n;

    repaint();
}

void SlidingSelector::setMode(Mode mode)
{
    this->mode = mode;

    repaint();
}
void SlidingSelector::setRange(int min,int max)
{
    MIN_NUM = min;
    MAX_NUM = max;

    if(Current_num < min)
        Current_num = min;
    if(Current_num > max)
        Current_num = max;

    //计算字符尺寸
    num_size = 3;
    int temp = MAX_NUM;
    while(temp > 0)
    {
        temp /= 10;
        num_size++;
    }

    setCurrentNum((max+min)/2);

    repaint();
}

void SlidingSelector::setCurrentNum(int num)
{
    this->Current_num = num;

    repaint();
}

void SlidingSelector::setCurrentFrame_color(QColor color)
{
    this->frame_color = color;

    repaint();
}

void SlidingSelector::setCurrentPartingLine_color(QColor color)
{
    this->partingLine_color = color;

    repaint();
}

void SlidingSelector::setCurrentNum_color(QColor color)
{
    this->currentNum_color = color;

    repaint();
}

void SlidingSelector::setBesideNum_color(QColor color)
{
    this->besideNum_color = color;

    repaint();
}
