﻿#ifndef SLIDINGSELECTOR_H
#define SLIDINGSELECTOR_H

#include <QWidget>
#include <QPainter>
#include <QMouseEvent>
#include <QPropertyAnimation>
#include "QColor"

enum Mode{Horizontal=0,Vertical};
enum Month{Jan=1,Feb,Mar,Apr,May,Jun,Jul,Aug,Sep,Oct,Nov,Dec};
enum Type{YEAR=1,DATE,MON,HOUR,MIN};
class SlidingSelector : public QWidget
{
    Q_OBJECT
    Q_PROPERTY(int mouseMoveLength READ readMouseMoveLength WRITE setMouseMoveLength)

public:
    explicit SlidingSelector(Type t,QWidget *parent = nullptr);


public:
    void setMode(Mode mode);//设置模式，水平模式和垂直模式
    void setRange(int min,int max);//设置可取值范围
    void setCurrentNum(int num);//设置当前值
    void setCurrentFrame_color(QColor color);//设置背景颜色
    void setCurrentPartingLine_color(QColor color);//设置分隔线颜色
    void setCurrentNum_color(QColor color);//设在当前值颜色
    void setBesideNum_color(QColor color);//设在两边值颜色

    int getCurrentNum() const //获取当前值
    {
        return Current_num;
    }



    int * getCurrentRange() const  //获取当前值域
    {
        int *temp;
        *(temp) = this->MIN_NUM;
        *(temp+1) = this->MAX_NUM;
        return temp;
    }

    QColor getCurrentFrame_color() const  //获取背景颜色
    {
        return frame_color;
    }

    QColor getCurrentPartingLine_color() const  //获取分隔线颜色
    {
        return partingLine_color;
    }

    QColor getCurrentNum_color() const  //获取当前值颜色
    {
        return currentNum_color;
    }

    QColor getBesideNum_color() const  //获取两边值颜色
    {
        return besideNum_color;
    }

    void setyear(int y) {
        year = y;
        repaint();
    }
    void setloop(bool loop){
        m_loop = loop;
    }
    QString datetostring(QDate d);

private:
    Mode mode=Horizontal;  //模式
    Month month;
    bool isDragging;

    QPropertyAnimation *propertyAnimation;  //实现动画

    QColor frame_color;  //背景颜色
    QColor partingLine_color;  //分隔线颜色
    QColor currentNum_color;  //当前值颜色
    QColor besideNum_color;  //两边值颜色

    int space;  //控件与框体间距
    int Current_num;  //当前值大小
    int MAX_NUM;  //值域的最大值
    int MIN_NUM;  //值域的最小值
    int num_size;  //数值尺寸
    int mouse_Press;  //鼠标点击处的坐标
    int mouse_move_length;  //鼠标移动的距离

    bool m_loop{false};        //是否循环显示

protected:
    void mousePressEvent(QMouseEvent *);  //鼠标点击事件
    void mouseMoveEvent(QMouseEvent *);  //鼠标移动事件
    void mouseReleaseEvent(QMouseEvent *);  //鼠标释放事件
    void wheelEvent(QWheelEvent*); //鼠标滚动事件
    void enterEvent(QEvent *event); //鼠标进入事件
    void paintEvent(QPaintEvent *);  //绘图事件
    void paint_frame(QPainter *);  //绘制背景框
    void paint_num(QPainter *,int num,int mouse_move_length,int mid);  //绘制数字
    void paint_parting_line(QPainter *);  //绘制分割线

    void Center_num(bool move = true,bool add = false);
    int readMouseMoveLength();
    void setMouseMoveLength(int n);


Q_SIGNALS:
    void currentValueChanged(int value);  //当前值发生了改变
private:
    Type type;
    int year;
    int detal;
    QStringList s_strWeekl;
};

#endif // SLIDINGSELECTOR_H
