#include "zone.h"
#include "ui_zone.h"
#include "datetime.h"
#include "dateManager.h"
#include "dateobject.h"
#include <QList>
#include <KAuth/KAuthAction>
#include <KAuth/KAuthExecuteJob>


ZoneItem::ZoneItem(QFrame *parent):
    QFrame(parent)
{
    layout = new QHBoxLayout();
    l_zone = new QLabel(this);
    layout->addSpacerItem(new QSpacerItem(20,20,QSizePolicy::Fixed));
    l_zone->setText("123");
    layout->addWidget(l_zone);
    layout->addSpacerItem(new QSpacerItem(20,20,QSizePolicy::Expanding));
    l_select = new QLabel(this);
    l_select->setPixmap(QPixmap(":/image/select.png"));
    l_select->setVisible(false);
    layout->addWidget(l_select);
    layout->addSpacerItem(new QSpacerItem(20,20,QSizePolicy::Fixed));
    setLayout(layout);
    setFixedHeight(50);
}

void ZoneItem::setSelectState(bool state){
    l_select->setVisible(state);
}

void ZoneItem::setText(QString text){
    l_zone->setText(text);
}

void ZoneItem::mouseReleaseEvent(QMouseEvent *event){
    Q_EMIT click(this);
}

ZoneItem::~ZoneItem(){

}

Zone::Zone(QWidget *parent) :
    Page(parent),
    item(nullptr),
    ui(new Ui::Zone)
{
    ui->setupUi(this);

    QFile f(":/qss/zone.qss");
    if(f.open(QIODevice::ReadOnly))
    {
        QString style = f.readAll().replace("\n", "");
        this->setStyleSheet(style);
    }

    connect(ui->lineEdit, &QLineEdit::textChanged, this, &Zone::onreturnPressed);
    dataList = DateManager().getDateModel();
    connect(&dateface,&NsdlDateTime::timezoneChangedSucess,this,&Zone::onSelectSucss);
    connect(&dateface,&NsdlDateTime::timezoneChangedError,this,&Zone::onSelectError);

    init();
}

Zone::~Zone()
{
    delete ui;
}

void Zone::backBtnClicked()
{
    //
}

void Zone::applyBtnClicked()
{
    QString text = newitem->enname();
    KAuth::Action action("org.nde.auth.helper.settimezone");
    if (!action.isValid()) {
        return ;
    }
    action.setHelperId(QStringLiteral("org.nde.auth.helper"));

    action.addArgument(QStringLiteral("timezone"), text);

    KAuth::ExecuteJob *job = action.execute();
    if(job->exec()) {
        Q_EMIT buttomBtnVisibleChanged(Page::PAGE_BBTN_APPLY, false);
        Q_EMIT backPage();
        return;
    } else {
    }
}

void Zone::refreshPage()
{
    Q_EMIT buttomBtnVisibleChanged(Page::PAGE_BBTN_APPLY, false);
    ui->lineEdit->clear();
    if(item) {
        item->setSelectState(false);
    }
    QString zonename = dateface.timezone();
    int size = itemlist.size();
    for (int i=0; i < size; i++) {
        ZoneItem * t = itemlist[i];
        if(t->enname().contains(zonename,Qt::CaseInsensitive)){
            t->setSelectState(true);
            item = t;
        }
    }
}

void Zone::onclick(ZoneItem* list){
    newitem = list;
    if(item) {
        item->setSelectState(false);
    }
    item = newitem;
    newitem->setSelectState(true);
     Q_EMIT buttomBtnVisibleChanged(Page::PAGE_BBTN_APPLY, true);
}

void Zone::onreturnPressed(){
    QString zonename = dateface.timezone();
    QString text = ui->lineEdit->text();
    int size = itemlist.size();
    int n = 0;
    for (int i=0; i < size; i++) {
        ZoneItem * t = itemlist[i];
        if(t->Text().contains(text,Qt::CaseInsensitive) || t->enname().contains(text,Qt::CaseInsensitive)){
            t->show();
            n++;
        } else {
            t->hide();
        }
    }
}

void Zone::onSelectSucss(){
    Q_EMIT buttomBtnVisibleChanged(Page::PAGE_BBTN_APPLY, false);
}

void Zone::onSelectError(){

}

void Zone::init(){
    itemlist.clear();
    int size = dataList.size();
    QString zonename = dateface.timezone();
    for (int i=0; i < size; i++) {
        QObject * t = dataList[i];
        ZoneItem* itemp = new ZoneItem();
        itemp->setText(static_cast<DateObject*>(t)->zoneName());
        itemp->setenText(static_cast<DateObject*>(t)->englishZoneName());
        if(static_cast<DateObject*>(t)->englishZoneName().compare(zonename)==0){
            itemp->setSelectState(true);
            item = itemp;
        }
        ui->layout_list->addWidget(itemp);
        itemlist.push_back(itemp);
        connect(itemp,&ZoneItem::click,this,&Zone::onclick);
    }
}
