#include "defaultapp.h"
#include "ui_defaultapp.h"
#include "defaultappsetting.h"
#include <QFile>
#include <glib.h>

DefaultApp::DefaultApp(QWidget *parent) :
    Page(parent),
    ui(new Ui::DefaultApp)
{
    ui->setupUi(this);

    QFile f(":/qss/defaultapp.qss");
    if(f.open(QIODevice::ReadOnly))
    {
        QString style = f.readAll().replace("\n", "");
        this->setStyleSheet(style);
    }

    delegate = new QStyledItemDelegate;
    ui->web_combobox->setItemDelegate(delegate);
    ui->mail_combobox->setItemDelegate(delegate);
    ui->music_combobox->setItemDelegate(delegate);
    ui->video_combobox->setItemDelegate(delegate);
    ui->image_combobox->setItemDelegate(delegate);
    ui->word_combobox->setItemDelegate(delegate);
    ui->text_combobox->setItemDelegate(delegate);
    ui->filemanager_combobox->setItemDelegate(delegate);
    ui->document_combobx->setItemDelegate(delegate);
    ui->archive_combobx->setItemDelegate(delegate);

    initUI();
}

DefaultApp::~DefaultApp()
{
    delete ui;
    delete applet;
    delete delegate;
}

void DefaultApp::refreshPage()
{

}


void DefaultApp::initUI()
{
    initItems();
    connect(ui->web_combobox,SIGNAL(currentIndexChanged(int)),this,SLOT(slot_set_default_web_app()));
    connect(ui->mail_combobox,SIGNAL(currentIndexChanged(int)),this,SLOT(slot_set_default_mail_app()));
    connect(ui->music_combobox,SIGNAL(currentIndexChanged(int)),this,SLOT(slot_set_default_music_app()));
    connect(ui->video_combobox,SIGNAL(currentIndexChanged(int)),this,SLOT(slot_set_default_vedio_app()));
    connect(ui->image_combobox,SIGNAL(currentIndexChanged(int)),this,SLOT(slot_set_default_image_app()));
    connect(ui->word_combobox,SIGNAL(currentIndexChanged(int)),this,SLOT(slot_set_default_word_app()));
    connect(ui->text_combobox,SIGNAL(currentIndexChanged(int)),this,SLOT(slot_set_default_text_editor_app()));
    connect(ui->filemanager_combobox,SIGNAL(currentIndexChanged(int)),this,SLOT(slot_set_default_file_manager_app()));
    connect(ui->document_combobx,SIGNAL(currentIndexChanged(int)),this,SLOT(slot_set_default_document_app()));
    connect(ui->archive_combobx,SIGNAL(currentIndexChanged(int)),this,SLOT(slot_set_default_archive_app()));
}

void DefaultApp::initItems()
{
    applet = new NDEApplet;
   //获取系统应用程序列表
    applet->web_browsers = g_app_info_get_all_for_type("x-scheme-handler/http");
    applet->mail_readers=g_app_info_get_all_for_type("x-scheme-handler/mailto");
    applet->music_players=g_app_info_get_all_for_type("audio/x-vorbis+ogg");
    applet->video_players=g_app_info_get_all_for_type("video/x-ogm+ogg");
    applet->image_viewers=g_app_info_get_all_for_type("image/png");
    applet->word_viewers=g_app_info_get_all_for_type("application/msword");
    applet->text_editor=g_app_info_get_all_for_type("text/plain");
    applet->file_manager=g_app_info_get_all_for_type("inode/directory");
    applet->document_viewers=g_app_info_get_all_for_type("application/pdf");
    applet->archive_manager=g_app_info_get_all_for_type("application/zip");

    //类型转换
    list_type_change(applet->web_browsers,&web_browsers);
    list_type_change(applet->mail_readers,&mail_readers);
    list_type_change(applet->music_players,&music_players);
    list_type_change(applet->video_players, &video_players);
    list_type_change(applet->image_viewers, &image_viewers);
    list_type_change(applet->word_viewers,&word_viewers);
    list_type_change(applet->text_editor,&text_editor);
    list_type_change(applet->file_manager,&file_manager);
    list_type_change(applet->document_viewers,&document_viewers);
    list_type_change(applet->archive_manager,&archive_manager);

    //添加下拉列表
    additem_to_combox(ui->web_combobox,&web_browsers);
    additem_to_combox(ui->mail_combobox,&mail_readers);
    additem_to_combox(ui->music_combobox,&music_players);
    additem_to_combox(ui->video_combobox,&video_players);
    additem_to_combox(ui->image_combobox,&image_viewers);
    additem_to_combox(ui->word_combobox,&word_viewers);
    additem_to_combox(ui->text_combobox,&text_editor);
    additem_to_combox(ui->filemanager_combobox, &file_manager);
    additem_to_combox(ui->document_combobx,&document_viewers);
    additem_to_combox(ui->archive_combobx,&archive_manager);

    //显示默认应用
    show_default_app(ui->web_combobox,applet->web_browsers,"x-scheme-handler/http");
    show_default_app(ui->mail_combobox,applet->mail_readers,"x-scheme-handler/mailto");
    show_default_app(ui->music_combobox,applet->music_players,"audio/x-vorbis+ogg");
    show_default_app(ui->video_combobox,applet->video_players,"video/x-ogm+ogg");
    show_default_app(ui->image_combobox,applet->image_viewers,"image/png");
    show_default_app(ui->word_combobox,applet->word_viewers,"application/msword");
    show_default_app(ui->text_combobox,applet->text_editor,"text/plain");
    show_default_app(ui->filemanager_combobox,applet->file_manager,"inode/directory");
    show_default_app(ui->document_combobx,applet->document_viewers,"application/pdf");
    show_default_app(ui->archive_combobx,applet->archive_manager,"application/zip");

    //初始化默认设置
    initsetting();
}


//将软件信息添加到下拉列表
void DefaultApp::additem_to_combox(QComboBox * combox, QList<NDEAppInfo> *applist)
{
    int i = 0;
    QList<NDEAppInfo>::Iterator index;
    for(index=applist->begin();index!=applist->end();++index )
    {
         combox->addItem((applist->at(i)).app_icon,(applist->at(i)).app_name);
         i++;
    }
}

//显示默认应用到combobox
void DefaultApp::show_default_app(QComboBox*  combobox,GList* list, char *mime)
{
    combobox->setCurrentIndex(get_default_app(list,mime));
}

//初始化默认设置
void DefaultApp::initsetting()
{
    slot_set_default_web_app();
    slot_set_default_mail_app();
    slot_set_default_music_app();
    slot_set_default_vedio_app();
    slot_set_default_image_app();
    slot_set_default_word_app();
    slot_set_default_text_editor_app();
    slot_set_default_file_manager_app();
    slot_set_default_document_app();
    slot_set_default_archive_app();
}


//设置默认程序槽函数
void DefaultApp::slot_set_default_web_app()
{
     set_default_app(ui->web_combobox,applet,applet->web_browsers,NA_TYPE_WEB_BROWSER);
}


void DefaultApp::slot_set_default_mail_app()
{
     set_default_app(ui->mail_combobox,applet,applet->mail_readers,NA_TYPE_EMAIL);
}


void DefaultApp::slot_set_default_music_app()
{
     set_default_app(ui->music_combobox,applet,applet->music_players,NA_TYPE_MUSIC);
}


void DefaultApp::slot_set_default_vedio_app()
{
    set_default_app(ui->video_combobox,applet,applet->video_players,NA_TYPE_VIDEO);
}


void DefaultApp::slot_set_default_image_app()
{
    set_default_app(ui->image_combobox,applet,applet->image_viewers,NA_TYPE_IMAGE);
}

void DefaultApp::slot_set_default_word_app()
{
    set_default_app(ui->word_combobox,applet,applet->word_viewers,NA_TYPE_WORD);
}


void DefaultApp::slot_set_default_text_editor_app()
{
    set_default_app(ui->text_combobox,applet,applet->text_editor,NA_TYPE_TEXTEDITOR);
}


void DefaultApp::slot_set_default_file_manager_app()
{
    set_default_app(ui->filemanager_combobox,applet,applet->file_manager,NA_TYPE_FILEMANAGER);
}


void DefaultApp::slot_set_default_document_app()
{
    set_default_app(ui->document_combobx,applet,applet->document_viewers,NA_TYPE_DOCUMENT);
}

void DefaultApp::slot_set_default_archive_app()
{
    set_default_app(ui->archive_combobx,applet,applet->archive_manager,NA_TYPE_ARCHIVER);
}
