#include "keyboard.h"
#include "ui_keyboard.h"
#include <QFile>
#include <Nde/Settings>
#include <QX11Info>
#include <X11/Xlib.h>
#include <X11/XKBlib.h>
#include "keyboardlayout.h"
#include "shortcut.h"

KeyBoard::KeyBoard(QWidget *parent) :
    Page(parent),
    ui(new Ui::KeyBoard),
    mSettings(nullptr),
    mDelay(500),
    mOldDelay(500),
    mInterval(30),
    mOldInterval(30)
{
    ui->setupUi(this);
    QFile f(":/qss/keyboard.qss");
    if(f.open(QIODevice::ReadOnly))
    {
        QString style = f.readAll().replace("\n", "");
        this->setStyleSheet(style);
    }

    initUI();
}

KeyBoard::~KeyBoard()
{
    delete ui;
    delete mSettings;
    mSettings = nullptr;
}

void KeyBoard::refreshPage()
{
    //
}

void KeyBoard::initUI()
{
    QByteArray configName = qgetenv("NDE_SESSION_CONFIG");
    if(configName.isEmpty())
      configName = "session";
    mSettings = new Nde::Settings(QLatin1String(configName), this);

    loadSettings();
    initControls();

    connect(ui->keyboardDelay, &QAbstractSlider::valueChanged, this, &KeyBoard::onSettingsChanged);
    connect(ui->keyboardInterval, &QAbstractSlider::valueChanged, this, &KeyBoard::onSettingsChanged);

    ui->keyboardLayoutFrame->setTitle(tr("KeyBoard Layout"));
    connect(ui->keyboardLayoutFrame, &ClickedFrame::clicked, this,&KeyBoard::onKeyBoardLayoutForward);
    ui->shortcutFrame->setTitle(tr("Shortcut"));
    connect(ui->shortcutFrame, &ClickedFrame::clicked,this, &KeyBoard::onShortcutForward);
}

void KeyBoard::loadSettings() {
  mSettings->beginGroup("Keyboard");
  mOldDelay = mDelay = mSettings->value("delay", 500).toInt();
  mOldInterval = mInterval = mSettings->value("interval", 30).toInt();
  mSettings->endGroup();
}

void KeyBoard::initControls() {
  ui->keyboardDelay->blockSignals(true);
  ui->keyboardDelay->setValue(mDelay);
  ui->keyboardDelay->blockSignals(false);

  ui->keyboardInterval->blockSignals(true);
  ui->keyboardInterval->setValue(mInterval);
  ui->keyboardInterval->blockSignals(false);
}

void KeyBoard::onSettingsChanged()
{
    if(mDelay != ui->keyboardDelay->value() || mInterval != ui->keyboardInterval->value())
    {
      mDelay = ui->keyboardDelay->value();
      mInterval = ui->keyboardInterval->value();
      XkbSetAutoRepeatRate(QX11Info::display(), XkbUseCoreKbd, mDelay, mInterval);

      mSettings->beginGroup("Keyboard");
      mSettings->setValue("delay", mDelay);
      mSettings->setValue("interval", mInterval);
      mSettings->endGroup();
    }
}

void KeyBoard::onKeyBoardLayoutForward()
{
    Q_EMIT nextPage(KeyboardLayout::pageKey());
}

void KeyBoard::onShortcutForward()
{
    Q_EMIT nextPage(Shortcut::pageKey());
}

