#ifndef KEYBOARDINSTANCE_H
#define KEYBOARDINSTANCE_H

#include <QObject>
#include <QMap>
#include <ndesettings.h>

struct LayoutVariantInfo {
  QString name;
  QString description;
  LayoutVariantInfo(QString _name, QString desc): name(_name), description(desc) {
  }
};

struct KeyboardLayoutInfo {
  QString description;
  QList<LayoutVariantInfo> variants;

  KeyboardLayoutInfo(QString desc = QString()): description(desc) {
  }

  const LayoutVariantInfo* findVariant(QString name) const {
    if(!name.isEmpty()) {
      for(const LayoutVariantInfo& vinfo : qAsConst(variants)) {
        if(vinfo.name == name)
          return &vinfo;
      }
    }
    return nullptr;
  }
};

class KeyboardInstance : public QObject
{
    Q_OBJECT
private:
    explicit KeyboardInstance(QObject *parent = nullptr);

public:
    static KeyboardInstance* getIns();

    bool isKeyboardLayoutSelected(QString layout, QString var);
    bool onlyOneKeyboardLayoutSelected();
    void addKeyboardLayout(QString layout, QString var);
    void removeKeyboardLayout(QString layout, QString var);
    QMap<QString, KeyboardLayoutInfo> getAllKeyboardLayout(){
        return m_allKeyboardLayouts;
    }
    QList<QPair<QString, QString> >  getSelectedKeyboardLayout(){
        return m_selKeyboardLayouts;
    }

Q_SIGNALS:
    void signalSelKeyboardLayoutsChanged();

private:
    //加载所有键盘布局
    void loadAllKeyboardLayout();
    //加载选择的键盘布局
    void loadSelectKeyboardLayout();
    //保存配置
    void saveKeyboardLayout();

private:
    QMap<QString, KeyboardLayoutInfo> m_allKeyboardLayouts;
    QList<QPair<QString, QString> > m_selKeyboardLayouts;
    Nde::Settings* m_settings{nullptr};
};

#endif // KEYBOARDINSTANCE_H
