#include "keyboardlayout.h"
#include "ui_keyboardlayout.h"
#include <QFile>
#include <QProcess>
#include "keyboardlayoutitem.h"
#include <QTextCodec>
#include "keyboardinstance.h"
#include "keyboardlayoutlist.h"

KeyboardLayout::KeyboardLayout(QWidget *parent) :
    Page(parent),
    ui(new Ui::KeyboardLayout)
{
    ui->setupUi(this);

    QFile f(":/qss/keyboardlayout.qss");
    if(f.open(QIODevice::ReadOnly))
    {
        QString style = f.readAll().replace("\n", "");
        this->setStyleSheet(style);
    }

    initControls();
    connect(KeyboardInstance::getIns(), &KeyboardInstance::signalSelKeyboardLayoutsChanged,
            this, &KeyboardLayout::refreshKeyboardLayoutList);
}

KeyboardLayout::~KeyboardLayout()
{
    delete ui;
}

void KeyboardLayout::refreshPage()
{
    //
}

void KeyboardLayout::initControls()
{
    while(ui->layout_list->count())
    {
        QLayoutItem * item = ui->layout_list->takeAt(0);
        if(item->widget()) item->widget()->deleteLater();
        delete item;
    }

    QMap<QString, KeyboardLayoutInfo> knownLayouts_ = KeyboardInstance::getIns()->getAllKeyboardLayout();
    QList<QPair<QString, QString> > currentLayouts_ = KeyboardInstance::getIns()->getSelectedKeyboardLayout();
    int count = 0;
    for(QList<QPair<QString, QString> >::iterator it = currentLayouts_.begin(); it != currentLayouts_.end(); ++it)
    {
        QString name = it->first;
        QString variant = it->second;
        bool isFirst = (count == 0);

        const KeyboardLayoutInfo& info = knownLayouts_.value(name);

        const LayoutVariantInfo* vinfo = info.findVariant(variant);
        QString vdes = "";
        if(vinfo) {
            vdes = vinfo->description;
        }
        KeyboardLayoutItem* listItem = new KeyboardLayoutItem(info.description, vdes, KEYBOART_TYPE::Delete, isFirst, this);
        listItem->setKeyboardLayoutData(name);
        listItem->setKeyboardVariantData(variant);
        connect(listItem, &KeyboardLayoutItem::deleteKeyboardLayoutItem, this, &KeyboardLayout::onDeleteKeyboardLayoutItem);
        ui->layout_list->addWidget(listItem);
        if(KeyboardInstance::getIns()->onlyOneKeyboardLayoutSelected())
        {
            listItem->setHideDeleteBtn(true);
        }
        else {
            listItem->setHideDeleteBtn(false);
        }

        ++count;
    }

    KeyboardLayoutItem* addItem = new KeyboardLayoutItem (tr("Add Keyboard Layout"), "", KEYBOART_TYPE::Addlayout, this);
    connect(addItem, &KeyboardLayoutItem::addKeyboardLayoutClicked, this,&KeyboardLayout::onAddKeyboardLayout);
    ui->layout_list->addWidget(addItem);
}

void KeyboardLayout::onAddKeyboardLayout()
{
    Q_EMIT nextPage(KeyboardLayoutList::pageKey());
}

void KeyboardLayout::onDeleteKeyboardLayoutItem()
{
    KeyboardLayoutItem* keyItem = qobject_cast<KeyboardLayoutItem*>(sender());
    if(keyItem)
    {
        if(keyItem->type() != KEYBOART_TYPE::Delete)
        {
            return;
        }

        QString layout = keyItem->getKeyboardLayoutData();
        QString variant = keyItem->getKeyboardVariantData();
        KeyboardInstance::getIns()->removeKeyboardLayout(layout, variant);
    }
}

void KeyboardLayout::refreshKeyboardLayoutList()
{
    initControls();
}
