#include "keyboardlayoutitem.h"
#include "ui_keyboardlayoutitem.h"
#include <QPainter>
#include "keyboardinstance.h"


KeyboardLayoutItem::KeyboardLayoutItem(QString layouts, QString variants,KEYBOART_TYPE  type, bool isFisrt, QWidget *parent) :
    QFrame(parent),
    ui(new Ui::KeyboardLayoutItem),
    mType(type),
    mIsFisrt(isFisrt),
    mChecked(false),
    mIsHideDeteleBtn(false)
{
    mLayouts =layouts;
    mVariants = variants;
    ui->setupUi(this);
    ui->layoutDeleteLabel->hide();
    ui->selectLabel->hide();
    QString text = mLayouts;
    if(!mVariants.isEmpty()) {
        text += " [variant: " + mVariants + "]";
    }
    ui->nameLabel->setText(text);
    if(mIsFisrt && mType == KEYBOART_TYPE::Delete)
    {
        this->setStyleSheet("QFrame {border:none; border-top-left-radius:4px; border-top-right-radius:4px; background-color:#f0f1f2}"
                            "QLabel {color: #222222; text-align: left;}");
    }
    else if(mType == KEYBOART_TYPE::Delete){
        this->setStyleSheet("QFrame {border:none; border-radius:0px; background-color:#f0f1f2}"
                            "QLabel {color: #222222;text-align: left;}");
    }
    if(mType == KEYBOART_TYPE::Addlayout)
    {
        this->setStyleSheet("QFrame {border:none; border-bottom-left-radius:4px; border-bottom-right-radius:4px; background-color:#f0f1f2;}"
                            "QLabel {color: #2c9dfc; text-align: left;}");
    }

    if(mType == KEYBOART_TYPE::Select)
    {
        this->setStyleSheet("QFrame {border:none; border-bottom-left-radius:4px; border-bottom-right-radius:4px; background-color:#f0f1f2;}"
                            "QLabel {color: #222222; text-align: left;}");
    }

    ui->layoutDeleteLabel->installEventFilter(this);
    ui->nameLabel->installEventFilter(this);
    ui->selectLabel->installEventFilter(this);
}

KeyboardLayoutItem::~KeyboardLayoutItem()
{
    delete ui;
}

void KeyboardLayoutItem::setChecked(bool checked)
{
    mChecked = checked;
    ui->selectLabel->setVisible(mChecked);
}

bool KeyboardLayoutItem::eventFilter(QObject *watched, QEvent *event)
{
    switch (event->type())
    {
    case QEvent::MouseButtonRelease:
        if(watched == ui->layoutDeleteLabel)
        {
            Q_EMIT deleteKeyboardLayoutItem();
        }
        if(watched == ui->selectLabel)
        {
            if(!mChecked || !KeyboardInstance::getIns()->onlyOneKeyboardLayoutSelected())
            {
                mChecked = !mChecked;
                ui->selectLabel->setVisible(mChecked);
                setChecked(mChecked);
                Q_EMIT selectedKeyboardLayoutItem();
            }
        }
        break;

    default:
        break;
    }
    return QFrame::eventFilter(watched, event);
}

bool KeyboardLayoutItem::event(QEvent *event)
{
    if(event->type() == QEvent::Enter)
    {
        if(mType == KEYBOART_TYPE::Delete && !mIsHideDeteleBtn)
        {
            ui->layoutDeleteLabel->show();
            if(mIsFisrt)
            {
                this->setStyleSheet("QFrame {border:none;border-top-left-radius:4px; border-top-right-radius:4px; background-color:#e4e5e6}"
                                    "QLabel {color: #222222;  text-align: left;}");
            }
            else
            {
                this->setStyleSheet("border:none; border-radius:0px; background-color:#e4e5e6}"
                                    "QLabel {color: #222222; text-align: left;}");
            }
        }
    }
    if(event->type() == QEvent::Leave)
    {
        if(mType == KEYBOART_TYPE::Delete)
        {
            ui->layoutDeleteLabel->hide();
            if(mIsFisrt)
            {
                this->setStyleSheet("QFrame {border:none; border-top-left-radius:4px; border-top-right-radius:4px; background-color:#f0f1f2}"
                                    "QLabel {color: #222222; text-align: left;}");
            }
            else
            {
                this->setStyleSheet("QFrame {border:none; border-radius:0px; background-color:#f0f1f2}"
                                    "QLabel {color: #222222; text-align: left;}");
            }
        }
    }
    return QFrame::event(event);
}

void KeyboardLayoutItem::paintEvent(QPaintEvent *event)
{
    QRect rect = this->rect();
    rect.setLeft(rect.left() + 14);
    if(mType == KEYBOART_TYPE::Addlayout)
    {
        QPainter painter(this);
        painter.setPen(QColor("#ddddde"));
        painter.drawLine(rect.topLeft(),rect.topRight());
    }
}

void KeyboardLayoutItem::mousePressEvent(QMouseEvent *event)
{
    if(event->button() == Qt::LeftButton)
    {
        if(mType == KEYBOART_TYPE::Addlayout)
        {
            Q_EMIT addKeyboardLayoutClicked();
        }
        if(mType == KEYBOART_TYPE::Select)
        {
            if(!mChecked || !KeyboardInstance::getIns()->onlyOneKeyboardLayoutSelected())
            {
                mChecked = !mChecked;
                ui->selectLabel->setVisible(mChecked);
                setChecked(mChecked);
                Q_EMIT selectedKeyboardLayoutItem();
            }
        }
    }
    QFrame::mouseMoveEvent(event);
}

void KeyboardLayoutItem::setHideDeleteBtn(bool hide)
{
    mIsHideDeteleBtn = hide;
}
