#include <QFile>
#include <QFileDialog>
#include <QStandardPaths>
#include <QTimer>
#include <QStyledItemDelegate>
#include "shortcutadd.h"
#include "ui_shortcutadd.h"

//定义底部按钮的的KEY
#define PAGE_BUTTOM_BTN_ADD (Page::PAGE_BBTN_USER+1)

ShortcutAdd::ShortcutAdd(QWidget *parent) :
  Page(parent),
  ui(new Ui::ShortcutAdd),
  interface_(nullptr),
  mShortcutTimer(new QTimer(this)),
  messageW_(nullptr),
  isNewShortcut_(false)
{
    ui->setupUi(this);
    QFile f(":/qss/shortcutadd.qss");
    if(f.open(QIODevice::ReadOnly))
    {
        QString style = f.readAll().replace("\n", "");
        this->setStyleSheet(style);
    }

    connect(ui->scanBtn, &QPushButton::pressed, this, &ShortcutAdd::onScanBtnClick);
    connect(ui->shortcutBtn, SIGNAL(clicked()), this, SLOT(onInputShortcutClicked()));
    connect(ui->shortcutName, &QLineEdit::textChanged, this, &ShortcutAdd::onCloseNameMessage);
    connect(ui->commandEdit, &QLineEdit::textChanged, this, &ShortcutAdd::onCloseCommandMessage);

    ui->shortcutBtn->setChecked(true);
    interface_ = new ShortcutInterface();

    mShortcutTimer->setInterval(1000);
    mShortcutTimer->setSingleShot(false);
    connect(mShortcutTimer, SIGNAL(timeout()), this, SLOT(shortcutTimer_timeout()));
    connect(interface_, SIGNAL(grabShortcutCancelled()), this, SLOT(grabShortcut_fail()));
    connect(interface_, SIGNAL(grabShortcutTimedout()), this, SLOT(grabShortcut_fail()));
    connect(interface_, SIGNAL(grabShortcutFailed()), this, SLOT(grabShortcut_fail()));
    connect(interface_, SIGNAL(shortcutGrabbed(QString)), this, SLOT(newShortcutGrabbed(QString)));

    messageW_ = new MessageWidget(this);
    messageW_->setDirection(MessageWidget::Direction::up);
    messageW_->setStartPos(20);
    messageW_->hide();
}

ShortcutAdd::~ShortcutAdd()
{
    delete ui;
    delete messageW_;
}

void ShortcutAdd::backBtnClicked()
{
    mShortcutTimer->stop();
    interface_->cancelShortcutGrab();
    ui->shortcutName->setText("");
    ui->commandEdit->setText("");
    ui->shortcutBtn->setText("");
}

QList<QPair<int, QString> > ShortcutAdd::buttomBtnsPair()
{
    QList<QPair<int, QString> > pairs;
    pairs << QPair<int, QString>(PAGE_BUTTOM_BTN_ADD, tr("Add"));
    return pairs;
}

QList<int> ShortcutAdd::buttomBtnsVisible()
{
    QList<int> page_bbtn_keys;
    page_bbtn_keys << PAGE_BUTTOM_BTN_ADD;
    return page_bbtn_keys;
}

void ShortcutAdd::buttomBtnClicked(const int key)
{
    if(key == PAGE_BUTTOM_BTN_ADD)
    {
        //
        this->addBtnClicked();
    }
}

void ShortcutAdd::refreshPage()
{
    //
}

void ShortcutAdd::onScanBtnClick()
{
    QString command;
    QFileDialog fd;

    QStringList directory = QStandardPaths::standardLocations(QStandardPaths::PicturesLocation);
    if (!directory.isEmpty()) {
        fd.setDirectory(directory.first());
    }

    if (fd.exec() == QFileDialog::Accepted) {
        command = fd.selectedFiles().first();
        ui->commandEdit->setText(command);
    }
}

static QStringList splitCommandLine(QString commandLine)
{
    commandLine.prepend(QLatin1Char(' ')).append(QLatin1Char(' '));
    QStringList result;
    QRegExp spacePattern(QStringLiteral("\\s+"));
    QRegExp itemPattern(QStringLiteral("([^ \r\n\t\"']+)|((\"([^\"]|\\\")*\")|('([^']|\\')*'))(?=\\s)"));

    for (int pos = 0; ;)
    {
        if (commandLine.indexOf(spacePattern, pos) != pos)
        {
            return QStringList();
        }
        pos += spacePattern.matchedLength();

        if (pos == commandLine.length())
        {
            break;
        }

        if (commandLine.indexOf(itemPattern, pos) != pos)
        {
            return QStringList();
        }
        pos += itemPattern.matchedLength();

        QString item = itemPattern.cap(2);
        if (item.length())
        {
            result << item.mid(1, item.length() - 2);
        }
        else
        {
            result << itemPattern.cap(1);
        }
    }
    return result;
}

void ShortcutAdd::shortcutTimer_timeout()
{
    if (mTimeoutCounter > 0)
    {
        --mTimeoutCounter;
        ui->shortcutBtn->setText(tr("Please enter a new shortcut") + "(" + QString::number(mTimeoutCounter) + ")");
    }
    else
    {
        ui->shortcutBtn->setText("");
        mShortcutTimer->stop();
    }
}

void ShortcutAdd::newShortcutGrabbed(const QString &newShortcut)
{
    ui->shortcutBtn->setChecked(false);
    mShortcutTimer->stop();
    ui->shortcutBtn->setText(newShortcut);
    if(filterShortcuts.contains(newShortcut))
    {
        messageW_->showWidget(ui->shortcutBtn->mapTo(this, QPoint(0, 0)).x() + 20,
                              ui->shortcutBtn->mapTo(this, QPoint(0, 0)).y() + ui->shortcutBtn->height() - 12,
                              tr("Shortcut key are not allowed:") + ui->shortcutBtn->text());
        return;
    }
    isNewShortcut_ = true;
}

void ShortcutAdd::onInputShortcutClicked(int timeout)
{
    isNewShortcut_ = false;
    interface_->cancelShortcutGrab();
    mTimeoutCounter = timeout;
    ui->shortcutBtn->setText(tr("Please enter a new shortcut") + "(" + QString::number(mTimeoutCounter) + ")");
    mShortcutTimer->start();
    interface_->grabShortcut(mTimeoutCounter * mShortcutTimer->interval());
    messageW_->close ();
}

void ShortcutAdd::addBtnClicked()
{
    QString sText = ui->shortcutBtn->text();
    if(sText.left(1) == "&")
    {
        sText = sText.right(sText.size()-1);
    }
    if(ui->shortcutName->text().isEmpty())
    {
        messageW_->showWidget(ui->shortcutName->mapTo(this, QPoint(0, 0)).x() + 20,
                              ui->shortcutName->mapTo(this, QPoint(0, 0)).y() + ui->shortcutName->height() - 12,
                              tr("Name cannot be empty"));
        return;
    }

    QStringList commandLine = splitCommandLine(ui->commandEdit->text());
    if (!commandLine.length())
    {
        messageW_->showWidget(ui->commandEdit->mapTo(this, QPoint(0, 0)).x() + 20,
                              ui->commandEdit->mapTo(this, QPoint(0, 0)).y() + ui->commandEdit->height() - 12,
                              tr("Command cannot be empty"));
        return;
    }

    QString filePath;
    if(!commandLine[0].contains("/",Qt::CaseSensitive))
    {
        filePath = QString("/usr/bin/") + commandLine[0];
    }
    else
    {
        filePath = commandLine[0];
    }

    QFileInfo file(filePath);
    if(!file.exists())
    {
        messageW_->showWidget(ui->commandEdit->mapTo(this, QPoint(0, 0)).x() + 20,
                              ui->commandEdit->mapTo(this, QPoint(0, 0)).y() + ui->commandEdit->height() - 12,
                              tr("Command not exist"));
        return;
    }

    if(!file.isExecutable() && commandLine.size() == 1)
    {
        commandLine.append(commandLine[0]);
        commandLine[0] = "xdg-open";
    }

    if(sText.isEmpty())
    {
        messageW_->showWidget(ui->shortcutBtn->mapTo(this, QPoint(0, 0)).x() + 20,
                              ui->shortcutBtn->mapTo(this, QPoint(0, 0)).y() + ui->shortcutBtn->height() - 12,
                              tr("Shortcut cannot be empty"));
        return;
    }

    shortcutMap_.clear();
    interface_->getShortcutList(shortcutMap_);
    if(!shortcutMap_.isEmpty())
    {
        QMap<qulonglong, GeneralActionInfo>::Iterator it=shortcutMap_.begin();
        while(it!=shortcutMap_.end())
        {
            if(it.value().description == ui->shortcutName->text()) {
                messageW_->showWidget(ui->shortcutName->mapTo(this, QPoint(0, 0)).x() + 20,
                                      ui->shortcutName->mapTo(this, QPoint(0, 0)).y() + ui->shortcutName->height() - 12,
                                      tr("Name already exists"));
                return;
            }

            if(it.value().shortcut == sText) {
                messageW_->showWidget(ui->shortcutBtn->mapTo(this, QPoint(0, 0)).x() + 20,
                                      ui->shortcutBtn->mapTo(this, QPoint(0, 0)).y() + ui->shortcutBtn->height() - 12,
                                      tr("Shortcut key already exists"));
                return;
            }
            it++;
        }
    }

    if(filterShortcuts.contains(sText))
    {
        messageW_->showWidget(ui->shortcutBtn->mapTo(this, QPoint(0, 0)).x() + 20,
                              ui->shortcutBtn->mapTo(this, QPoint(0, 0)).y() + ui->shortcutBtn->height() - 12,
                              tr("Shortcut key are not allowed:") + sText);
        return;
    }

    QPair<QString, qulonglong> result = interface_->addCommandAction(sText, commandLine[0], commandLine.mid(1), ui->shortcutName->text());
    if(result.first == nullptr)
    {
        return;
    }

    ui->shortcutName->setText("");
    ui->commandEdit->setText("");
    ui->shortcutBtn->setText("");

    Q_EMIT backPage();
}

void ShortcutAdd::mousePressEvent(QMouseEvent *)
{
    mShortcutTimer->stop();
    interface_->cancelShortcutGrab();
    if(!isNewShortcut_)
    {
        ui->shortcutBtn->setText("");
    }
    messageW_->close ();
}

void ShortcutAdd::resizeEvent(QResizeEvent* event)
{
    QWidget::resizeEvent(event);
    mShortcutTimer->stop();
    interface_->cancelShortcutGrab();
    if(!isNewShortcut_)
    {
        ui->shortcutBtn->setText("");
    }
    messageW_->close ();
}

void ShortcutAdd::onCloseNameMessage()
{
    messageW_->close ();
}

void ShortcutAdd::onCloseCommandMessage()
{
    messageW_->close ();
}
