#include <QToolButton>
#include <QTimer>
#include <QProcess>
#include <QStyledItemDelegate>
#include "shortcutitemframe.h"
#include <QEvent>

ShortcutItemFrame::ShortcutItemFrame(ShortcutInterface *interface, GeneralActionInfo info, qulonglong id, bool isEidt, QFrame *parent)
    :interface_(interface),
     hLayout_(nullptr),
     btnLayout_(nullptr),
     l_shortcut(nullptr),
//     enableBox_(nullptr),
     label_delete(nullptr),
     spacer_(nullptr),
     spacer2_(nullptr),
     intputShortcut_(nullptr),
     mShortcutTimer(new QTimer(this)),
     info_(info),
     mId(id),
     enableLayout_(nullptr),
//     toggleEnable_(nullptr),
     enableWidget_(nullptr)
{
    hLayout_ = new QHBoxLayout();
    hLayout_->setContentsMargins(14, 9, 14, 9); //设置框内的间距
    l_shortcut = new QLabel(this);
    l_shortcut->setContentsMargins(0,0,0,0);

    spacer_ = new QSpacerItem(20, 20, QSizePolicy::Expanding);
    QString shortcut_display = QString("%1").arg(info_.description);
    l_shortcut->setText(shortcut_display);
    l_shortcut->setStyleSheet("QLabel{border:none; color: #222222; background-color: rgba(255,255,255,0);}");
    hLayout_->addWidget(l_shortcut);
    hLayout_->addSpacerItem(spacer_);

    btnLayout_ = new QHBoxLayout();
    btnLayout_->setContentsMargins(0, 0, 0, 0);
    hLayout_->addLayout(btnLayout_);
    sList_ = info_.shortcut.split("+");
    if(!isEidt) {
        for(int i = 0; i < sList_.size(); ++i) {
            QToolButton * btn = new QToolButton();
            btn->setText(sList_.at(i));
            btn->setContentsMargins(0, 0, 0, 0);
            btn->setFixedHeight(30);
            btn->setStyleSheet(QLatin1String("QToolButton{border:none; color: #222222; border-radius:2px; background-color: #d8d8d9; }"
                                             "QToolButton:hover{border: 1px solid #2c9dfc; color: #222222; border-radius:2px; background-color: #d8d8d9; }"));
            connect(btn, SIGNAL(clicked()), this, SLOT(onButtonClicked()));
            btnLayout_->addWidget(btn);
        }
    }


    intputShortcut_ = new QToolButton();
    intputShortcut_->setMinimumWidth(150);
    intputShortcut_->setFixedHeight(30);
    intputShortcut_->setCheckable(true);
    intputShortcut_->hide();
    intputShortcut_->setContentsMargins(0, 0, 0, 0);
    intputShortcut_->setStyleSheet(QLatin1String("QToolButton { border:none; color: #999999; background-color: #ffffff; border-radius:2px;}"
                                                 "QToolButton:hover{border: 1px solid #2c9dfc; color: #999999; background-color: #ffffff; border-radius:2px;}"
                                                 "QToolButton:checked{border:none; color: #999999; background-color: #ffffff; border-radius:2px;}"));
    hLayout_->addWidget(intputShortcut_);
    connect(intputShortcut_, SIGNAL(clicked()), this, SLOT(onInputShortcutClicked()));



    // delete label
    label_delete = new QLabel(this);
    label_delete->setPixmap(QPixmap(":/images/delete.png"));
    label_delete->setContentsMargins(0, 0, 0, 0);
    label_delete->setStyleSheet(QLatin1String("QLabel {margin-right: 0px;}"));
    label_delete->installEventFilter(this);
    hLayout_->addWidget(label_delete);

    // enable
//    enableLayout_ = new QHBoxLayout();
//    enableLayout_->setContentsMargins(0,0,0,0);
//    enableLayout_->setSpacing(0);
//    enableWidget_ = new QWidget();
//    enableWidget_->setLayout(enableLayout_);
//    enableWidget_->setMaximumSize(QSize(46, 20));
//    toggleEnable_ = new NdeWidget::QMaterialToggle(enableWidget_);
//    toggleEnable_->setChecked(info_.enabled);
//    checked_ = info_.enabled;
//    connect(toggleEnable_, &NdeWidget::QMaterialToggle::toggled, this, &ShortcutItemFrame::onToggleEnableChanged);
//    enableLayout_->addWidget(toggleEnable_);
//    hLayout_->addWidget(enableWidget_);

    if(isEidt) {
//        toggleEnable_->show();
        if(info_.type == QLatin1String("command"))
        {
            label_delete->show();
        }
        else {
            label_delete->hide();
        }

    }
    else {
//        toggleEnable_->hide();
        label_delete->hide();
    }

    setLayout(hLayout_);
    setStyleSheet("QFrame{border:none; border-radius:4px; background: #f0f1f2}");
    setMinimumHeight(50);
    setMaximumHeight(50);

    mShortcutTimer->setInterval(1000);
    mShortcutTimer->setSingleShot(false);
    connect(mShortcutTimer, SIGNAL(timeout()), this, SLOT(shortcutTimer_timeout()));
}

ShortcutItemFrame::~ShortcutItemFrame()
{

}

void ShortcutItemFrame::onButtonClicked(int timeout) {
    if (!interface_)
    {
        return;
    }

    while(btnLayout_->count())
    {
        QLayoutItem * item = btnLayout_->takeAt(0);
        if(item->widget()) item->widget()->deleteLater();
        delete item;
    }
    connect(interface_, SIGNAL(grabShortcutCancelled()), this, SLOT(grabShortcut_fail()));
    connect(interface_, SIGNAL(grabShortcutTimedout()), this, SLOT(grabShortcut_fail()));
    connect(interface_, SIGNAL(grabShortcutFailed()), this, SLOT(grabShortcut_fail()));
    connect(interface_, SIGNAL(shortcutGrabbed(QString)), this, SLOT(newShortcutGrabbed(QString)));

    intputShortcut_->show();
    intputShortcut_->setChecked(true);
    interface_->cancelShortcutGrab();
    mTimeoutCounter = timeout;
    mOldShortcut = info_.shortcut;
    intputShortcut_->setText(tr("Please enter a new shortcut") + "(" + QString::number(mTimeoutCounter) + ")");
    mShortcutTimer->start();
    interface_->grabShortcut(mTimeoutCounter * mShortcutTimer->interval());

    emit shortcutShow(mId);
}

void ShortcutItemFrame::shortcutTimer_timeout()
{
    if (mTimeoutCounter > 0)
    {
        --mTimeoutCounter;
        intputShortcut_->setText(tr("Please enter a new shortcut") + "(" + QString::number(mTimeoutCounter) + ")");
    }
    else
    {
        intputShortcut_->hide();
        for(int i = 0; i < sList_.size(); ++i) {
            QToolButton * btn = new QToolButton();
            btn->setText(sList_.at(i));
            btn->setContentsMargins(0, 0, 0, 0);
            btn->setFixedHeight(30);
            btn->setStyleSheet(QLatin1String("QToolButton{border:none; color: #222222; border-radius:2px; background-color: #d8d8d9; }"
                                             "QToolButton:hover{border: 1px solid #2c9dfc; color: #222222; border-radius:2px; background-color: #d8d8d9; }"));
//            btn->setStyleSheet(QLatin1String("QToolButton{color: #222222; border-radius:2px; background-color: #d8d8d9; }"));

            connect(btn, SIGNAL(clicked()), this, SLOT(onButtonClicked()));
            btnLayout_->addWidget(btn);
        }
        mShortcutTimer->stop();
    }
}

void ShortcutItemFrame::grabShortcut_fail()
{
    if (intputShortcut_->isChecked())
    {
        emit shortcutGrabbed(mOldShortcut);
    }
}

void ShortcutItemFrame::newShortcutGrabbed(const QString &newShortcut)
{
    intputShortcut_->setChecked(false);
    mShortcutTimer->stop();
    intputShortcut_->setText(newShortcut);
    shortcutMap_.clear();
    interface_->getShortcutList(shortcutMap_);
    if(shortcutMap_.isEmpty()) {
        intputShortcut_->hide();
        for(int i = 0; i < sList_.size(); ++i) {
            QToolButton * btn = new QToolButton();
            btn->setText(sList_.at(i));
            btn->setContentsMargins(0, 0, 0, 0);
            btn->setFixedHeight(30);
            btn->setStyleSheet(QLatin1String("QToolButton{border:none; color: #222222; border-radius:2px; background-color: #d8d8d9; }"
                                             "QToolButton:hover{border: 1px solid #2c9dfc; color: #222222; border-radius:2px; background-color: #d8d8d9; }"));
            connect(btn, SIGNAL(clicked()), this, SLOT(onButtonClicked()));
            btnLayout_->addWidget(btn);
        }

        disconnect(interface_, SIGNAL(grabShortcutCancelled()), this, SLOT(grabShortcut_fail()));
        disconnect(interface_, SIGNAL(grabShortcutTimedout()), this, SLOT(grabShortcut_fail()));
        disconnect(interface_, SIGNAL(grabShortcutFailed()), this, SLOT(grabShortcut_fail()));
        disconnect(interface_, SIGNAL(shortcutGrabbed(QString)), this, SLOT(newShortcutGrabbed(QString)));
        return;
    }

    QMap<qulonglong, GeneralActionInfo>::Iterator it=shortcutMap_.begin();
    while(it!=shortcutMap_.end())
    {
        if(it.value().shortcut == newShortcut) {
            intputShortcut_->setText(tr("Shortcut key already exists"));
            return;
        }
        it++;
    }

    if(filterShortcuts.contains(newShortcut))
    {
        intputShortcut_->setText(tr("Shortcut key are not allowed:") + newShortcut);
        return;
    }

    if(!interface_->changeShortcut(mId, newShortcut).isEmpty() && interface_->enableAction(mId, true)) {
        intputShortcut_->hide();
        mOldShortcut = newShortcut;
        sList_.clear();
        sList_ = newShortcut.split("+");
        for(int i = 0; i < sList_.size(); ++i) {
            QToolButton * btn = new QToolButton();
            btn->setText(sList_.at(i));
            btn->setContentsMargins(0, 0, 0, 0);
            btn->setFixedHeight(30);
            btn->setStyleSheet(QLatin1String("QToolButton{border:none; color: #222222; border-radius:2px; background-color: #d8d8d9; }"
                                             "QToolButton:hover{border: 1px solid #2c9dfc; color: #222222; border-radius:2px; background-color: #d8d8d9; }"));
            connect(btn, SIGNAL(clicked()), this, SLOT(onButtonClicked()));
            btnLayout_->addWidget(btn);
        }

        disconnect(interface_, SIGNAL(grabShortcutCancelled()), this, SLOT(grabShortcut_fail()));
        disconnect(interface_, SIGNAL(grabShortcutTimedout()), this, SLOT(grabShortcut_fail()));
        disconnect(interface_, SIGNAL(grabShortcutFailed()), this, SLOT(grabShortcut_fail()));
        disconnect(interface_, SIGNAL(shortcutGrabbed(QString)), this, SLOT(newShortcutGrabbed(QString)));
    }

    emit shortcutGrabbed(newShortcut);
}

void ShortcutItemFrame::onInputShortcutClicked(int timeout)
{
    interface_->cancelShortcutGrab();
    mTimeoutCounter = timeout;
    mOldShortcut = info_.shortcut;
    intputShortcut_->setText(tr("Please enter a new shortcut") + "(" + QString::number(mTimeoutCounter) + ")");
    mShortcutTimer->start();
    interface_->grabShortcut(mTimeoutCounter * mShortcutTimer->interval());
}


bool ShortcutItemFrame::eventFilter(QObject *watched, QEvent *event)
{
    switch (event->type())
    {
    case QEvent::MouseButtonRelease:
        if(watched == label_delete)
        {
            onDeleteShortcut();
        }
        break;
    default:
        break;
    }
    return QWidget::eventFilter(watched, event);
}

void ShortcutItemFrame::hideShortcutInputBtn()
{
    if(!intputShortcut_->isHidden()) {
        intputShortcut_->setChecked(false);
        mShortcutTimer->stop();
        intputShortcut_->hide();
        for(int i = 0; i < sList_.size(); ++i) {
            QToolButton * btn = new QToolButton();
            btn->setText(sList_.at(i));
            btn->setContentsMargins(0, 0, 0, 0);
            btn->setFixedHeight(30);
            btn->setStyleSheet(QLatin1String("QToolButton{border:none; color: #222222; border-radius:2px; background-color: #d8d8d9; }"
                                             "QToolButton:hover{border: 1px solid #2c9dfc; color: #222222; border-radius:2px; background-color: #d8d8d9; }"));
            connect(btn, SIGNAL(clicked()), this, SLOT(onButtonClicked()));
            btnLayout_->addWidget(btn);
        }

        disconnect(interface_, SIGNAL(grabShortcutCancelled()), this, SLOT(grabShortcut_fail()));
        disconnect(interface_, SIGNAL(grabShortcutTimedout()), this, SLOT(grabShortcut_fail()));
        disconnect(interface_, SIGNAL(grabShortcutFailed()), this, SLOT(grabShortcut_fail()));
        disconnect(interface_, SIGNAL(shortcutGrabbed(QString)), this, SLOT(newShortcutGrabbed(QString)));
    }
}

void ShortcutItemFrame::editShortcutItem(bool checked)
{
    if(checked)
    {
        while(btnLayout_->count())
        {
            QLayoutItem * item = btnLayout_->takeAt(0);
            if(item->widget()) item->widget()->deleteLater();
            delete item;
        }
//        toggleEnable_->show();
        if(info_.type == QLatin1String("command"))
        {
            label_delete->show();
        }
        else {
            label_delete->hide();
        }
    }
    else
    {
//        toggleEnable_->hide();
        label_delete->hide();
        for(int i = 0; i < sList_.size(); ++i) {
            QToolButton * btn = new QToolButton();
            btn->setText(sList_.at(i));
            btn->setContentsMargins(0, 0, 0, 0);
            btn->setFixedHeight(30);
            btn->setStyleSheet(QLatin1String("QToolButton{border:none; color: #222222; border-radius:2px; background-color: #d8d8d9; }"
                                             "QToolButton:hover{border: 1px solid #2c9dfc; color: #222222; border-radius:2px; background-color: #d8d8d9; }"));
            connect(btn, SIGNAL(clicked()), this, SLOT(onButtonClicked()));
            btnLayout_->addWidget(btn);
        }
    }

}

void ShortcutItemFrame::onDeleteShortcut()
{
    interface_->removeAction(mId);
}

bool ShortcutItemFrame::setEnableShortcut()
{
    if(checked_ != info_.enabled)
    {
        interface_->enableAction(mId, checked_);
        info_.enabled = checked_;
        if(checked_) {
            return true;
        }
    }
    return false;
}

void ShortcutItemFrame::onToggleEnableChanged(bool checked)
{
    checked_ = checked;
}
