#include "languageandregion.h"
#include "ui_languageandregion.h"
#include "languageinstance.h"
#include <QFile>
#include <QDebug>

LanguageAndRegion::LanguageAndRegion(QWidget *parent) :
    Page(parent),
    ui(new Ui::LanguageAndRegion)
{
    ui->setupUi(this);

    QFile f(":/qss/languageandregion.qss");
    if(f.open(QIODevice::ReadOnly))
    {
        QString style = f.readAll().replace("\n", "");
        this->setStyleSheet(style);
    }

    GError *error = nullptr;
    m_improxy = fcitx_input_method_new(G_BUS_TYPE_SESSION,
                                     G_DBUS_PROXY_FLAGS_NONE,
                                     fcitx_utils_get_display_number(),
                                     nullptr,
                                     &error);
    g_signal_connect(m_improxy, "imlist-changed", G_CALLBACK(onFcitxImlistChanged), this);

    ui->label_add->installEventFilter(this);
    connect(ui->frame_language, &ClickedFrame::clicked, this, &LanguageAndRegion::onLanguageBtnClick);

    updateInputUI();
}

LanguageAndRegion::~LanguageAndRegion()
{
    delete ui;
    if(m_array)
    {
        g_ptr_array_set_free_func(m_array, (GDestroyNotify) fcitx_im_item_free);
        g_ptr_array_free(m_array, FALSE);
        m_array = nullptr;
    }
    if(m_improxy)
    {
        g_signal_handlers_disconnect_by_func(m_improxy, (gpointer)G_CALLBACK(onFcitxImlistChanged), this);
        g_object_unref(m_improxy);
        m_improxy = nullptr;
    }
}

void LanguageAndRegion::refreshPage()
{
    QLocale system = LanguageInstance::getIns()->getSystemLocale();
    QLocale currentlocale = LanguageInstance::getIns()->getCurLocale();
    QString itemstring = QString("%1 (%2)").arg(currentlocale.nativeLanguageName()).arg(currentlocale.nativeCountryName());
    ui->label_language_content->setText(itemstring);

    if((currentlocale == system))
    {
        ui->label_language_tips->hide();
    }
    else {
        ui->label_language_tips->show();
    }
}

void LanguageAndRegion::onLanguageBtnClick()
{
    Q_EMIT nextPage(Languagesetting::pageKey());
}

bool LanguageAndRegion::eventFilter(QObject *watched, QEvent *event)
{
    switch (event->type())
    {
    case QEvent::MouseButtonRelease:
        if(watched == ui->label_add)
        {
            onAddFcitxIM();
        }
        break;
    default:
        break;
    }
    return QWidget::eventFilter(watched, event);
}

void LanguageAndRegion::deleteOneItem(FcitxIMItem *item)
{
    item->enable = false;
    fcitx_input_method_set_imlist(m_improxy, m_array);
}

void LanguageAndRegion::onAddFcitxIM(void)
{
    Q_EMIT nextPage(InputMethodSetting::pageKey());
}

void LanguageAndRegion::updateInputUI()
{
    if(m_array != nullptr)
    {
        g_ptr_array_set_free_func(m_array, (GDestroyNotify)fcitx_im_item_free);
        g_ptr_array_free(m_array, FALSE);
        m_array = nullptr;
    }

    m_array = fcitx_input_method_get_imlist(m_improxy);
    if(nullptr == m_array)
    {
        return;
    }

    // add input method item to layout
    for(guint i = 0; i < m_array->len; i++)
    {
        FcitxIMItem* item = static_cast<FcitxIMItem*>(g_ptr_array_index(m_array,i));

        if(item->enable)
        {
            QMap<guint,InputMethodItem *>::iterator it = m_FcitxIMItemlist.find(i);
            if(it == m_FcitxIMItemlist.end())
            {
                InputMethodItem *inputItem = new InputMethodItem(item);
                ui->layout_list->addWidget(inputItem);
                m_FcitxIMItemlist.insert(i, inputItem);

                connect(inputItem, &InputMethodItem::deleteFcitxIMItem, this, &LanguageAndRegion::deleteOneItem);
            }
            else {
                InputMethodItem *inputItem = it.value();
                inputItem->updateFcitxIMItem(item);
            }
        }
        else
        {
            QMap<guint,InputMethodItem *>::iterator it = m_FcitxIMItemlist.find(i);
            if(it != m_FcitxIMItemlist.end())
            {
                InputMethodItem *inputItem = it.value();
                ui->layout_list->removeWidget(inputItem);
                inputItem->deleteLater();
                it = m_FcitxIMItemlist.erase(it);
            }
        }
    }

    //刷新界面显示
    for(int i = 0; i < ui->layout_list->count(); i++)
    {
        QLayoutItem * layoutItem = ui->layout_list->itemAt(i);
        InputMethodItem * inputItem = qobject_cast<InputMethodItem*>(layoutItem->widget());
        if(inputItem)
        {
            if(i == (ui->layout_list->count() - 1))
            {
                inputItem->updatePosition(1);
            }
            else if(i > 0 && i < (ui->layout_list->count() - 1))
            {
                inputItem->updatePosition(0);
            }
            else {
                inputItem->updatePosition(-1);
            }
        }
    }
}

void LanguageAndRegion::onFcitxImlistChanged(FcitxInputMethod *im, LanguageAndRegion* pThis)
{
    pThis->updateInputUI();
}
