#include "loglistitem.h"
#include "ui_loglistitem.h"
#include <QDateTime>
#include <QTimer>
#include <QDir>
#include <KAuth>
#include <QSettings>
#include <QtDebug>

LogListItem::LogListItem(const QString &name, LogListItem::LOG_LIST_TYPE logType, QWidget *parent ) :
    QFrame(parent),
    ui(new Ui::LogListItem),
    mType(LogListItem::VarLog),
    mListType(logType)
{
    ui->setupUi(this);
    QFile f(":/qss/loglistitem.qss");
    if(f.open(QIODevice::ReadOnly))
    {
        QString style = f.readAll().replace("\n", "");
        this->setStyleSheet(style);
    }

    if(logType == LogListItem::LogMainList) {
        ui->checkBox->setVisible(false);
        ui->label_next->setVisible(true);
    }
    else {
        ui->checkBox->setVisible(true);
        ui->label_next->setVisible(false);
    }
    ui->label->setText(name);
    if(logType == LogListItem::LogSettingList) {
        qint64 mStartSec = QSettings{}.value(name).toLongLong();
        ui->checkBox->setChecked( mStartSec > 0);
    }
    if(logType == LogListItem::LogExportList) {
        ui->checkBox->setChecked( true );
    }
}

LogListItem::~LogListItem()
{
    delete ui;
}
void LogListItem::setLogType(const LogListItem::LOG_TYPE type) {
    mType = type;
}
LogListItem::LogListItem::LOG_TYPE LogListItem::getLogType() const {
    return mType;
}
LogListItem::LogListItem::LOG_LIST_TYPE LogListItem::getListType() const {
    return mListType;
}

void LogListItem::on_checkBox_clicked()
{
    if(mListType != LogListItem::LogSettingList)
        return;
    if(ui->checkBox->isChecked()) {
        QSettings{}.setValue(ui->label->text(), QDateTime::currentMSecsSinceEpoch());
    }
    else {
        QSettings{}.setValue(ui->label->text(), -1);
    }
}
QString LogListItem::getText() const {
    return  ui->label->text();
}
bool LogListItem::itemChecked()
{
    return ui->checkBox->isChecked();
}

void LogListItem::mousePressEvent(QMouseEvent *event)
{
    QWidget::mousePressEvent(event);
}

void LogListItem::mouseReleaseEvent(QMouseEvent *event)
{
    openLog(ui->label->text(), mType);
    QWidget::mouseReleaseEvent(event);
}
