#include "systemlogdelete.h"
#include "ui_systemlogdelete.h"
#include <QDir>
#include <KAuth>
#include <QDebug>
#include <QSettings>
#include <QRegularExpression>

//定义底部按钮的的KEY
#define PAGE_BUTTOM_BTN_DELETE (Page::PAGE_BBTN_USER+1)

SystemLogDelete::SystemLogDelete(QWidget *parent) :
    Page(parent),
    ui(new Ui::SystemLogDelete)
{
    ui->setupUi(this);
    QFile f(":/qss/systemlogdelete.qss");
    if(f.open(QIODevice::ReadOnly))
    {
        QString style = f.readAll().replace("\n", "");
        this->setStyleSheet(style);
    }

    loadLogDir(LogListItem::LogDeleteList);
    connect(SystemLogInstance::getIns(), &SystemLogInstance::signalLogFilesChanged,this,[=](){
        clearLayoutList();
        loadLogDir(LogListItem::LogDeleteList);
    });
}

SystemLogDelete::~SystemLogDelete()
{
    delete ui;
}

void SystemLogDelete::backBtnClicked()
{
    //
}

QList<QPair<int, QString> > SystemLogDelete::buttomBtnsPair()
{
    QList<QPair<int, QString> > pairs;
    pairs.append(QPair<int, QString>(PAGE_BUTTOM_BTN_DELETE, tr("Delete Log")));
    return pairs;
}

QList<int> SystemLogDelete::buttomBtnsRed()
{
    QList<int> list;
    list<<PAGE_BUTTOM_BTN_DELETE;
    return list;
}

QList<int> SystemLogDelete::buttomBtnsVisible()
{
    QList<int> list;
    list<<PAGE_BUTTOM_BTN_DELETE;
    return list;
}

void SystemLogDelete::buttomBtnClicked(const int key)
{
    if(key == PAGE_BUTTOM_BTN_DELETE)
    {
        deleteLog();
    }
}

void SystemLogDelete::refreshPage()
{
    //
}

void SystemLogDelete::clearLayoutList()
{
    while(ui->layout_list->count())
    {
        QLayoutItem* item = ui->layout_list->takeAt(0);
        if(item->widget()) item->widget()->deleteLater();
        delete item;
    }
}

void SystemLogDelete::deleteLog()
{
    QStringList logFileList;
    for(int i = 0; i < ui->layout_list->count(); i++)
    {
        LogListItem* item = qobject_cast<LogListItem*>(ui->layout_list->itemAt(i)->widget());
        if(!item->itemChecked())
            continue;

        QDir logDir(LOG_PATH);
        if(!logDir.exists())
        {
            qWarning() << LOG_PATH << " not exist";
            return;
        }
        QStringList fileList;
        fileList << item->getText() + "*";
        QFileInfoList list = logDir.entryInfoList(fileList);
        for(const QFileInfo & info: list)
        {
            QString name = info.absoluteFilePath();

            if (logFileList.contains(name)){
                qWarning() << info.fileName() << "have contained";
                continue;
            }

            QFileInfo info_tmp(name);
            if(info_tmp.isDir())
            {
                QFileInfoList list_tmp = QDir(name).entryInfoList();
                if(name == "/var/log/audit")
                {
                    KAuth::Action readAction("org.nde.auth.helper.getfiles");
                    readAction.setHelperId("org.nde.auth.helper");
                    QVariantMap args;
                    args["dirName"] = info.absoluteFilePath();
                    readAction.setArguments(args);
                    readAction.setParentWidget(this);
                    KAuth::ExecuteJob *job = readAction.execute();
                    if(!job->exec()) {
                        qWarning() << "Kauth returned an error code :" << job->error();
                    } else {
                        QStringList contents = job->data()["Dirs"].toStringList();
                        for(const QString &file : contents) {
                            //                                ui->logFileCombox->addItem(file);
                            list_tmp.append(QFileInfo(file));
                        }
                    }
                }

                for(QFileInfo tmp : list_tmp)
                {
                    if(tmp.fileName() == "." || tmp.fileName() == "..")
                        continue;
                    if (logFileList.contains(tmp.absoluteFilePath())){
                        qWarning() << info.fileName() << "have contained";
                        continue;
                    }
                    logFileList.append(tmp.absoluteFilePath());
                }
            }
            else
                logFileList.append(name);
        }
    }
    if(!logFileList.isEmpty()){
        KAuth::Action readAction("org.nde.auth.helper.rmfiles");
        readAction.setHelperId("org.nde.auth.helper");
        QVariantMap args;
        args["fileList"] = logFileList;
        readAction.setArguments(args);
        readAction.setParentWidget(this);
        KAuth::ExecuteJob *job = readAction.execute();
        if(!job->exec()) {
            qWarning() << "Kauth returned an error code :" << job->error();
        }
    }
}

void SystemLogDelete::loadLogDir(LogListItem::LOG_LIST_TYPE listType)
{
    QDir logDir(LOG_PATH);
    if(!logDir.exists())
    {
        qWarning() << LOG_PATH << " not exist";
        return;
    }
    QFileInfoList list = logDir.entryInfoList();
    if(list.count() <= 0)
    {
        qWarning () << LOG_PATH << "is no log files";
        return;
    }
    QStringList logCategorys;
    QRegularExpression fileExp("-\\d+");
    QMap<QString, qint64> map;
    for(const QFileInfo & fileInfo : list)
    {
        QString baseName = fileInfo.baseName().remove(fileExp);

        if(baseName.isEmpty() || baseName == "README" || baseName == "sssd") // "sssd"该日志与域用户有关，删掉会导致域用户登录失败，因此不给予显示在日志列表
        {
            continue;
        }
        if(fileInfo.isDir())
        {
            if(isDirEmpty(baseName, fileInfo))
            {
                continue;
            }
        }

        if (map.contains(baseName))
        {
            map.find(baseName).value() += fileInfo.size();
        }else
        {
            map.insert(baseName, fileInfo.size());
        }
    }

    qint64 maxSize = 1024 * 1024 * 1024 ;
    for (QMap<QString, qint64>::ConstIterator it = map.constBegin(); it != map.constEnd(); ++it)
    {
        if (it.value() > maxSize)
            continue;
        else
        {
            LogListItem* listItem = new LogListItem (it.key(), listType, this);
            ui->layout_list->addWidget(listItem);
        }
    }
}

bool SystemLogDelete::isDirEmpty(QString baseName, QFileInfo fileInfo)
{
    QDir logDir(LOG_PATH);
    if(!logDir.exists()) {
        qWarning() << LOG_PATH << " not exist";
        return true;
    }

    QStringList fileList;
    fileList << baseName + "*";
    QFileInfoList childList = logDir.entryInfoList(fileList);
    if(childList.count() <= 0) {
        qWarning () << LOG_PATH << "is no log files";
        return true;
    }

    for(const QFileInfo & info: childList) {
        if(info.isDir()) {
            KAuth::Action readAction("org.nde.auth.helper.getfiles");
            readAction.setHelperId("org.nde.auth.helper");
            QVariantMap args;
            args["dirName"] = info.absoluteFilePath();
            readAction.setArguments(args);
            readAction.setParentWidget(this);
            KAuth::ExecuteJob *job = readAction.execute();
            if(!job->exec()) {
                qWarning() << "Kauth returned an error code :" << job->error();
            } else {
                QStringList contents = job->data()["Dirs"].toStringList();
                if(contents.count() <= 0)
                    return true;
            }
        }
    }
    return false;
}
