#ifndef SYSTEMLOGDETAIL_H
#define SYSTEMLOGDETAIL_H

#include <QDateTime>
#include <KJob>
#include "loglistitem.h"
#include "../baseui/page.h"

namespace Ui {
class SystemLogDetail;
}

class Local;

class SystemLogDetail : public Page
{
    Q_OBJECT
public:
    explicit SystemLogDetail(QWidget *parent = nullptr);
    ~SystemLogDetail();

public:
    //页面唯一键
    static QString pageKey() {return "systemlogdetail";}

    //页面标题
    virtual QString pageTitle(){return tr("Log Detail");}

    //Back按钮按下后的接口，主要用于back之前应该处理的一些操作
    virtual void backBtnClicked(){}

    //该页面底部的按钮QPair<key, text>
    virtual QList<QPair<int, QString> > buttomBtnsPair();
    //红色的底部按钮,底部按钮默认为蓝色
    virtual QList<int> buttomBtnsRed();
    //默认可见的底部按钮
    virtual QList<int> buttomBtnsVisible();
    //底部按钮按下事件响应的接口
    virtual void buttomBtnClicked(const int key);

    //每次进入该界面都会执行该函数，界面刷新的代码可以放在此函数
    virtual void refreshPage() ;

public:
    void openLogInfo(const QString & logName , LogListItem::LOG_TYPE type);

private slots:
    void logIndexChanged(int index);
    void updateJournalCmd(const QString &cmd);
    void on_startDateTimeEdit_dateTimeChanged(const QDateTime &dateTime);
    void on_endDateTimeEdit_dateTimeChanged(const QDateTime &dateTime);
    void on_startTimeCheckBox_clicked();
    void on_endTimeCheckBox_clicked();
    void on_levelCombox_currentIndexChanged(int index);
    void on_lineEdit_search_returnPressed();
    void on_findBackwardCheck_toggled(bool checked);
    void exportResult(KJob *kjob);

private:
    void deleteLog();
    void exportLog();
    void updateJournalLog();
    void exportLog(QStringList fileList);
    bool isDirEmpty(QString baseName, QFileInfo fileInfo);

    Ui::SystemLogDetail *ui;
    Local *mLocal;
    QString filterStr;
    int logLevel;
    bool startFlag;
    bool endFlag;
    int filterCount{0};
    QString lastExportFilePathName;
    QDateTime startDateTime_;
    QDateTime endDateTime_;
};
#endif // SYSTEMLOGDETAIL_H
