#include "systemlogexport.h"
#include "ui_systemlogexport.h"
#include <QDir>
#include <KAuth>
#include <QFileDialog>
#include <QMessageBox>
#include <QRegularExpression>
#include <QDebug>

//定义底部按钮的的KEY
#define PAGE_BUTTOM_BTN_CANCEL (Page::PAGE_BBTN_USER+1)
#define PAGE_BUTTOM_BTN_EXPORT (Page::PAGE_BBTN_USER+2)

SystemLogExport::SystemLogExport(QWidget *parent) :
    Page(parent),
    ui(new Ui::SystemLogExport)
{
    ui->setupUi(this);
    QFile f(":/qss/systemlogexport.qss");
    if(f.open(QIODevice::ReadOnly))
    {
        QString style = f.readAll().replace("\n", "");
        this->setStyleSheet(style);
    }

    loadLogDir(LogListItem::LogExportList);
    connect(SystemLogInstance::getIns(), &SystemLogInstance::signalLogFilesChanged,this,[=](){
        clearLayoutList();
        loadLogDir(LogListItem::LogExportList);
    });

    ui->frameExporting->hide();
}

void SystemLogExport::loadLogDir(LogListItem::LOG_LIST_TYPE listType) {
    QDir logDir(LOG_PATH);
    if(!logDir.exists()) {
        qWarning() << LOG_PATH << " not exist";
        return;
    }
    QFileInfoList list = logDir.entryInfoList();
    if(list.count() <= 0) {
        qWarning () << LOG_PATH << "is no log files";
        return;
    }
    QStringList logCategorys;
    QRegularExpression fileExp("-\\d+");
    QMap<QString, qint64> map;
    for(const QFileInfo & fileInfo : list) {
        QString baseName = fileInfo.baseName().remove(fileExp);

        if(baseName.isEmpty() || baseName == "README" || baseName == "sssd") // "sssd"该日志与域用户有关，删掉会导致域用户登录失败，因此不给予显示在日志列表
        {
            continue;
        }
        if(fileInfo.isDir())
        {
            if(isDirEmpty(baseName, fileInfo)) {
                continue;
            }
        }

        if (map.contains(baseName)){
            map.find(baseName).value() += fileInfo.size();
        }else{
            map.insert(baseName, fileInfo.size());
        }
    }

    qint64 maxSize = 1024 * 1024 * 1024 ;
    for (QMap<QString, qint64>::ConstIterator it = map.constBegin(); it != map.constEnd(); ++it){
        if (it.value() > maxSize)
            continue;
        else
        {
                LogListItem* listItem = new LogListItem (it.key(), listType, this);
                ui->layout_list->addWidget(listItem);
        }
    }
}
SystemLogExport::~SystemLogExport()
{
    delete ui;
}

void SystemLogExport::backBtnClicked()
{
    //
}

QList<QPair<int, QString> > SystemLogExport::buttomBtnsPair()
{
    QList<QPair<int, QString> > pairs;
    pairs.append(QPair<int, QString>(PAGE_BUTTOM_BTN_CANCEL, tr("Cancel Export")));
    pairs.append(QPair<int, QString>(PAGE_BUTTOM_BTN_EXPORT, tr("Export Log")));
    return pairs;
}

QList<int> SystemLogExport::buttomBtnsVisible()
{
    QList<int> list;
    list<<PAGE_BUTTOM_BTN_EXPORT;
    return list;
}

void SystemLogExport::buttomBtnClicked(const int key)
{
    if(key == PAGE_BUTTOM_BTN_EXPORT)
    {
        exportLog();
    }
    else if(key == PAGE_BUTTOM_BTN_CANCEL)
    {
        cancelExportLog();
    }
}

void SystemLogExport::refreshPage()
{
    //
}

void SystemLogExport::exportLog(QStringList fileList)
{
    if(fileList.at(0).contains("Journal"))
    {
        fileList.clear();
        fileList.append("/run/log/journal/");
    }

    QFileDialog fileDialog;
    QString fileName = fileDialog.getSaveFileName(this,tr("Export Log File"),"","");
    if(fileName.isEmpty())
    {
        return;
    }

    lastExportFilePathName = fileName + ".tar.gz";

    KAuth::Action readAction("org.nde.auth.helper.compresslogs");
    readAction.setHelperId("org.nde.auth.helper");
    QVariantMap args;
    args["fileList"] = fileList;
    args["compressFile"] = fileName;
    readAction.setArguments(args);
    readAction.setParentWidget(this);

    Q_EMIT buttomBtnVisibleChanged(PAGE_BUTTOM_BTN_EXPORT, false);
    Q_EMIT buttomBtnVisibleChanged(PAGE_BUTTOM_BTN_CANCEL, true);
    ui->frameExporting->show();

    KAuth::ExecuteJob *job = readAction.execute();
    connect(job, &KAuth::ExecuteJob::finished, this, &SystemLogExport::exportResult);
    job->start();
}

void SystemLogExport::exportResult(KJob *kjob)
{    
    Q_EMIT buttomBtnVisibleChanged(PAGE_BUTTOM_BTN_EXPORT, true);
    Q_EMIT buttomBtnVisibleChanged(PAGE_BUTTOM_BTN_CANCEL, false);

    ui->frameExporting->hide();

    auto job = qobject_cast<KAuth::ExecuteJob *>(kjob);

    if (job->error() != 0){
        QMessageBox msg(QMessageBox::Warning, tr("Error"), tr("Some errors occurred while compressing."), QMessageBox::Ok);
        msg.setButtonText(QMessageBox::Ok, tr("Ok"));
        msg.exec();

        if (lastExportFilePathName.isEmpty() || !QFile().exists(lastExportFilePathName))
            return ;
        QFileInfo fileInfo(lastExportFilePathName);
        QFile::remove(lastExportFilePathName);
    }
}

void SystemLogExport::clearLayoutList()
{
    while(ui->layout_list->count())
    {
        QLayoutItem* item = ui->layout_list->takeAt(0);
        if(item->widget()) item->widget()->deleteLater();
        delete item;
    }
}

void SystemLogExport::exportLog()
{
    QStringList logFileList;
    for(int i = 0; i < ui->layout_list->count(); i++)
    {
        LogListItem* item = qobject_cast<LogListItem*>(ui->layout_list->itemAt(i)->widget());
        if(!item->itemChecked())
            continue;

        QDir logDir(LOG_PATH);
        if(!logDir.exists())
        {
            qWarning() << LOG_PATH << " not exist";
            return;
        }
        QStringList fileList;
        fileList << item->getText() + "*";
        QFileInfoList list = logDir.entryInfoList(fileList);
        for(const QFileInfo & info: list)
        {
            QString name = info.fileName();
            if (logFileList.contains(name))
            {
                qWarning() << info.fileName() << "have contained";
                continue;
            }
            QFileInfo info_tmp(name);
            if(info_tmp.isDir())
            {
                QFileInfoList list_tmp = QDir(name).entryInfoList();
                if(name == "/var/log/audit")
                {
                    KAuth::Action readAction("org.nde.auth.helper.getfiles");
                    readAction.setHelperId("org.nde.auth.helper");
                    QVariantMap args;
                    args["dirName"] = info.absoluteFilePath();
                    readAction.setArguments(args);
                    readAction.setParentWidget(this);
                    KAuth::ExecuteJob *job = readAction.execute();
                    if(!job->exec())
                    {
                        qWarning() << "Kauth returned an error code :" << job->error();
                    } else {
                        QStringList contents = job->data()["Dirs"].toStringList();
                        for(const QString &file : contents)
                        {
                            //                                ui->logFileCombox->addItem(file);
                            list_tmp.append(QFileInfo(file));
                        }
                    }
                }

                for(QFileInfo tmp : list_tmp)
                {
                    if(tmp.fileName() == "." || tmp.fileName() == "..")
                        continue;
                    if (logFileList.contains(tmp.absoluteFilePath()))
                    {
                        qWarning() << info.fileName() << "have contained";
                        continue;
                    }
                    logFileList.append(tmp.absoluteFilePath());
                }
            }
            else
                logFileList.append(name);
        }
    }
    exportLog(logFileList);
}

void SystemLogExport::cancelExportLog()
{
    Q_EMIT buttomBtnVisibleChanged(PAGE_BUTTOM_BTN_EXPORT, true);
    Q_EMIT buttomBtnVisibleChanged(PAGE_BUTTOM_BTN_CANCEL, false);
    ui->frameExporting->hide();

    if (lastExportFilePathName.isEmpty() || !QFile().exists(lastExportFilePathName))
        return ;

    QFileInfo file(lastExportFilePathName);
    QFile::remove(lastExportFilePathName);
}

bool SystemLogExport::isDirEmpty(QString baseName, QFileInfo fileInfo)
{
    QDir logDir(LOG_PATH);
    if(!logDir.exists()) {
        qWarning() << LOG_PATH << " not exist";
        return true;
    }

    QStringList fileList;
    fileList << baseName + "*";
    QFileInfoList childList = logDir.entryInfoList(fileList);
    if(childList.count() <= 0) {
        qWarning () << LOG_PATH << "is no log files";
        return true;
    }

    for(const QFileInfo & info: childList) {
        if(info.isDir()) {
            KAuth::Action readAction("org.nde.auth.helper.getfiles");
            readAction.setHelperId("org.nde.auth.helper");
            QVariantMap args;
            args["dirName"] = info.absoluteFilePath();
            readAction.setArguments(args);
            readAction.setParentWidget(this);
            KAuth::ExecuteJob *job = readAction.execute();
            if(!job->exec()) {
                qWarning() << "Kauth returned an error code :" << job->error();
            } else {
                QStringList contents = job->data()["Dirs"].toStringList();
                if(contents.count() <= 0)
                    return true;
            }
        }
    }
    return false;
}
