#include "systemlogmanager.h"
#include "ui_systemlogmanager.h"
#include <QDir>
#include <KAuth>
#include <QDebug>
#include <QSettings>
#include <QRegularExpression>

#include "systemlogdetail.h"
#include "systemlogdelete.h"
#include "systemlogexport.h"
#include "systemlogsetting.h"

//定义底部按钮的的KEY
#define PAGE_BUTTOM_BTN_DELETE (Page::PAGE_BBTN_USER+1)
#define PAGE_BUTTOM_BTN_EXPORT (Page::PAGE_BBTN_USER+2)

SystemLogManager::SystemLogManager(QWidget *parent) :
    Page(parent),
    ui(new Ui::SystemLogManager)
{
    ui->setupUi(this);
    QFile f(":/qss/logmanager.qss");
    if(f.open(QIODevice::ReadOnly))
    {
        QString style = f.readAll().replace("\n", "");
        this->setStyleSheet(style);
    }

    ui->autoDeleteSwitch->setChecked(QSettings{}.value("autoDelete", false).toBool());
    ui->settingButton->setVisible(QSettings{}.value("autoDelete", false).toBool());
    connect(ui->autoDeleteSwitch, &NdeWidget::QMaterialToggle::toggled, this, [&](bool enabled) {
        QSettings{}.setValue("autoDelete", enabled);
        ui->settingButton->setVisible(enabled);
    });

    QTimer::singleShot(200, this, &SystemLogManager::doAutoClean);

    loadJournalLog();
    loadLogDir();
    connect(SystemLogInstance::getIns(), &SystemLogInstance::signalLogFilesChanged,this,[=](){
        clearLayoutList();
        loadJournalLog();
        loadLogDir();
    });
}

void SystemLogManager::loadJournalLog() {
    LogListItem* listItem = new LogListItem (tr("Systemd Journal Log"), LogListItem::LogMainList,  this);
    connect(listItem, &LogListItem::openLog, this, &SystemLogManager::openLogInfo);
    listItem->setLogType(LogListItem::JournalLog);
    ui->layout_list->addWidget(listItem);
}

void SystemLogManager::loadLogDir(LogListItem::LOG_LIST_TYPE listType) {
    QDir logDir(LOG_PATH);
    if(!logDir.exists()) {
        qWarning() << LOG_PATH << " not exist";
        return;
    }
    QFileInfoList list = logDir.entryInfoList();
    if(list.count() <= 0) {
        qWarning () << LOG_PATH << "is no log files";
        return;
    }
    QStringList logCategorys;
    QRegularExpression fileExp("-\\d+");
    QMap<QString, qint64> map;
    for(const QFileInfo & fileInfo : list) {
        QString baseName = fileInfo.baseName().remove(fileExp);

        if(baseName.isEmpty() || baseName == "README" || baseName == "sssd") // "sssd"该日志与域用户有关，删掉会导致域用户登录失败，因此不给予显示在日志列表
        {
            continue;
        }
        if(fileInfo.isDir())
        {
            if(isDirEmpty(baseName, fileInfo)) {
                continue;
            }
        }

        if (map.contains(baseName)){
            map.find(baseName).value() += fileInfo.size();
        }else{
            map.insert(baseName, fileInfo.size());
        }
    }

    qint64 maxSize = 1024 * 1024 * 1024 ;
    for (QMap<QString, qint64>::ConstIterator it = map.constBegin(); it != map.constEnd(); ++it){
        if (it.value() > maxSize)
            continue;
        else {
                LogListItem* listItem = new LogListItem (it.key(), listType, this);
                connect(listItem, &LogListItem::openLog, this, &SystemLogManager::openLogInfo);
                ui->layout_list->addWidget(listItem);
        }
    }
}

SystemLogManager::~SystemLogManager()
{
    delete ui;
}

QList<QPair<int, QString> > SystemLogManager::buttomBtnsPair()
{
    QList<QPair<int, QString> > pairs;
    pairs.append(QPair<int, QString>(PAGE_BUTTOM_BTN_DELETE, tr("Onekey Delete")));
    pairs.append(QPair<int, QString>(PAGE_BUTTOM_BTN_EXPORT, tr("Onekey Export")));
    return pairs;
}

QList<int> SystemLogManager::buttomBtnsVisible()
{
    QList<int> list;
    list<<PAGE_BUTTOM_BTN_DELETE;
    list<<PAGE_BUTTOM_BTN_EXPORT;
    return list;
}

void SystemLogManager::buttomBtnClicked(const int key)
{
    if(key == PAGE_BUTTOM_BTN_DELETE)
    {
        //
        Q_EMIT nextPage(SystemLogDelete::pageKey());
    }
    else if (key == PAGE_BUTTOM_BTN_EXPORT)
    {
        //
        Q_EMIT nextPage(SystemLogExport::pageKey());
    }
}

void SystemLogManager::refreshPage()
{
    //
}

void SystemLogManager::doAutoClean()
{
    QDir logDir(LOG_PATH);
    if(!logDir.exists()) {
        qWarning() << LOG_PATH << " not exist";
        return;
    }
    QFileInfoList list = logDir.entryInfoList();
    if(list.count() <= 0) {
        qWarning () << LOG_PATH << "is no log files";
        return;
    }
    QStringList logCategorys;
    QRegularExpression fileExp("-\\d+");
    for(const QFileInfo & fileInfo : list) {
        QString baseName = fileInfo.baseName().remove(fileExp);
        if(!baseName.isEmpty() && !logCategorys.contains(baseName)) {
            QSettings{}.value(baseName).toString();
            int mStartSec = QSettings{}.value(baseName).toInt();
            QDateTime time = QDateTime::fromMSecsSinceEpoch(mStartSec);
            int maxLogSize = QSettings{}.value("maxLogSize", 150).toInt() * 1024 * 1024;

            QDir logDir(LOG_PATH);
            if(!logDir.exists()) {
                qWarning() << LOG_PATH << " not exist";
                return;
            }
            QStringList fileList;
            fileList << baseName + "*";

            QFileInfoList list = logDir.entryInfoList(fileList);
            if(list.count() <= 0) {
                qWarning () << LOG_PATH << "is no log files";
                return;
            }
            qSort(list.begin(), list.end(), timeMoreThan);

            int size = 0;
            for(const QFileInfo & info: list) {
                if(info.isDir()) {
                    KAuth::Action readAction("org.nde.auth.helper.getfiles");
                    readAction.setHelperId("org.nde.auth.helper");
                    QVariantMap args;
                    args["dirName"] = info.absoluteFilePath();
                    readAction.setArguments(args);
                    readAction.setParentWidget(this);
                    KAuth::ExecuteJob *job = readAction.execute();
                    if(!job->exec()) {
                        qWarning() << "Kauth returned an error code :" << job->error();
                    }else {
                        QStringList contents = job->data()["Dirs"].toStringList();
                        for(const QString &file : contents) {
                            QFileInfo info(file);
                            size += info.size();
                            if(size > maxLogSize && QSettings{}.value("autoDelete", false).toBool() &&
                                    time.date().day() >= QSettings{}.value("autoDeleteInterval", 0).toInt()){

                                KAuth::Action readAction("org.nde.auth.helper.rmfiles");
                                readAction.setHelperId("org.nde.auth.helper");
                                QVariantMap args;
                                args["fileList"] = QStringList {} << file;
                                readAction.setArguments(args);
                                readAction.setParentWidget(this);
                                KAuth::ExecuteJob *job = readAction.execute();
                                if(!job->exec()) {
                                    qWarning() << "Kauth returned an error code :" << job->error();
                                }
                            }
                        }
                    }
                }
                else {
                    size += info.size();
                    if(size > maxLogSize && QSettings{}.value("autoDelete", false).toBool() &&
                            time.date().day() >= QSettings{}.value("autoDeleteInterval", 0).toInt()){

                        KAuth::Action readAction("org.nde.auth.helper.rmfiles");
                        readAction.setHelperId("org.nde.auth.helper");
                        QVariantMap args;
                        args["fileList"] = QStringList {} << info.absoluteFilePath();
                        readAction.setArguments(args);
                        readAction.setParentWidget(this);
                        KAuth::ExecuteJob *job = readAction.execute();
                        if(!job->exec()) {
                            qWarning() << "Kauth returned an error code :" << job->error();
                        }
                    }
                }
            }
            logCategorys.append(baseName);
        }
    }
}

void SystemLogManager::on_settingButton_clicked()
{
    Q_EMIT nextPage(SystemLogSetting::pageKey());
}

void SystemLogManager::clearLayoutList()
{
    while(ui->layout_list->count())
    {
        QLayoutItem* item = ui->layout_list->takeAt(0);
        if(item->widget()) item->widget()->deleteLater();
        delete item;
    }
}

bool SystemLogManager::isDirEmpty(QString baseName, QFileInfo fileInfo)
{
    QDir logDir(LOG_PATH);
    if(!logDir.exists()) {
        qWarning() << LOG_PATH << " not exist";
        return true;
    }

    QStringList fileList;
    fileList << baseName + "*";
    QFileInfoList childList = logDir.entryInfoList(fileList);
    if(childList.count() <= 0) {
        qWarning () << LOG_PATH << "is no log files";
        return true;
    }

    for(const QFileInfo & info: childList) {
        if(info.isDir()) {
            KAuth::Action readAction("org.nde.auth.helper.getfiles");
            readAction.setHelperId("org.nde.auth.helper");
            QVariantMap args;
            args["dirName"] = info.absoluteFilePath();
            readAction.setArguments(args);
            readAction.setParentWidget(this);
            KAuth::ExecuteJob *job = readAction.execute();
            if(!job->exec()) {
                qWarning() << "Kauth returned an error code :" << job->error();
            } else {
                QStringList contents = job->data()["Dirs"].toStringList();
                if(contents.count() <= 0)
                    return true;
            }
        }
    }
    return false;
}
