#ifndef MOUSE_H
#define MOUSE_H

#include <QWidget>
#include <Nde/Settings>
#include <designer/ndewidgetmaterialtoggle.h>
#include "../baseui/page.h"

namespace Ui {
class Mouse;
}

class TouchpadDevice;

class Mouse : public Page
{
    Q_OBJECT
public:
    explicit Mouse(QWidget *parent = nullptr);
    ~Mouse();

public:
    //页面唯一键
    static QString pageKey() {return "mouse";}

    //页面标题
    virtual QString pageTitle() {return tr("Mouse and Touchpad");}

    //该页面底部的按钮QPair<key, text>
    virtual QList<QPair<int, QString> > buttomBtnsPair(){
        return QList<QPair<int, QString> >();
    }

    //刷新界面显示接口
    virtual void refreshPage();

private Q_SLOTS:
    void onhandChanged(int value);
    void onMouseSpeedValueChanged(int value);
    void onSensitivityValueChanged(int value);
    void onScroolValueChanged(int value);
    void onTouchpadChanged(bool checked);
    void onMouseNaturalScrollingToggle(bool checked);
    void onNaturalScrollingToggle(bool checked);
    void onSingleClickToggle(bool checked);
    void onTow_FingerScrollingToggle(bool checked);
    void onEdgeScrollingToggle(bool checked);
    void onSpeedSliderValueChanged(int value);

private:
    void loadSettings();
    void initControls();
    void setLeftHandedMouse();
    void intTouchpad();
    void accept();
//    void scrollingChanged();

private:
    Ui::Mouse *ui;
    Nde::Settings* mSettings;
    Nde::Settings* mQtSettings;
    NdeWidget::QMaterialToggle * mMouseNaturalScrollingToggle;
    NdeWidget::QMaterialToggle * mTouchpadToggle;
    NdeWidget::QMaterialToggle * mNaturalScrollingToggle;
    NdeWidget::QMaterialToggle * mSingleClickToggle;
    NdeWidget::QMaterialToggle * mTow_FingerScrollingToggle;
    NdeWidget::QMaterialToggle * mEdgeScrollingToggle;
    TouchpadDevice * mTouchpadDevice;
    TouchpadDevice * mMouseDeviec;
    QList<TouchpadDevice> devices;
    int mMouseSpeed;
    int mMouseSensitivity;
    int mWheelScroll;
    bool mLeftHanded;
    bool isTouchpad;

};


#endif // MOUSE_H
