#include "picture.h"
#include "ui_picture.h"
#include <QProcess>
#include <QDir>
#include <QDebug>
#include <QFileDialog>
#include <QImageReader>
#include <kauth.h>
#include <qmessagebox.h>
#include <QSettings>
#include "wallpaperitem.h"

//定义底部按钮的的KEY
#define PAGE_BUTTOM_BTN_DELETE (Page::PAGE_BBTN_USER+1)
#define PAGE_BUTTOM_BTN_ADD (Page::PAGE_BBTN_USER+2)
#define PAGE_BUTTOM_BTN_SET (Page::PAGE_BBTN_USER+3)

#define WALLPAPER_PATH  "/usr/share/nde/wallpapers/"
#define WALLPAPER_THUMBNAIL_PATH  "/usr/share/nde/wallpapers/convert/"
#define SYSTEM_WALLPAPER_PATH "/usr/share/backgrounds/images/others/"
#define SYSTEM_THUMBNAIL_PATH "/usr/share/backgrounds/images/others/convert/"

Picture::Picture(QWidget *parent) :
    Page(parent),
    ui(new Ui::Picture)
{
    ui->setupUi(this);
    QFile f(":/qss/picture.qss");
    if(f.open(QIODevice::ReadOnly))
    {
        QString style = f.readAll().replace("\n", "");
        this->setStyleSheet(style);
    }
    ui->frame_selectpath->hide();
    ui->frame_interval->hide();
    InitUI();
    connect(list, SIGNAL(currentItemChanged(QListWidgetItem*,QListWidgetItem*)) ,this, SLOT(slot_itemClicked()));
    m_interface = new QDBusInterface(QStringLiteral("org.freedesktop.Notifications"),
                                     QStringLiteral("/org/freedesktop/Notifications"),
                                     QStringLiteral("org.freedesktop.Notifications"),
                                     QDBusConnection::sessionBus(),
                                     this);
}

Picture::~Picture()
{
    delete ui;
    delete list;
}

QList<QPair<int, QString> > Picture::buttomBtnsPair()
{
    QList<QPair<int, QString> > pairs;
    pairs.append(QPair<int, QString>(PAGE_BUTTOM_BTN_DELETE, tr("Delete Wallpaper")));
    pairs.append(QPair<int, QString>(PAGE_BUTTOM_BTN_ADD, tr("Upload Wallpaper")));
    pairs.append(QPair<int, QString>(PAGE_BUTTOM_BTN_SET, tr("Set Wallpaper")));
    return pairs;
}

QList<int> Picture::buttomBtnsRed()
{
    QList<int> list;
    list<<PAGE_BUTTOM_BTN_DELETE;
    return list;
}

QList<int> Picture::buttomBtnsVisible()
{
    QList<int> list;
    list<<PAGE_BUTTOM_BTN_ADD;
    list<<PAGE_BUTTOM_BTN_SET;
    //自动切换壁纸模式不显示按钮
    return m_wallpaperpattern.toInt() ? QList<int>() : list;
}

void Picture::buttomBtnClicked(const int key)
{
    if(key == PAGE_BUTTOM_BTN_DELETE)
    {
        //
        on_delButton_clicked();
    }
    else if(key == PAGE_BUTTOM_BTN_ADD)
    {
        //
        on_addButton_clicked();
    }
    else if(key == PAGE_BUTTOM_BTN_SET)
    {
        //
        on_setButton_clicked();
    }
}

void Picture::refreshPage()
{
    //
    setCurrentWallpaper();
}

void Picture::resizeEvent(QResizeEvent *event)
{
    resizeWallpaperList();
    Page::resizeEvent(event);
}

//获取系统当前设置壁纸
QString Picture::get_current_wallpaper()
{
    QString filePath = QString(qgetenv("HOME")) + "/.config/nde-fileman/nde/settings.conf";
    QSettings settings(filePath, QSettings::IniFormat);
    QString wallpaper = settings.value("Desktop/Wallpaper").toString();
    return  wallpaper;
}


//删除按钮显示隐藏事件
void Picture::slot_itemClicked()
{
    WallpaperItem * item = qobject_cast<WallpaperItem *>(list->itemWidget(list->currentItem()));
    if(item)
    {
        QString currentPath =  item->path();
        currentPath.remove("convert/");
        if( (isSystemDir(currentPath) || currentPath == get_current_wallpaper()))
        {
            Q_EMIT buttomBtnVisibleChanged(PAGE_BUTTOM_BTN_DELETE, false);
        }
        else
        {
            Q_EMIT buttomBtnVisibleChanged(PAGE_BUTTOM_BTN_DELETE, true);
        }
    }
    else {
        Q_EMIT buttomBtnVisibleChanged(PAGE_BUTTOM_BTN_DELETE, false);
    }

}


//判断是否是系统壁纸目录
bool Picture::isSystemDir(QString path)
{
    if(path != nullptr){
        if( path.contains( SYSTEM_WALLPAPER_PATH ))
        {
            return true;
        }
        return false;
    }
    return true;
}

//设置壁纸
void Picture::on_setButton_clicked()
{
    if(list->currentItem() != nullptr)
    {
        WallpaperItem * item = qobject_cast<WallpaperItem *>(list->itemWidget(list->currentItem()));
        if(item)
        {
            QString wallpaperPath = item->path();
            wallpaperPath.remove("convert/");
            QProcess p;
            p.execute( "bash",QStringList()<<"-c"<<QString("nde-fileman  -w  \'%1\'").arg(wallpaperPath));
            p.waitForFinished();
            setCurrentWallpaper();
            Q_EMIT buttomBtnVisibleChanged(PAGE_BUTTOM_BTN_DELETE, false);
        }
    }
}

//初始化ui
void Picture::InitUI()
{
    list= new WallpaerItemList(this);
    list->setItemSize(QSize(190,130));
    list->setImageSize(QSize(190,130));
    list->setSpacing(5);
    ui->layout_list->addWidget(list);

    QDir system_wallpaperDir = QDir(SYSTEM_THUMBNAIL_PATH);
    QStringList system_namelist =  system_wallpaperDir.entryList(QDir::Files | QDir::NoDot | QDir::NoDotAndDotDot | QDir::Readable);
    //读系统目录
    for(QString name : system_namelist)
    {
        QString path = system_wallpaperDir.absoluteFilePath(name);
        QString itemName = name.split(".").takeFirst();
        list->addImage(path,itemName);
    }

    QDir wallpaperDir = QDir(WALLPAPER_THUMBNAIL_PATH);
    QStringList namelist =  wallpaperDir.entryList(QDir::Files | QDir::NoDot | QDir::NoDotAndDotDot | QDir::Readable);
    //读用户上传目录
    for(QString name : namelist)
    {
        QString path = wallpaperDir.absoluteFilePath(name);
        QString itemName = name.split(".").takeFirst();
        list->addImage(path,itemName);
    }

    QString filePath = QString(qgetenv("HOME")) + "/.config/nde-fileman/nde/settings.conf";
    QSettings settings(filePath, QSettings::IniFormat);
    m_wallpaperpattern = settings.value("Desktop/WallpaperPattern").toString();
    initwallPaperGraphic(m_wallpaperpattern.toInt(), getwallPapersetPath());

    setCurrentWallpaper();
    resizeWallpaperList();
}

void Picture::resizeWallpaperList()
{
    QSize size = list->size();
    int space = list->spacing();
    QSize itemSize = list->itemSize();

    int col = (size.width()-space)/(itemSize.width()+space);
    col = col > 0 ? col:1;
    int row = list->count()/col + (list->count()%col ? 1:0);
    row = row > 0 ? row:1;

    list->setMinimumHeight((itemSize.height()+space)*row);
}

//添加壁纸列表项
void Picture::addItem(QString path)
{
    if(path != nullptr){
        int index = path.lastIndexOf("/");
        QString name = path.right(path.length()-index-1);
        // QString itemPath = WALLPAPER_PATH + name;
        QString itemPath = WALLPAPER_THUMBNAIL_PATH + name;
        //list->addImage(itemPath,name.split(".").takeFirst());
        QListWidgetItem * item = new QListWidgetItem();
        WallpaperItem * itemWgt = new WallpaperItem();
        itemWgt->setPath(itemPath);
        itemWgt->setText(name.split(".").takeFirst());
        itemWgt->setImageSize(list->imageSize());
        item->setSizeHint(list->itemSize());
        list->insertItem(0,item);
        list->setItemWidget(item,itemWgt);
        setCurrentWallpaper();
    }
    resizeWallpaperList();
}

//删除壁纸列表项
void Picture::deleteItem()
{
    QListWidgetItem *item = list->takeItem(list->currentRow());
    delete item;

    setCurrentWallpaper();
    resizeWallpaperList();
}


//上传壁纸
void Picture::on_addButton_clicked() {
    QFileDialog dlg;
    dlg.setAcceptMode(QFileDialog::AcceptOpen);
    dlg.setFileMode(QFileDialog::ExistingFile);
    // compose a name fileter from QImageReader
    QString filter;
    filter.reserve(256);
    filter = tr("Image Files");
    filter += " (";
    const QList<QByteArray> formats = QImageReader::supportedImageFormats();
    for(const QByteArray& format : formats) {
        filter += "*.";
        filter += format.toLower();
        filter += ' ';
    }
    filter += ')';
    dlg.setNameFilter(filter);
    dlg.setNameFilterDetailsVisible(false);
    if(dlg.exec() == QDialog::Accepted) {
        QString filepath;
        filepath = dlg.selectedFiles().constFirst();
        int index = filepath.lastIndexOf("/");
        QString name = filepath.right(filepath.length()-index-1);
        QString path = filepath.left(index+1);
        QString itemPath = WALLPAPER_PATH + name;
        bool isExisted = false;
        if(QFileInfo(itemPath).exists()){
            isExisted = true;
        }
        if(picturesChanged(filepath,"ADD")){
            if(isExisted)
            {
                filepath = path + newName_;
            }
            addItem(filepath);
        }
        else{
            QList<QVariant> args;
            args << tr("nde-systemsetting")                  //appname
                 << static_cast<unsigned int>(0)             //id
                 << "icon-l-settings"                        //icon
                 << tr("Picture add failed.")                   //summary
                 << tr("Picture add failed.")                         //body
                 << QStringList()                            //actions
                 << QVariantMap()                            //hints
                 << 5000;                                    //timeout
            m_interface->callWithArgumentList(QDBus::AutoDetect, "Notify", args);
        }

    }
}

//删除壁纸
void Picture::on_delButton_clicked()
{
    if(list->currentItem() != nullptr){
        WallpaperItem * item = qobject_cast<WallpaperItem *>(list->itemWidget(list->currentItem()));
        if(item)
        {
            QString wallpaperPath = item->path();
            if( picturesChanged(wallpaperPath,"DEL") )
            {
                deleteItem();
            }
        }
    }
}

//调用root执行权限
bool Picture::picturesChanged(QString path,QString type)
{
    KAuth::Action action("org.nde.auth.helper.pictureschanged");
    if(!action.isValid())
    {
        qInfo()<<"org.nde.auth.helper.pictureschanged is valid";
        exit(1);
    }
    action.setHelperId(QStringLiteral("org.nde.auth.helper"));
    action.addArgument(QStringLiteral("filepath"),path);
    action.addArgument(QStringLiteral("type"),type);
    KAuth::ExecuteJob *job = action.execute();
    if( job->exec()){
        if(type == "ADD"){
            newName_ = job->data()["names"].toString();
        }
        return true;
    }
    return false;
}

//壁纸模式修改
void Picture::on_comboBox_pattern_currentIndexChanged(int index)
{
    QString imageFilePath = getwallPapersetPath();
    QString filePath = QString(qgetenv("HOME")) + "/.config/nde-fileman/nde/settings.conf";
    QSettings settings(filePath, QSettings::IniFormat);
    QString wallpaperInterval = settings.value("Desktop/WallpaperInterval").toString();
    int interval;
    if(wallpaperInterval.toInt() != 0) {
        interval = wallpaperInterval.toInt();
    }else {
        interval = getwallPaperInterval(0);
    }

    QProcess p;
    if(index == 0){
        setwallPaperGraphic(index, imageFilePath);
        p.execute( "bash",QStringList()<<"-c"<<QString("nde-fileman -P \'%1\' -i %2").arg(imageFilePath).arg(0));
        setCurrentWallpaper();
    }else if (index == 1) {
        setwallPaperGraphic(index, imageFilePath);
        p.execute( "bash",QStringList()<<"-c"<<QString("nde-fileman -P \'%1\' -i %2").arg(imageFilePath).arg(interval));
    }
}

//选择切换壁纸路径
void Picture::on_selectPathButton_clicked()
{
    QWidget *qwidget = new QWidget();
    QString path;
    path = QFileDialog::getExistingDirectory(qwidget,tr("Choose Directory"),"/",QFileDialog::ShowDirsOnly);
    QDir dir(path);
    QStringList filter;
    //filter image files.
    filter << QString("*.jpg")<< QString("*.png")<< QString("*.jpeg") << QString("*.bmp") << QString("*.gif");
    QStringList files = dir.entryList(filter,QDir::Files|QDir::Readable,QDir::Name);
    QList<QFileInfo> *fileInfo = new QList<QFileInfo>(dir.entryInfoList(filter));
    int interval = getLastWallpaperInterval().toInt();
    QProcess p;
    if(!path.isEmpty() && !fileInfo->isEmpty()) {
        ui->lineEdit->setText(path);
        p.execute( "bash",QStringList()<<"-c"<<QString("nde-fileman -P \'%1\' -i %2").arg(path).arg(interval));
    }
    if(!path.isEmpty() && fileInfo->isEmpty()) {
        //message info
        QMessageBox::information(this,tr("Directory"),
                                 tr("There is no wallpaper file."),
                                 tr("OK"));
        p.execute( "bash",QStringList()<<"-c"<<QString("nde-fileman -i %1").arg(0));
    }
}

//修改壁纸切换间隔
void Picture::on_comboBox_interval_currentIndexChanged(int index)
{
    int interval;
    interval = getwallPaperInterval(index);
    //int interval = getwallPaperInterval(ui->comboBox_interval->currentText());
    QProcess p;
    p.execute( "bash",QStringList()<<"-c"<<QString("nde-fileman -i %1").arg(interval));
}

//壁纸图形界面切换
void Picture::setwallPaperGraphic(int index, QString path)
{
    if(index == 0){
        list->show();
        ui->frame_selectpath->hide();
        ui->frame_interval->hide();
        Q_EMIT buttomBtnVisibleChanged(PAGE_BUTTOM_BTN_ADD, true);
        Q_EMIT buttomBtnVisibleChanged(PAGE_BUTTOM_BTN_SET, true);
        slot_itemClicked();
    }else if (index == 1) {
        list->hide();
        ui->frame_selectpath->show();
        ui->frame_interval->show();
        ui->lineEdit->setText(path);
        int intervalIndex = getwallPaperIntervalIndex(getLastWallpaperInterval().toInt());
        ui->comboBox_interval->setCurrentIndex(intervalIndex);
        Q_EMIT buttomBtnVisibleChanged(PAGE_BUTTOM_BTN_ADD, false);
        Q_EMIT buttomBtnVisibleChanged(PAGE_BUTTOM_BTN_SET, false);
        Q_EMIT buttomBtnVisibleChanged(PAGE_BUTTOM_BTN_DELETE, false);
    }
}

//获取当前切换壁纸间隔时间
int Picture::getwallPaperInterval(int index)
{
    int interval;
    switch(index){
    case 0:
        interval = 1;
        break;
    case 1:
        interval = 10;
        break;
    case 2:
        interval = 30;
        break;
    case 3:
        interval = 60;
        break;
    case 4:
        interval = 360;
        break;
    case 5:
        interval = 1440;
        break;
    default:
        interval = 1;
        break;
    }
    return interval;
}

//获取当前间隔时间index
int Picture::getwallPaperIntervalIndex(int interval)
{
    int index;
    switch(interval){
    case 1:
        index = 0;
        break;
    case 10:
        index = 1;
        break;
    case 30:
        index = 2;
        break;
    case 60:
        index = 3;
        break;
    case 360:
        index = 4;
        break;
    case 1440:
        index = 5;
        break;
    default:
        index = 0;
        break;
    }
    return index;
}

//获取当前切换壁纸路径
QString Picture::getwallPapersetPath()
{
    QString filePath = QString(qgetenv("HOME")) + "/.config/nde-fileman/nde/settings.conf";
    QSettings settings(filePath, QSettings::IniFormat);
    QString path = settings.value("Desktop/UserWallpaperPath").toString();
    if(path.isEmpty()) {
        path = SYSTEM_WALLPAPER_PATH;
    }
    return path;
}

void Picture::setWallpaperCloseTimer() {

    QString wallpaperInterval = getLastWallpaperInterval();
    if(wallpaperInterval.toInt() != 0) {
        QProcess p;
        p.execute( "bash",QStringList()<<"-c"<<QString("nde-fileman -i %1").arg(0));
    }
}

//获取上一次壁纸切换间隔
QString Picture::getLastWallpaperInterval()
{
    QString filePath = QString(qgetenv("HOME")) + "/.config/nde-fileman/nde/settings.conf";
    QSettings settings(filePath, QSettings::IniFormat);
    QString wallpaperInterval = settings.value("Desktop/WallpaperInterval").toString();

    return wallpaperInterval;
}

//初始化壁纸界面
void Picture::initwallPaperGraphic(int index, QString path)
{
    if(index == 0){
        list->show();
        ui->frame_selectpath->hide();
        ui->frame_interval->hide();
        ui->comboBox_pattern->setCurrentIndex(0);
        Q_EMIT buttomBtnVisibleChanged(PAGE_BUTTOM_BTN_ADD, true);
        Q_EMIT buttomBtnVisibleChanged(PAGE_BUTTOM_BTN_SET, true);
        slot_itemClicked();
    }else if (index == 1) {
        list->hide();
        ui->frame_selectpath->show();
        ui->frame_interval->show();
        ui->lineEdit->setText(path);
        ui->comboBox_pattern->setCurrentIndex(1);
        int intervalIndex = getwallPaperIntervalIndex(getLastWallpaperInterval().toInt());
        ui->comboBox_interval->setCurrentIndex(intervalIndex);
        Q_EMIT buttomBtnVisibleChanged(PAGE_BUTTOM_BTN_ADD, false);
        Q_EMIT buttomBtnVisibleChanged(PAGE_BUTTOM_BTN_SET, false);
        Q_EMIT buttomBtnVisibleChanged(PAGE_BUTTOM_BTN_DELETE, false);
    }
}

QString Picture::getcurrentwallPaperPath(){
    QString filePath = QString(qgetenv("HOME")) + "/.config/nde-fileman/nde/settings.conf";
    QSettings settings(filePath, QSettings::IniFormat);
    QString path = settings.value("Desktop/Wallpaper").toString();

    return path;
}

void Picture::setCurrentWallpaper()
{
    QString currentpath = getcurrentwallPaperPath();

    for (int i = 0; i < list->count(); i++)
    {
        WallpaperItem * item = qobject_cast<WallpaperItem *>(list->itemWidget(list->item(i)));
        if(item)
        {
            QString itempath = item->path();
            itempath.remove("convert/");

            if(itempath == currentpath)
            {
                item->setCurrentWallpaper(true);
                list->setCurrentItem(list->item(i));
            }
            else
            {
                item->setCurrentWallpaper(false);
            }
        }
    }
}
