/* BEGIN_COMMON_COPYRIGHT_HEADER
 * (c)LGPL2+
 *
 * Nde - a lightweight, Qt based, desktop toolset
 * https://lxqt.org
 *
 * Copyright: 2011 Razor team
 * Authors:
 *   Christian Surlykke <christian@surlykke.dk>
 *
 * This program or library is free software; you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * END_COMMON_COPYRIGHT_HEADER */

#include <QList>
#include <QDebug>
#include <QMap>
#include <QDateTime>
#include <cmath>

#include "batteryhelper.h"
#include "powermanagementsettings.h"

QList<Solid::Battery*> BatteryHelper::batterys;

QString BatteryHelper::stateToString(Solid::Battery::ChargeState state)
{
    switch (state)
    {
    case Solid::Battery::NoCharge:
        return tr("Empty");
    case Solid::Battery::Discharging:
        return tr("Discharging");
    case Solid::Battery::FullyCharged:
        return tr("Fully charged");
    case Solid::Battery::Charging:
    default:
        return tr("Charging");
    }
}

Solid::Battery::ChargeState BatteryHelper::getBatteryState()
{
    Solid::Battery::ChargeState state = Solid::Battery::FullyCharged;
    for (Solid::Battery * bty : qAsConst(batterys))
    {
        if (bty->chargeState() == Solid::Battery::Charging){
            state = Solid::Battery::Charging;
            break;
        }
        if (bty->chargeState() == Solid::Battery::Discharging){
            state = Solid::Battery::Discharging;
            break;
        }
        if (bty->chargeState() == Solid::Battery::NoCharge){
            state = Solid::Battery::NoCharge;
            break;
        }

    }
    return state;
}

int BatteryHelper::getBatteryChargecent()
{
    int ChargePercent = 0;
    for (Solid::Battery * bty : qAsConst(batterys))
    {
        ChargePercent = ChargePercent + bty->chargePercent();
    }
    if (!batterys.empty()){
        ChargePercent = ChargePercent / batterys.size();
    }
    return ChargePercent;
}
