#include "screensaver.h"
#include "ui_screensaver.h"
#include <QProcess>
#include <QDir>
#include <QtDBus/QDBusInterface>
#include <QSettings>
#include <QDebug>

#define SYSTEM_WALLPAPER_PATH "/usr/share/backgrounds/images/others/"
#define SYSTEM_THUMBNAIL_PATH "/usr/share/backgrounds/images/others/convert/"

ScreenSaver::ScreenSaver(QWidget *parent) :
    Page(parent),
    ui(new Ui::ScreenSaver)
{
    ui->setupUi(this);
    QFile f(":/qss/screensaver.qss");
    if(f.open(QIODevice::ReadOnly))
    {
        QString style = f.readAll().replace("\n", "");
        this->setStyleSheet(style);
    }

    delegate = new QStyledItemDelegate;
    ui->comboBox->setItemDelegate(delegate);
    initUI();
    connect(list, SIGNAL(currentItemChanged(QListWidgetItem*,QListWidgetItem*)) ,this, SLOT(setScreenSaver()));
    connect(ui->comboBox,SIGNAL(currentIndexChanged(int)),this,SLOT(setScreensaverTime()));
}

void ScreenSaver::initUI()
{
    QHBoxLayout * layout = new QHBoxLayout();
    layout->setContentsMargins(0,0,0,0);
    layout->setSpacing(0);
    ui->widget->setLayout(layout);
    toggle_show = new NdeWidget::QMaterialToggle(ui->widget);
    connect(toggle_show, &NdeWidget::QMaterialToggle::toggled, this, &ScreenSaver::onToggleShowChecked);
    layout->addWidget(toggle_show);

    QHBoxLayout* layout_2 = new QHBoxLayout();
    layout_2->setContentsMargins(0,0,0,0);
    layout_2->setSpacing(0);
    ui->widget_2->setLayout(layout_2);
    toggle_lock = new NdeWidget::QMaterialToggle(ui->widget_2);
    connect(toggle_lock, &NdeWidget::QMaterialToggle::toggled, this, &ScreenSaver::onToggleLockChecked);
    layout_2->addWidget(toggle_lock);

    setToggleLockStatus();
    setToggleShowStatus();
    addItem_to_comboBox();
    getScreensaverTime();

     //加载壁纸图片
    list= new WallpaerItemList(this);
    list->setItemSize(QSize(190,130));
    list->setImageSize(QSize(190,130));
    list->setSpacing(5);
    ui->layout_list->addWidget(list);

    QDir system_wallpaperDir = QDir(SYSTEM_THUMBNAIL_PATH);
    QStringList system_namelist =  system_wallpaperDir.entryList(QDir::Files | QDir::NoDot | QDir::NoDotAndDotDot | QDir::Readable);
    for(QString name : system_namelist)
    {
        QString path = system_wallpaperDir.absoluteFilePath(name);
        QString itemName = name.split(".").takeFirst();
        list->addImage(path,itemName);
    }
    setCurrentScreensaver();
    resizeWallpaperList();
}


ScreenSaver::~ScreenSaver()
{
    delete ui;
    delete list;
    delete delegate;
}

void ScreenSaver::refreshPage()
{
    //
}

void ScreenSaver::resizeEvent(QResizeEvent *event)
{
    resizeWallpaperList();
    Page::resizeEvent(event);
}

void ScreenSaver::addItem_to_comboBox()
{
    QStringList timeList;
    timeList  <<tr("5 Minutes") <<tr("15 Minutes")<<tr("30 Minutes") <<tr("45 Minutes")<<tr("1 Hour")<<tr("2 Hours")<<tr("Never") ;
    ui->comboBox->clear();
    ui->comboBox->addItems(timeList);
}

void ScreenSaver::setToggleShowStatus()
{
    QSettings settings("nde","screensaver");
    QString status = settings.value("Status").toString();
    if( status == "true")
    {
        toggle_show->setChecked(true);
    }else {
        toggle_show->setChecked(false);
    }
}

void ScreenSaver::setToggleLockStatus()
{
    QSettings settings("nde","screensaver");
    QString status = settings.value("isLockscreen").toString();
    if( status == "false")
    {
          toggle_lock->setChecked(false);
    }else {
          toggle_lock->setChecked(true);
    }
}

void ScreenSaver::onToggleShowChecked()
{
    QProcess p;
    if(toggle_show->isChecked())
    {
        for (int i = 0; i < list->count(); i++) {
            WallpaperItem * itemWgt = ((WallpaperItem *)list->itemWidget(list->item(i)));
            itemWgt->setCurrentWallpaper(false);
            itemWgt->setSelected(false);
        }
        p.execute( "bash",QStringList()<<"-c"<<QString("nde-screensaver  -s  true"));
        p.waitForFinished();
    }
    else
    {
        setCurrentScreensaver();

        p.execute( "bash",QStringList()<<"-c"<<QString("nde-screensaver  -s  false"));
        p.waitForFinished();
    }
}

void ScreenSaver::onToggleLockChecked()
{
    QProcess p;
   if(toggle_lock->isChecked())
   {
       p.execute( "bash",QStringList()<<"-c"<<QString("nde-screensaver  -l  true"));
       p.waitForFinished();
   }
   else
   {
       p.execute( "bash",QStringList()<<"-c"<<QString("nde-screensaver  -l  false"));
       p.waitForFinished();
   }
}

void ScreenSaver::setScreenSaver()
{
    QString wallpaperPath = ((WallpaperItem *)list->itemWidget(list->currentItem()))->path();
    wallpaperPath.remove("convert/");
    QProcess p;
    p.execute( "bash",QStringList()<<"-c"<<QString("nde-screensaver -w  %1").arg(wallpaperPath));
    p.waitForFinished();
    setCurrentScreensaver();
    toggle_show->setChecked(false);
    onToggleShowChecked();
}

void ScreenSaver::setScreensaverTime()
{
    int time ;
    int index = ui->comboBox->currentIndex();
    switch (index) {
    case 0:
        time = 5*60;
        break;
    case 1:
        time = 15*60;
        break;
    case 2:
        time = 30*60;
        break;
    case 3:
        time = 45*60;
        break;
    case 4:
        time = 60*60;
        break;
    case 5:
        time = 120*60;
        break;
    case 6:
        time = 0;
        break;
    default:{
        qWarning() << "comboBox index parameter error";
        return;
    }
    }
    QDBusInterface*_interface_setIdletime = new QDBusInterface(QString::fromLatin1("org.nde.ScreenSaver"),
                                                 QString::fromLatin1("/ScreenSaver"),
                                                 QString::fromLatin1("org.nde.ScreenSaver"),
                                                 QDBusConnection::sessionBus());
    _interface_setIdletime->call("setIdleTime_t",time);
    delete _interface_setIdletime;
}

void ScreenSaver::resizeWallpaperList()
{
    QSize size = list->size();
    int space = list->spacing();
    QSize itemSize = list->itemSize();

    int col = (size.width()-space)/(itemSize.width()+space);
    col = col > 0 ? col:1;
    int row = list->count()/col + (list->count()%col ? 1:0);
    row = row > 0 ? row:1;

    qWarning() << "AAAAAAA" << row << col << list->count();

    list->setMinimumHeight((itemSize.height()+space)*row);
}

void ScreenSaver::getScreensaverTime()
{
    int time = 5*60;
    QSettings settings("nde","screensaver");
    QString s = settings.value("idleTime_t").toString();
    if(s != nullptr )
    {
          time = s.toInt();
    }
    switch (time) {
    case 5*60:
           ui->comboBox->setCurrentIndex(0);
           break;
     case 15*60:
            ui->comboBox->setCurrentIndex(1);
            break;
       case 30*60:
            ui->comboBox->setCurrentIndex(2);
            break;
       case 45*60:
            ui->comboBox->setCurrentIndex(3);
            break;
       case 60*60:
            ui->comboBox->setCurrentIndex(4);
            break;
       case 120*60:
            ui->comboBox->setCurrentIndex(5);
            break;
       case 0:
            ui->comboBox->setCurrentIndex(6);
            break;
    default:
            qWarning() << "time parameter is not in the selectable range";
            break;
    }
}

QString ScreenSaver::getcurrentScreensaverPath(){
    QSettings settings("nde","screensaver");
    QString path = settings.value("Screensaver").toString();

    return path;
}

void ScreenSaver::setCurrentScreensaver()
{
    QString currentpath = getcurrentScreensaverPath();

    for (int i = 0; i < list->count(); i++) {
        WallpaperItem * itemWgt = ((WallpaperItem *)list->itemWidget(list->item(i)));
        QString itempath = itemWgt->path();
        itempath.remove("convert/");

        if(itempath == currentpath && !toggle_show->isChecked()){
            itemWgt->setCurrentWallpaper(true);
            list->setCurrentItem(list->item(i));
        }
        else {
            itemWgt->setCurrentWallpaper(false);
        }
    }
}
