#include "listwidgetitem.h"
#include "ui_listwidgetitem.h"
#include "service_tool.h"
#include <QDebug>
#include <QResizeEvent>
#include <QTimer>

ListWidgetItem::ListWidgetItem(const QString &name, const QString description, const bool status, const bool active, QWidget *parent) :
    QFrame(parent),
    ui(new Ui::ListWidgetItem),
    mDesc(description)
{
    ui->setupUi(this);

    ui->lblServiceName->setText(name);
//    ui->lblServiceDescription->setText("- " + description);
    ui->checkServiceRunning->setChecked(active);
    ui->checkServiceStartup->setChecked(status);

    ui->lblServiceName->setToolTip(name);
    ui->lblServiceDescription->setToolTip(description);
    connect(ui->checkServiceRunning, SIGNAL(toggled(bool)), this, SLOT(onCheckServiceRunningClicked(bool)));
    connect(ui->checkServiceStartup, SIGNAL(toggled(bool)), this, SLOT(onCheckServiceStartupClicked(bool)));
}

ListWidgetItem::~ListWidgetItem()
{
    delete ui;
}
void ListWidgetItem::resizeEvent(QResizeEvent *event)
{
    ui->lblServiceDescription->clear();
    QTimer::singleShot(50, this, [&]{
        QFontMetrics fontMetrics(this->font());
        int nameSize = fontMetrics.width(ui->lblServiceName->text());
        int descWidth = ui->checkServiceStartup->geometry().x() - ui->lblServiceName->geometry().x() - nameSize - 50;
        qWarning() << ui->lblServiceName->text() << descWidth;
        ui->lblServiceDescription->setText(fontMetrics.elidedText("- " + mDesc, Qt::ElideRight, descWidth));
    });
}
void ListWidgetItem::onCheckServiceStartupClicked(bool checked)
{
    QString name = ui->lblServiceName->text();

    ServiceTool::changeServiceStatus(name, checked);

    ui->checkServiceStartup->setChecked(ServiceTool::serviceIsEnabled(name));
}

void ListWidgetItem::onCheckServiceRunningClicked(bool checked)
{
    QString name = ui->lblServiceName->text();

    ServiceTool::changeServiceActive(name, checked);

    ui->checkServiceRunning->setChecked(ServiceTool::serviceIsActive(name));
}
