#include "categorizedmodel.h"

#include <XdgIcon>
#include <QTimer>
#include <QDebug>
#include <QProcess>
#include <QFileInfo>
#include <Solid/Battery>
#include <Solid/Device>
#include <QDBusInterface>
#include <NetworkManagerQt/Manager>
#include <QFileSystemWatcher>
#define CONFIG "/etc/nde/nde-systemsettings.conf"
#define USER_ICON_DIR "/var/lib/AccountsService/icons/"

CategorizedModel::CategorizedModel(const QStringList &desktopDirs, QObject *parent) : QStandardItemModel(parent)
  ,m_bHasWlan(false)
{
    mPluginList = Nde::PluginInfo::search(desktopDirs, "NDeSettings/Plugin");
    QString settingsPaht = CONFIG;
    mSettings = new QSettings(settingsPaht,QSettings::IniFormat);

    // 判断是否有电池
    const auto devices = Solid::Device::listFromType(Solid::DeviceInterface::Battery, QString());
     for (const Solid::Device& device : devices)
         if (device.as<Solid::Battery>()->type() == Solid::Battery::PrimaryBattery)
             hasBattery = true;
    // 判断是否能盖屏
    QDBusInterface *UPowerInterface = new QDBusInterface(QL1S("org.freedesktop.UPower"),
                                                           QL1S("/org/freedesktop/UPower"),
                                                           QL1S("org.freedesktop.UPower"),
                                                           QDBusConnection::systemBus(), this);
     hasLid = UPowerInterface->property("LidIsPresent").toBool();

     QDBusInterface * _interface_activation = new QDBusInterface(QString::fromLatin1("org.nde.License"),
                                                QString::fromLatin1("/License"),
                                                QString::fromLatin1("org.nde.License"),
                                                QDBusConnection::systemBus());
     if(_interface_activation->isValid()){
         QDBusMessage response1 = _interface_activation->call("getOSVersion");
         if (response1.type() == QDBusMessage::ReplyMessage){
             OSVersion = response1.arguments().takeFirst().toString();
         }
     }
     if(_interface_activation)
         delete _interface_activation;

     //判断是否有无线网卡
     for (auto const & device : NetworkManager::networkInterfaces())
     {
         if(device->type() == NetworkManager::Device::Wifi)
         {
             m_bHasWlan = true;
             break;
         }
     }

    //更新头像
     QFileSystemWatcher * filePath = new QFileSystemWatcher;
     filePath->addPath(USER_ICON_DIR);
     connect(filePath, &QFileSystemWatcher::directoryChanged, this, &CategorizedModel::updateUserIcon);
}

QStandardItem *  CategorizedModel::SelectItem(QString category)
{
    CategorizedModelItem *categoryItem = nullptr;
    for (int i = 0; i < rowCount(); i++) {
        CategorizedModelItem * item = static_cast<CategorizedModelItem *>(this->item(i));
        for (int i = 0; i < item->rowCount(); i++) {

            CategorizedModelItem * itemp = static_cast<CategorizedModelItem *>(item->child(i));
            if(category == itemp->id())
            {
                categoryItem = itemp;
                break;
            }
        }

    }
    return categoryItem;
}

void CategorizedModel::initModel()
{
    for (Nde::PluginInfo pluginInfo : mPluginList) {
        CategorizedModelItem *categoryItem = nullptr;
        QString category = pluginInfo.value("Categories").toString();
        int PluIndex = pluginInfo.value("PluIndex","-1").toInt();
        int CategoryIndex = pluginInfo.value("CategoryIndex","-1").toInt();
        QString iconname = pluginInfo.value("Icon").toString();
        int PluHeight = pluginInfo.value("PluHeight","40").toInt();

        if(!pluginVisible(pluginInfo.id())) continue;

        for (int i = 0; i < rowCount(); i++) {
            CategorizedModelItem * item = static_cast<CategorizedModelItem *>(this->item(i));
            if(category == item->category())
            {
                categoryItem = item;
                break;
            }
        }

        if(!categoryItem)
        {
            categoryItem = new CategorizedModelItem(category,CategoryIndex);
            bool insert = false;
            for (int i = 0; i < rowCount(); i++) {
                CategorizedModelItem * item = static_cast<CategorizedModelItem *>(this->item(i));
                if(item  && item->index() > CategoryIndex){
                    insertRow(item->row(),categoryItem);
                    insert = true;
                    break;
                }
            }
            if(!insert){
                appendRow(categoryItem);
            }
        }
        QIcon icon;
        if(category == "UserInfo") {
            icon.addFile(getUserIconPath());
        }else
        {
            icon.addFile(":/icon/" + iconname + ".png");
        }
        icon.actualSize(QSize(24,24));

        CategorizedModelItem *item = new CategorizedModelItem (icon, pluginInfo.name(), pluginInfo.id(),PluIndex);

        if(category == "UserInfo") {
            item->setheight(PluHeight);
            m_userItem = item;
        }else
        {
            item->setheight(PluHeight);
        }

        bool insert = false;
        for (int i = 0; i < categoryItem->rowCount(); i++) {
            CategorizedModelItem * itemp = static_cast<CategorizedModelItem *>(categoryItem->child(i));
            if(itemp && itemp->index() > PluIndex){
                categoryItem->insertRow(itemp->row(),item);
                insert = true;
                break;
            }
        }
        if(!insert){
            categoryItem->appendRow(item);
        }

//        QTimer::singleShot(200, [this, pluginInfo, enable](){
//            emit childItemAdded(pluginInfo.id(), enable);
//        });
    }
}
QVariant CategorizedModel::data(const QModelIndex& index, int role/* = Qt::DisplayRole*/) const
{
    if(!index.isValid()) {
        return QVariant();
    }

    CategorizedModelItem* item = static_cast<CategorizedModelItem*>(QStandardItemModel::itemFromIndex(index));
    if(item){
        switch(role) {
            case RowHeight:
            {
                return item->height();
            }

        }
    }
    return QStandardItemModel::data(index, role);
}

bool CategorizedModel::pluginVisible(const QString pluginID)
{
    int visible = 1;
    for (Nde::PluginInfo pluginInfo : mPluginList)
    {
        if(pluginInfo.id() != pluginID)
        {
            continue;
        }
        visible = pluginInfo.value("Visible","1").toInt();
        visible = mSettings->value("left_visible/"+pluginInfo.id(),visible).toInt(); //设置左侧是否显示该模块
        if(pluginID == "power") // 如果没有电池或屏盖则不显示电源
        {
            if(!(visible && (hasBattery || hasLid)))
            {
                visible = 0;
            }
        }
        else if(pluginID == "wlan")
        {
            if(visible && m_bHasWlan)
                visible = 1;
            else
                visible = 0;
        }
        break;
    }

    return  visible ? true:false;
}

bool CategorizedModel::pluginEnable(const QString pluginID)
{
    int enable = 1;
    for (Nde::PluginInfo pluginInfo : mPluginList)
    {
        if(pluginInfo.id() != pluginID)
        {
            continue;
        }
        enable= pluginInfo.value("Enable","1").toInt();
        enable = mSettings->value("right_enable/"+pluginInfo.id(),enable).toInt();//设置右侧是否使能模块界面
    }
    return enable ? true:false;
}

QString CategorizedModel::getUserIconPath()
{
    QString currentUserName = QString(getenv("USER")).trimmed();// 当前登录用户名
    QString iconFile = USER_ICON_DIR + currentUserName; //用户头像路径
    QFileInfo fileInfo(iconFile);
    if(fileInfo.isFile()) {
        return iconFile;
    }
    return ":/icon/icon_user.png";
}

void CategorizedModel::updateUserIcon()
{
    QIcon icon;
    icon.addFile(getUserIconPath());
    m_userItem->setIcon(icon);
}
