#ifndef APPLICATIONFRAME_H
#define APPLICATIONFRAME_H

#include <QFrame>
#include <QHBoxLayout>
#include <QLabel>
#include <designer/ndewidgetmaterialtoggle.h>
#include <XdgDesktopFile>
#include <QDateTime>
#include <Nde/Settings>

QString getDFID(const XdgDesktopFile d);

struct FixedAppData
{
    XdgDesktopFile _desktopFile; //desktop文件
    QDateTime _fixedTime;

    FixedAppData()
    {
        _fixedTime = QDateTime::currentDateTime();
    }
    FixedAppData(const FixedAppData & f)
    {
        _desktopFile = f._desktopFile;
        _fixedTime = f._fixedTime;
    }
    void operator=(const FixedAppData & f)
    {
        _desktopFile = f._desktopFile;
        _fixedTime = f._fixedTime;
    }
    bool operator==(const FixedAppData & f)
    {
        if(f._desktopFile.isValid() && _desktopFile.isValid())
        {
            return getDFID(f._desktopFile) == getDFID(_desktopFile);
        }
        return false;
    }
};

struct SystemIconData
{
    QString name;
    QString command;
    QString icon;
};

class ApplicationFrame : public QFrame
{
    Q_OBJECT
public:
    explicit ApplicationFrame(const XdgDesktopFile &desktopFile, QFrame *parent = nullptr);
    explicit ApplicationFrame(const SystemIconData &systemApp, QFrame *parent = nullptr);
    explicit ApplicationFrame(const QString &plugin, const QString &text, QFrame *parent = nullptr);
    ~ApplicationFrame();
    void setCommonChecked();
    void setPluginChecked();

private Q_SLOTS:
    void onToggleEnableChanged(bool checked);

private:
    void initUI();
    void load(const XdgDesktopFile &desktopFile);
    void saveFixedSetting();
    void parseFixedSetting();
    void systemAppLoad(SystemIconData appAame);
    void pluginLoad(QString plugin, QString text);

private:
    QHBoxLayout * hLayout_;
    QLabel * iconLabel_;
    QLabel * textLabel_;
    QSpacerItem * spacer_;
    QHBoxLayout * enableLayout_;
    NdeWidget::QMaterialToggle * toggleEnabel_;
    QWidget *enableWidget_;
    XdgDesktopFile desktopFile_;
    QList<FixedAppData> fixedAppDatas_; //用户固定到常用的应用
    Nde::Settings * commonSetting_;  //常用应用配置
    Nde::Settings * systemIconSetting_;  //系统图标应用配置
    Nde::Settings * panelSetting_;  //任务栏应用配置
    bool isCommon_{false};
    bool isShowSystemIcon_{false};
    SystemIconData systemApp_;
    QString plugin_;
    bool isPlugin_{false};
    bool pluginChecked_{true};
};

#endif // APPLICATIONFRAME_H
