#include <QProcess>
#include <QFile>
#include <QtDBus/QDBusInterface>
#include <KScreen/ConfigMonitor>
#include <QStyledItemDelegate>
#include "taskbar.h"
#include "ui_taskbar.h"
#include "applicationframe.h"
#include "commonmenu.h"
#include "showsystemiconintray.h"
#include <QtDebug>

Taskbar::Taskbar(QWidget *parent) :
    Page(parent),
    ui(new Ui::Taskbar),
    panelHidable_(false),
    isAcceptPosionChange_(false),
    isAcceptTask_(false),
    isDualScreenMode_(false)
{
    ui->setupUi(this);
    QFile f(":/qss/taskbar.qss");
    if(f.open(QIODevice::ReadOnly))
    {
        QString style = f.readAll().replace("\n", "");
        this->setStyleSheet(style);
    }

    positionMap_.clear();
    positionMap_.insert("Bottom", 0);
    positionMap_.insert("Top", 1);
    positionMap_.insert("Left", 2);
    positionMap_.insert("Right", 3);

    buttonStyleMap_.clear();
    buttonStyleMap_.insert("Icon", 0);
    buttonStyleMap_.insert("IconText", 1);

    loadSettings();
    initUI();

    connect(KScreen::ConfigMonitor::instance(), &KScreen::ConfigMonitor::configurationChanged, this, &Taskbar::onConfigurationChanged);

    QDBusConnection connect = QDBusConnection::sessionBus();
    if(!connect.registerService("org.nde.systemsettings.Taskbar"))
    {
        qWarning() << "D-BUS register service failed:com.nde.Taskbar";
    }
    if(!connect.registerObject("/org/nde/Taskbar", this, QDBusConnection::ExportAllSlots))
    {
        qWarning() << "D-BUS register object failed:/org/nde/Taskbar";
    }
}

Taskbar::~Taskbar()
{
    delete ui;
}

void Taskbar::refreshPage()
{
    //
}

void Taskbar::initUI()
{
    // 开始菜单
    QHBoxLayout * displayCommonLayout = new QHBoxLayout();
    displayCommonLayout->setContentsMargins(0,0,0,0);
    displayCommonLayout->setSpacing(0);
    ui->startMenuOneWidget->setLayout(displayCommonLayout);
    toggleDisplayCommonlyApp_ = new NdeWidget::QMaterialToggle(ui->startMenuOneWidget);
    toggleDisplayCommonlyApp_->setChecked(menuCommonShow_);
    connect(toggleDisplayCommonlyApp_, &NdeWidget::QMaterialToggle::toggled, this, &Taskbar::onToggleDisplayCommonlyAppChanged);
    displayCommonLayout->addWidget(toggleDisplayCommonlyApp_);

    QHBoxLayout * displayFullScreenModeLayout = new QHBoxLayout();
    displayFullScreenModeLayout->setContentsMargins(0,0,0,0);
    displayFullScreenModeLayout->setSpacing(0);
    ui->startMenuTwoWidget->setLayout(displayFullScreenModeLayout);
    toggleDisplayFullScreenMode_ = new NdeWidget::QMaterialToggle(ui->startMenuTwoWidget);
    toggleDisplayFullScreenMode_->setChecked(menuDisplayMode_);
    connect(toggleDisplayFullScreenMode_, &NdeWidget::QMaterialToggle::toggled, this, &Taskbar::onToggleDisplayFullScreenModeChanged);
    displayFullScreenModeLayout->addWidget(toggleDisplayFullScreenMode_);

    ui->selectAppFrame->setTitle(tr("Select the application to add to the common menu"));
    ui->selectAppFrame->setEnabled(menuCommonShow_);
    connect(ui->selectAppFrame,SIGNAL(clicked()),this,SLOT(onSelectAppBtnClick()));

    // 任务栏-自动隐藏
    QHBoxLayout * hideDesktopModeLayout = new QHBoxLayout();
    hideDesktopModeLayout->setContentsMargins(0,0,0,0);
    hideDesktopModeLayout->setSpacing(0);
    ui->taskbarOneWidget->setLayout(hideDesktopModeLayout);
    toggleHideDesktopMode_ = new NdeWidget::QMaterialToggle(ui->taskbarOneWidget);
    toggleHideDesktopMode_->setChecked(panelHidable_);
    connect(toggleHideDesktopMode_, &NdeWidget::QMaterialToggle::toggled, this, &Taskbar::onToggleHideDesktopModeChanged);
    hideDesktopModeLayout->addWidget(toggleHideDesktopMode_);

    // 任务栏-任务栏位置
    QStringList taskbarPositionList;
    taskbarPositionList << tr("Bottom") << tr("Top") << tr("Left") << tr("Right");
    ui->taskbarPositionComboBox->addItems(taskbarPositionList);
    QMap<QString, int>::Iterator it = positionMap_.begin();
    while(it != positionMap_.end())
    {
        if(position_ == it.key()) {
            ui->taskbarPositionComboBox->setCurrentIndex(it.value());
            break;
        }
        ++it;
    }
    connect(ui->taskbarPositionComboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(onPositionChanged(int)));

    // 任务栏-任务栏大小
    QStringList taskbarSizeList;
    taskbarSizeList << tr("Small") << tr("Big");
    ui->taskbarSizeCcomboBox->addItems(taskbarSizeList);
    if(panelSize_ == 36)
    {
        ui->taskbarSizeCcomboBox->setCurrentIndex(0);
    }
    else if(panelSize_ == 42)
    {
        ui->taskbarSizeCcomboBox->setCurrentIndex(1);
    }
    connect(ui->taskbarSizeCcomboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(onTaskbarSizeModeChanged(int)));

    // 任务栏-任务栏按钮
    QStringList taskbarButtonList;
    taskbarButtonList << tr("Always merge") << tr("Never merge");
    ui->taskbarButtonComboBox->addItems(taskbarButtonList);
    if(buttonStyle_ == "Icon")
    {
        ui->taskbarButtonComboBox->setCurrentIndex(0);
    }
    else if(buttonStyle_ == "IconText"){
        ui->taskbarButtonComboBox->setCurrentIndex(1);
    }
    connect(ui->taskbarButtonComboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(onTaskbarButtonChanged(int)));

    // 任务栏-任务栏双屏扩展显示方式
    QStringList dualScreenList;
    dualScreenList << tr("Show on the main screen") << tr("Show on all screens");
    ui->dual_screenComboBox->addItems(dualScreenList);
    if(!isDualScreenMode_)
    {
        ui->dual_screenComboBox->setCurrentIndex(0);
    }
    else {
        ui->dual_screenComboBox->setCurrentIndex(1);
    }
    if(!isDualScreen())
    {
        ui->dual_screenComboBox->setEnabled(false);
        ui->label_3->setStyleSheet(QLatin1String("QLabel {color: #a8abb1;}"));
        ui->dual_screenComboBox->setStyleSheet(QLatin1String("ndeQComboBox {color: #a8abb1;}"));
    }
    connect(ui->dual_screenComboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(onDualScreenChanged(int)));

    // 小托盘
    ui->selectSystemIconFrame->setTitle(tr("Select the system icon displayed in the tray"));
    connect(ui->selectSystemIconFrame,SIGNAL(clicked()),this,SLOT(onSelectSystemIconBtnClick()));

    // 其他(小部件)
    // mainmenu,desktopswitch,quicklaunch,taskbar,statusnotifier,worldclock,notificationbar,showdesktop
//    pluginMap_.insert("mainmenu", tr("Show Start Menu"));
    pluginMap_.insert("desktopswitch", tr("Show Multi Views Bar"));
    pluginMap_.insert("quicklaunch", tr("Show Quick Launch Bar"));
//    pluginMap_.insert("taskbar", tr("Show Application Taskbar"));
//    pluginMap_.insert("statusnotifier", tr("Show Status Notifier Bar"));
    pluginMap_.insert("worldclock", tr("Show Calendar Bar"));
    pluginMap_.insert("notificationbar", tr("Show Notication Bar"));
//    pluginMap_.insert("showdesktop", tr("Show Desktop Button"));
    QString path = menuSetting_->fileName();
    QSettings panelSetting(path,QSettings::IniFormat);
    QStringList plugins;
    plugins << "desktopswitch" << "quicklaunch" << "notificationbar";
    QStringList pluginLists = menuSetting_->value("panel1/plugins", plugins).toStringList();
    for (const QString& plugin : pluginLists) {
        if(pluginMap_.find(plugin) != pluginMap_.end())
        {
            QString text = pluginMap_.find(plugin).value();
            if(!text.isEmpty()) {
                ApplicationFrame * item = new ApplicationFrame(plugin, text);
                ui->otherVerticalLayout->addWidget(item);
                mapApps_.insert(plugin, item);
            }
        }
    }
}

void Taskbar::onToggleDisplayCommonlyAppChanged(bool checked)
{
    QDBusInterface* interface = new QDBusInterface(QString::fromLatin1("com.nde.panel.mainmenu"),
                                                     QString::fromLatin1("/com/nde/panel/mainmenu"),
                                                     QString::fromLatin1("com.nde.panel.mainmenu"),
                                                     QDBusConnection::sessionBus());
    interface->call("setMenuCommonBtnShow", checked);
    delete interface;
    ui->selectAppFrame->setEnabled(checked);
    menuCommonShow_ = checked;
}

void Taskbar::onToggleDisplayFullScreenModeChanged(bool)
{
    if(menuDisplayMode_ == DisplayNormal) {
        this->setMenuDisplayMode(DisplayFull);
    }
    else {
        this->setMenuDisplayMode(DisplayNormal);
    }
}

void Taskbar::setMenuDisplayMode(MenuDisplayMode mode)
{
    menuDisplayMode_ = mode;
    menuSetting_->beginGroup("mainmenu");
    menuSetting_->setValue("MenuDisplayMode", mode);
    menuSetting_->endGroup();
}

void Taskbar::toggleDisplayFullScreenModeChanged(int mode)
{
    menuDisplayMode_ = (MenuDisplayMode)mode;
    toggleDisplayFullScreenMode_->setChecked(mode);
}

void Taskbar::onSelectAppBtnClick()
{
    Q_EMIT nextPage(CommonMenu::pageKey());
}

void Taskbar::onToggleHideDesktopModeChanged(bool checked)
{
    QDBusInterface* interface = new QDBusInterface(QString::fromLatin1("com.nde.panel.ndepanel"),
                                                 QString::fromLatin1("/com/nde/panel/ndepanel"),
                                                 QString::fromLatin1("com.nde.panel.ndepanel"),
                                                 QDBusConnection::sessionBus());
    interface->call("onAutoHideSetting", checked);
    delete interface;
}

void Taskbar::onToggleHideDesktopMode(bool checked)
{
    toggleHideDesktopMode_->setChecked(checked);
}

void Taskbar::onPositionChanged(int index)
{
    if(!isAcceptPosionChange_) {
        QMap<QString, int>::Iterator it = positionMap_.begin();
        while(it != positionMap_.end())
        {
            if(index == it.value())
            {
                position_ = it.key();
                break;
            }
            ++it;
        }

        QDBusInterface* interface = new QDBusInterface(QString::fromLatin1("com.nde.panel.ndepanel"),
                                                     QString::fromLatin1("/com/nde/panel/ndepanel"),
                                                     QString::fromLatin1("com.nde.panel.ndepanel"),
                                                     QDBusConnection::sessionBus());
        interface->call("onPositionSetting", index);
        delete interface;
    }
    else
    {
        isAcceptPosionChange_ =false;
    }
}

void Taskbar::onPositionPanel(int index)
{
    isAcceptPosionChange_ = true;
    ui->taskbarPositionComboBox->setCurrentIndex(index);
}

void Taskbar::onTaskbarSizeModeChanged(int index)
{
    switch ((PanelSizeMode)index) {
        case SmallMode:
        {
            QDBusInterface* interface = new QDBusInterface(QString::fromLatin1("com.nde.panel.ndepanel"),
                                                         QString::fromLatin1("/com/nde/panel/ndepanel"),
                                                         QString::fromLatin1("com.nde.panel.ndepanel"),
                                                         QDBusConnection::sessionBus());
            interface->call("setTaskbarSizeBySystemsetting", "Small");
            delete interface;
            break;
        }
        case BigMode:
        {
            QDBusInterface* interface = new QDBusInterface(QString::fromLatin1("com.nde.panel.ndepanel"),
                                                         QString::fromLatin1("/com/nde/panel/ndepanel"),
                                                         QString::fromLatin1("com.nde.panel.ndepanel"),
                                                         QDBusConnection::sessionBus());
            interface->call("setTaskbarSizeBySystemsetting", "Big");
            delete interface;
            break;
        }
    }
}

void Taskbar::onTaskbarButtonChanged(int index)
{
    if(!isAcceptTask_)
    {
        QMap<QString, int>::Iterator it = buttonStyleMap_.begin();
        while(it != buttonStyleMap_.end())
        {
            if(index == it.value())
            {
                buttonStyle_ = it.key();
                break;
            }
            ++it;
        }

        QDBusInterface* interface = new QDBusInterface(QString::fromLatin1("com.nde.panel.ndepanel"),
                                                     QString::fromLatin1("/com/nde/panel/ndepanel"),
                                                     QString::fromLatin1("com.nde.panel.ndepanel"),
                                                     QDBusConnection::sessionBus());
        interface->call("onpluginTaskBar", index);
        delete interface;
    }
    else {
        isAcceptTask_ = false;
    }
}

void Taskbar::onDualScreenChanged(int index)
{
    if(dualScreenClick_){
        return ;
    }
    dualScreenClick_ = true;
    switch (index) {
        case MainScreen:
        {
            setMainScreenShow();
            break;
        }
        case AllScreen:
        {
            setAllScreenShow();
            break;
        }
    }
    QTimer r;
    r.singleShot(100,[this](){
        this->dualScreenClick_=false;
    });
}

void Taskbar::onTaskbarButton(QString name)
{
    isAcceptTask_ = true;
    if(name  == "Icon")
    {
        ui->taskbarButtonComboBox->setCurrentIndex(0);
    }
    else if(name == "IconText")
    {
        ui->taskbarButtonComboBox->setCurrentIndex(1);
    }
}

void Taskbar::onSelectSystemIconBtnClick()
{
    Q_EMIT nextPage(ShowSystemIconInTray::pageKey());
}

void Taskbar::onPluginModeChange(QString pluginName)
{
    mapApps_.find(pluginName).value()->setPluginChecked();
}

void Taskbar::loadSettings()
{
    menuSetting_ = new Nde::Settings(QLatin1String("panel"), this);
    menuSetting_->beginGroup("mainmenu");
    menuDisplayMode_ = (MenuDisplayMode) (menuSetting_->value("MenuDisplayMode", 0).toInt());
    menuCommonShow_ = menuSetting_->value("MenuCommonShow", true).toBool();
    menuSetting_->endGroup();

    menuSetting_->beginGroup("panel1");
    panelHidable_ = menuSetting_->value("hidable", false).toBool();
    position_ = menuSetting_->value("position", "Bottom").toString();
    panelSize_ = menuSetting_->value("panelSize", 36).toInt();
    menuSetting_->endGroup();

    menuSetting_->beginGroup("taskbar");
    buttonStyle_ = menuSetting_->value("buttonStyle", "Icon").toString();
    menuSetting_->endGroup();

    QStringList panels = menuSetting_->value("panels", "panel1").toStringList();
    if(panels.count() > 1)
    {
        isDualScreenMode_ = true;
    }
    else
    {
        isDualScreenMode_ = false;
    }
}

bool Taskbar::isDualScreen()
{
    const auto allScreens = QGuiApplication::screens();
    if(allScreens.count() > 1)
    {
        return true;
    }
    return false;
}

void Taskbar::setMainScreenShow()
{
    QDBusInterface* interface = new QDBusInterface(QString::fromLatin1("com.nde.panel.ndepanelapplication"),
                                                 QString::fromLatin1("/com/nde/panel/ndepanelapplication"),
                                                 QString::fromLatin1("com.nde.panel.ndepanelapplication"),
                                                 QDBusConnection::sessionBus());
    interface->call("removePanelFromSystemsettings", QString("panel2"));
    delete interface;
}

void Taskbar::setAllScreenShow()
{
    QDBusInterface* interface = new QDBusInterface(QString::fromLatin1("com.nde.panel.ndepanelapplication"),
                                                 QString::fromLatin1("/com/nde/panel/ndepanelapplication"),
                                                 QString::fromLatin1("com.nde.panel.ndepanelapplication"),
                                                 QDBusConnection::sessionBus());
    interface->call("addPanelFromSystemsettings", QString("panel2"));
    delete interface;


}

void Taskbar::onConfigurationChanged()
{
    if(isDualScreen())
    {
        onScreenAdded();
    }
    else
    {
        onScreenRemoved();
    }
}

void Taskbar::onScreenAdded()
{
    QFile f(":/qss/ndeqcombobox.qss");
    if(f.open(QIODevice::ReadOnly))
    {
        QString style = f.readAll().replace("\n", "");
        ui->dual_screenComboBox->setStyleSheet(style);
    }
    QStyledItemDelegate *delegate = new QStyledItemDelegate;
    ui->dual_screenComboBox->setItemDelegate(delegate);
    ui->dual_screenComboBox->setEnabled(true);
    ui->label_3->setStyleSheet(QLatin1String("QLabel {color: #222222;}"));
}

void Taskbar::onScreenRemoved()
{
    ui->dual_screenComboBox->setEnabled(false);
    ui->dual_screenComboBox->setStyleSheet(QLatin1String("ndeQComboBox {color: #a8abb1;}"));
    if(ui->dual_screenComboBox->currentIndex() == 1) {
        ui->dual_screenComboBox->setCurrentIndex(0);
    }
    ui->label_3->setStyleSheet(QLatin1String("QLabel {color: #a8abb1;}"));
    setMainScreenShow();
}
