#ifndef TASKBAR_H
#define TASKBAR_H

#include <QWidget>
#include <designer/ndewidgetmaterialtoggle.h>
#include <Nde/Settings>
#include "../baseui/page.h"

namespace Ui {
class Taskbar;
}

class ApplicationFrame;

enum MenuDisplayMode {
    DisplayNormal = 0, //普通窗口模式
    DisplayFull = 1,   //全屏模式
};

enum PanelSizeMode {
    SmallMode,
    BigMode
};

enum DualScreenMode {
    MainScreen,
    AllScreen
};

class Taskbar : public Page
{
    Q_OBJECT
    Q_CLASSINFO("D-Bus Interface", "org.nde.systemsettings.Taskbar")

public:
    explicit Taskbar(QWidget *parent = nullptr);
    ~Taskbar();

public:
    //页面唯一键
    static QString pageKey() {return "taskbar";}

    //页面标题
    virtual QString pageTitle() {return tr("Taskbar");}

    //该页面底部的按钮QPair<key, text>
    virtual QList<QPair<int, QString> > buttomBtnsPair(){
        return QList<QPair<int, QString> >();
    }
    virtual QList<int> buttomBtnsVisible(){ return QList<int>(); }

    //刷新界面显示接口
    virtual void refreshPage();

public Q_SLOTS:
    //dbus 接口
    void toggleDisplayFullScreenModeChanged(int mode);
    void onToggleHideDesktopMode(bool checked);
    void onPositionPanel(int index);
    void onTaskbarButton(QString name);
    void onPluginModeChange(QString pluginName);

private Q_SLOTS:
    void onToggleDisplayCommonlyAppChanged(bool checked);
    void onToggleDisplayFullScreenModeChanged(bool checked);
    void onSelectAppBtnClick();

    void onToggleHideDesktopModeChanged(bool checked);
    void onPositionChanged(int );
    void onTaskbarSizeModeChanged(int);
    void onTaskbarButtonChanged(int);
    void onDualScreenChanged(int);
    void onSelectSystemIconBtnClick();

    void onConfigurationChanged();
    void onScreenAdded();
    void onScreenRemoved();

private:
    void initUI();
    void setMenuDisplayMode(MenuDisplayMode mode);
    void loadSettings();    // 加载配置文件，初始化相应参数
    bool isDualScreen();    // 判断是否是双屏
    void setMainScreenShow();  // 在主屏上显示任务栏
    void setAllScreenShow();   // 在所有屏上显示任务栏

private:
    Ui::Taskbar *ui;
    NdeWidget::QMaterialToggle * toggleDisplayCommonlyApp_;
    NdeWidget::QMaterialToggle * toggleDisplayFullScreenMode_;

    NdeWidget::QMaterialToggle * toggleHideDesktopMode_;
    Nde::Settings *menuSetting_;      //菜单配置
    MenuDisplayMode menuDisplayMode_;               //全屏或窗口模式显示
    bool menuCommonShow_;           //开始菜单是否显示常用按钮
    bool panelHidable_;
    QString position_;
    QMap<QString, int> positionMap_;
    QString buttonStyle_;
    QMap<QString, int> buttonStyleMap_;
    int panelSize_;
    bool isAcceptPosionChange_;
    bool isAcceptTask_;

    QSettings *confSetting_;
    QMap<QString,QString> pluginMap_;
    QMap<QString, ApplicationFrame*> mapApps_;

    bool isDualScreenMode_;
    bool dualScreenClick_{false};
};



#endif // TASKBAR_H
