#include "iconthemeconfig.h"

#include <Nde/Settings>
#include <QStringList>
#include <QStringBuilder>
#include <QIcon>
#include <QDebug>

IconThemeConfig::IconThemeConfig(Nde::Settings* settings):
    m_settings(settings)
{
    initIconsThemes();
}


void IconThemeConfig::initIconsThemes()
{
    QStringList processed;
    const QStringList baseDirs = QIcon::themeSearchPaths();
    static const QStringList iconNames = QStringList()
                    << QStringLiteral("user-home")
                    << QStringLiteral("user-trash")
                    << QStringLiteral("input-keyboard")
                    << QStringLiteral("applications-internet")
                    << QStringLiteral("applications-graphics")
                    << QStringLiteral("applications-multimedia");

    const int iconNamesN = iconNames.size();

    for (const QString &baseDirName : baseDirs)
    {
        QDir baseDir(baseDirName);
        if (!baseDir.exists())
            continue;

        const QFileInfoList dirs = baseDir.entryInfoList(QDir::AllDirs | QDir::NoDotAndDotDot, QDir::Name);
        for (const QFileInfo &dir : dirs)
        {
            if (!processed.contains(dir.canonicalFilePath()))
            {
                processed << dir.canonicalFilePath();

                IconThemeInfo theme(QDir(dir.canonicalFilePath()));
                if (theme.isValid() && (!theme.isHidden()))
                {
                    QVector<QIcon> icons = theme.icons(iconNames);

                    QString themeDescription;
                    if (theme.comment().isEmpty())
                    {
                        themeDescription = theme.text();
                    }
                    else
                    {
                        themeDescription = theme.text() % QStringLiteral(" (") % theme.comment() % QStringLiteral(")");
                    }


                    themeInfos << theme;
                    iconsMap.insert(theme.name(), icons);
                    currentIconThemeName = QIcon::themeName();
                }
            }
        }
    }
}


IconThemeConfig::~IconThemeConfig()
{
}


void IconThemeConfig::apply(QString iconThemeText)
{
    if (getCurrentIconThemeText() != iconThemeText){
        for (const IconThemeInfo &theme : themeInfos){
            if (theme.text() == iconThemeText){
                QIcon::setThemeName(theme.name());
                m_settings->setValue("icon_theme",  theme.name());
                m_settings->sync();
                currentIconThemeName = theme.name();
            }
        }
    }

}

QStringList IconThemeConfig::listIconThemeText()
{
    QStringList themeTexts;
    for (const IconThemeInfo &theme : themeInfos){
        themeTexts << theme.text();
    }
    return themeTexts;
}

QStringList IconThemeConfig::listIconThemeName()
{
    QStringList themeNames;
    for (const IconThemeInfo &theme : themeInfos){
        themeNames << theme.name();
    }
    return themeNames;
}

QStringList IconThemeConfig::listIconThemeComment()
{
    QStringList themeComments;
    for (const IconThemeInfo &theme : themeInfos){
        themeComments << theme.comment();
    }
    return themeComments;
}

QString IconThemeConfig::getCurrentIconThemeName()
{
    return  currentIconThemeName;
}

QString IconThemeConfig::getCurrentIconThemeText()
{
    for (const IconThemeInfo &theme : themeInfos){
        if (currentIconThemeName == theme.name())
            return theme.text();
    }
    return QString();
}

QVector<QIcon> IconThemeConfig::listIconThemeIcon(QString iconName)
{
    for (const IconThemeInfo &theme : themeInfos){
        if (iconName == theme.text()){
            if(iconsMap.contains(theme.name())){
                return iconsMap.find(theme.name()).value();
            }
        }
    }

    QVector<QIcon> null;
    return null;
}
