
#include "iconthemeinfo.h"



IconThemeInfo::IconThemeInfo(const QDir &dir):
    mValid(false),
    mHidden(false)
{
    mName = dir.dirName();
    if (dir.exists(QStringLiteral("index.theme")))
        load(dir.absoluteFilePath(QStringLiteral("index.theme")));
}


void IconThemeInfo::load(const QString &fileName)
{
    mFileName = fileName;
    mValid = false;
    QSettings file(mFileName, QSettings::IniFormat);
    if (file.status() != QSettings::NoError)
        return;

    if (file.value(QStringLiteral("Icon Theme/Directories")).toStringList().join(QLatin1Char(' ')).isEmpty())
        return;

    mHidden = file.value(QStringLiteral("Icon Theme/Hidden"), false).toBool();
    mText = file.value(QStringLiteral("Icon Theme/Name")).toString();
    mComment = file.value(QStringLiteral("Icon Theme/Comment")).toString();

    mValid = true;
}


QVector<QIcon> IconThemeInfo::icons(const QStringList &iconNames) const
{
    QVector<QIcon> icons;
    QString current_theme = QIcon::themeName();

    QIcon::setThemeName(mName);
    for (const auto & i : iconNames)
    {
        icons.push_back({QIcon::fromTheme(i).pixmap({PREVIEW_ICON_SIZE, PREVIEW_ICON_SIZE})});
    }

    QIcon::setThemeName(current_theme);

    return icons;
}
