#include "maintheme.h"
#include <QStandardPaths>
#include <QProcess>
#include <QDebug>
#include <QPainter>
#include <QFileInfo>
#include <QStyleFactory>
#include <QDBusInterface>

#include <XdgDirs>
#include <QDir>

#define DEFAULT_WALLPAPERPATH "/usr/share/backgrounds/images/others/"

static bool isWallpaperChanged(const QString & themeWallpaper)
{
    static const QString config_path = QStandardPaths::writableLocation(QStandardPaths::ConfigLocation)
        + QStringLiteral("/nde-fileman/nde/settings.conf");
    static const QString wallpaper_key = QStringLiteral("Desktop/Wallpaper");
    const QString current_wallpaper = QSettings{config_path, QSettings::IniFormat}.value(wallpaper_key).toString();
    return themeWallpaper != current_wallpaper;
}

QString styleTheme()
{
    //QString themeCfg = QString::fromLatin1("%1/theme.cfg").arg(d->mPath);
    QString themeCfg = QString::fromLatin1("/usr/share/nde/themes/Dark/theme.cfg");

    if (themeCfg.isEmpty())
        return QString();

    QSettings s(themeCfg, QSettings::IniFormat);
    QString themeDir = QFileInfo(themeCfg).absolutePath();
    // There is something strange... If I remove next line the wallpapers array is not found...
    s.childKeys();

    if (s.contains(QL1S("style")))
        return QString::fromLatin1("%1").arg(s.value(QL1S("style")).toString());

    return QString();
}

QString kwinTheme()
{
    //QString themeCfg = QString::fromLatin1("%1/theme.cfg").arg(d->mPath);
    QString themeCfg = QString::fromLatin1("/usr/share/nde/themes/Dark/theme.cfg");

    if (themeCfg.isEmpty())
        return QString();

    QSettings s(themeCfg, QSettings::IniFormat);
    s.childKeys();

    if (s.contains(QL1S("kwin_theme")))
        return QString::fromLatin1("%1").arg(s.value(QL1S("kwin_theme")).toString());

    return QString();
}

bool applyKwinTheme(QString name){
    const QString kwinCfg = QStandardPaths::writableLocation(QStandardPaths::ConfigLocation)
        + QStringLiteral("/kwinrc");

    if (kwinCfg.isEmpty())
        return false;

    QSettings s(kwinCfg, QSettings::IniFormat);

    if (s.value("theme").toString() == QString("__aurorae__svg__"+name)){
        return false;
    }

    s.beginGroup(QLatin1String("org.kde.kdecoration2"));
    s.setValue("BorderSize", "Normal");
    s.setValue("ButtonsOnLeft", "");
    s.setValue("ButtonsOnRight", "IAX");
    s.setValue("CloseOnDoubleClickOnMenu", "false");
    s.setValue("library", "org.kde.kwin.aurorae");
    s.setValue("theme", QString("__aurorae__svg__"+name));
    s.endGroup();

    return true;
}


QStringList allKwinThemes()
{
    QStringList ret;
    QSet<QString> processed;

    QStringList paths;
    paths << XdgDirs::dataHome(false);
    paths << XdgDirs::dataDirs();

    for(const QString &path : qAsConst(paths))
    {
        QDir dir(QString::fromLatin1("%1/aurorae/themes").arg(path));
        const QFileInfoList dirs = dir.entryInfoList(QDir::AllDirs | QDir::NoDotAndDotDot);

        for(const QFileInfo &dir : dirs)
        {
            if (!processed.contains(dir.fileName()))
            {
                processed << dir.fileName();
                ret << dir.fileName();
            }

        }
    }

    return ret;
}

NdeThemeConfig::NdeThemeConfig(Nde::Settings *settings) :
    mSettings(settings)
{
    const QList<Nde::NdeTheme> themes = Nde::NdeTheme::allThemes();
    for(const Nde::NdeTheme &theme : themes)
    {
        ndeThemeNames << theme.name();
    }

    mCurrentThemeName = mSettings->value("theme").toString();
}


NdeThemeConfig::~NdeThemeConfig()
{

}

QString NdeThemeConfig::cursorThemeFromThemeConfig()
{
    QString themeCfg = QString::fromLatin1("/usr/share/nde/themes/%1/theme.cfg").arg(mCurrentThemeName);

    if (themeCfg.isEmpty())
        return QString();

    QSettings s(themeCfg, QSettings::IniFormat);
    s.childKeys();

    if (s.contains(QL1S("cursor_theme")))
        return QString::fromLatin1("%1").arg(s.value(QL1S("cursor_theme")).toString());

    return QString();
}

QString NdeThemeConfig::iconThemeFromThemeConfig()
{
    QString themeCfg = QString::fromLatin1("/usr/share/nde/themes/%1/theme.cfg").arg(mCurrentThemeName);

    if (themeCfg.isEmpty())
        return QString();

    QSettings s(themeCfg, QSettings::IniFormat);
    s.childKeys();

    if (s.contains(QL1S("icon_theme")))
        return QString::fromLatin1("%1").arg(s.value(QL1S("icon_theme")).toString());

    return QString();
}

QString NdeThemeConfig::currentNdeTheme()
{
    return mCurrentThemeName;
}

QStringList NdeThemeConfig::listNdeThemeName()
{
    return ndeThemeNames;
}

void NdeThemeConfig::apply(QString mainThemeName)
{
    Nde::NdeTheme currentTheme{mSettings->value("theme").toString()};

    if(mSettings->value("theme").toString() != mainThemeName){
        mCurrentThemeName = mainThemeName;
        mSettings->setValue("theme", mainThemeName);
        mSettings->sync();
    }else{
        return ;
    }

    //set cursor if config exist
    QString cursorTheme = cursorThemeFromThemeConfig();
    if (!cursorTheme.isEmpty()){
        Q_EMIT setCursorFromConfig(cursorTheme);
    }

    //set icon if config exist
    QString iconTheme = iconThemeFromThemeConfig();
    if (!iconTheme.isEmpty() /*&& mSettings->value("icon_theme").toString() != iconTheme*/){
        Q_EMIT setIconFromConfig(iconTheme);
    }

//    // set style
//    QString style = styleTheme();
//    if (!style.isEmpty() && QStyleFactory::keys().contains(style)){
//        mSettings->beginGroup(QLatin1String("Qt"));
//        if(mSettings->value("style").toString() != styleTheme())
//            mSettings->setValue("style", styleTheme());
//        mSettings->endGroup();
//    }

//    // set desktop background
   Nde::NdeTheme theme(mainThemeName);
   if(theme.isValid()) {
       QString wallpaper = DEFAULT_WALLPAPERPATH + theme.originDesktopBackground();
       QFileInfo wallpaperInfo(wallpaper);
       if(wallpaperInfo.exists() && (isWallpaperChanged(wallpaper))) {
           // call pcmanfm-qt to update wallpaper
           QStringList args;
           args << "--set-wallpaper" << wallpaper;
           QProcess::startDetached("nde-fileman", args);
       }
   }

//    // set kwin theme
//    QString kwinDecoration = kwinTheme();

//    if (!kwinDecoration.isEmpty() && allKwinThemes().contains(kwinDecoration)){
//        bool ret = applyKwinTheme(kwinDecoration);

//        if (ret){
//            QDBusInterface *m_interface = new QDBusInterface(QStringLiteral("org.kde.KWin"),
//                                             QStringLiteral("/KWin"),
//                                             QStringLiteral("org.kde.KWin"),
//                                             QDBusConnection::sessionBus(),
//                                             this);
//            if (m_interface->isValid()){
//                m_interface->call("reconfigure");
//            }
//        }
//    }
}
