#include "mainthemelist.h"
#include "themepatternbtn.h"
#include <QListWidgetItem>
#include <QResizeEvent>
#include <QDebug>
#include <QScrollBar>

MainThemeList::MainThemeList(QWidget *parent) : QListWidget(parent)
{
    this->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
    this->setMovement(Static);
    this->setSelectionMode(SingleSelection);
    this->setResizeMode(Adjust);
    this->setSizeAdjustPolicy(AdjustToContents);
    this->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    this->setViewMode(IconMode);
    this->setLayoutMode(Batched);
    this->setDragEnabled(false);
    this->setFlow(LeftToRight);
    this->verticalScrollBar()->setSingleStep(10);
    this->setSpacing(mSpace);

    mSelectedIndex = 0;
    mSpace = 10;
    m_itemSize = QSize(THEME_BTN_SIZE_WID,THEME_BTN_SIZE_HEI + THEME_BTN_BOTTM_SIZE_HEI);

    this->setGridSize(m_itemSize);
    this->setMinimumHeight(THEME_BTN_SIZE_HEI + THEME_BTN_BOTTM_SIZE_HEI+mSpace);
}

void MainThemeList::addItem(QString name, int id)
{
    QListWidgetItem * item = new QListWidgetItem(this);
    ThemePatternBtn * itemWgt = new ThemePatternBtn(this, id);
    itemWgt->setThemeName(name);

    item->setSizeHint(m_itemSize);

    this->setItemWidget(item, itemWgt);

    themeBtnList << itemWgt;
    connect(itemWgt, SIGNAL(clicked(int)), this, SLOT(themePressEvent(int)));
}

void MainThemeList::themePressEvent(int index)
{
    mSelectedIndex = index;
    for (int i = 0; i < themeBtnList.count(); i++){
        themeBtnList.at(i)->setCheck(index == i);
    }
}

void MainThemeList::setCurrentTheme(QString name)
{
    for (int i = 0; i < themeBtnList.count(); i++){
        if (themeBtnList.at(i)->themeName() == name){
            themeBtnList.at(i)->setCheck(true);
            mSelectedIndex = i;
        }
    }
}

int MainThemeList::getSelectedIndex()
{
    return mSelectedIndex;
}

void MainThemeList::resizeEvent(QResizeEvent *e)
{
    QSize size = this->size();
    int verticalScrollBarWidth = this->verticalScrollBar()->isVisible() ? this->verticalScrollBar()->width():0;
    QSize newGridSize = m_itemSize;

    int col = (size.width()-verticalScrollBarWidth-mSpace)/(m_itemSize.width()+mSpace);
    //int leftSpace = size.width()-verticalScrollBarWidth  - mSpace - (m_itemSize.width()+mSpace)*col;
    if(col > 0 )
    {
        newGridSize = QSize(m_itemSize.width() + mSpace, m_itemSize.height()+mSpace);
        this->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    }
    else
    {
        this->setHorizontalScrollBarPolicy(Qt::ScrollBarAsNeeded);
    }

    if (this->count()%col == 0)
        this->setMinimumHeight((this->count()/col)* (m_itemSize.height()+mSpace));
    else
        this->setMinimumHeight((this->count()/col+1)*(m_itemSize.height()+mSpace));

    this->setGridSize(newGridSize);

    QListWidget::resizeEvent(e);
}
