#include "avatarsetting.h"
#include "ui_avatarsetting.h"
#include <QFile>
#include <QDir>
#include <KAuth/KAuthAction>
#include <KAuth/KAuthExecuteJob>
#include <QFileDialog>
#include <QStandardPaths>
#include <QLabel>
#include <QBitmap>
#include <QPainter>
#include <QFileInfo>
#include <QTimer>
#include <QSize>
#include <QImage>
#include <QImageReader>
#include <QProcess>
#include "userinstance.h"

#define IMAGES_PATH "/var/lib/AccountsService/images/"
#define IMAGES_LOCAL_PATH "/var/lib/AccountsService/images/local/"

AvatarSetting::AvatarSetting(QWidget *parent) :
    Page(parent),
    ui(new Ui::AvatarSetting)
{
    ui->setupUi(this);

    QFile f(":/qss/avatarsetting.qss");
    if(f.open(QIODevice::ReadOnly))
    {
        QString style = f.readAll().replace("\n", "");
        this->setStyleSheet(style);
    }

//    updateWidget();

    connect(ui->avatarListWidget, &QListWidget::itemClicked, this, &AvatarSetting::onAvatarItemClick);
}

AvatarSetting::~AvatarSetting()
{
    delete ui;
}

void AvatarSetting::backBtnClicked()
{

}

void AvatarSetting::applyBtnClicked()
{

}

void AvatarSetting::refreshPage()
{

}

void AvatarSetting::updateWidget() {
    ui->avatarListWidget->setResizeMode(QListView::Adjust);
    ui->avatarListWidget->setViewMode(QListView::IconMode);
    ui->avatarListWidget->setMovement(QListView::Static);
    ui->avatarListWidget->setSpacing(28);

    // 判断路径是否存在
    QDir dir(IMAGES_PATH);
    if (!dir.exists()) {
        return;
    }
    // 设置过滤器
    dir.setFilter(QDir::Files | QDir::NoSymLinks);
    QStringList filters;
    filters << "*.png" << "*.jpg";
    dir.setNameFilters(filters);
    imgList_ = dir.entryList();
    if (imgList_.count() <= 0) {
        return;
    }

    // 创建单元项
    for (int i = 0; i < imgList_.count(); ++i)
    {
        QString path;
        path = QString(IMAGES_PATH + imgList_.at(i));
        createIconItem(path);
    }
    if(userAccount_ != nullptr) {
        QString otherIconPath = QString(IMAGES_LOCAL_PATH + userAccount_->userName());
        QFileInfo file(otherIconPath);
        if(file.exists()) {
            createIconItem(otherIconPath);
        }
    }
//    else if (userAccount_ == nullptr) {
//        QString otherIconPath = QString(IMAGES_LOCAL_PATH + getCurrentUserName());
//        QFileInfo file(otherIconPath);
//        if(file.exists()) {
//            createIconItem(otherIconPath);
//        }
//    }
//    先屏蔽添加图片按钮，后续添加
    createIconItem(QString(":/images/addIcon.png"));
}

void AvatarSetting::onAvatarItemClick(QListWidgetItem* item) {
//    if(avatarClick_){
//        return ;
//    }
//    avatarClick_ = true;
    QString home_path = QStandardPaths::writableLocation(QStandardPaths::HomeLocation);
    int index = ui->avatarListWidget->row(item);
    QString iconPath;
    int listNum = imgList_.count();
    if(index < listNum) {
        iconPath = QString(IMAGES_PATH + imgList_.at(index));
    }
    else if((ui->avatarListWidget->count() == listNum + 1 && index == listNum)|| (ui->avatarListWidget->count() == listNum + 2 && index == listNum + 1))
    {
        QFileDialog fd;
        fd.setNameFilter(tr("Images") + "(*.png *.bmp *.jpg *.jpeg)");

        QStringList directory = QStandardPaths::standardLocations(QStandardPaths::PicturesLocation);
        if (!directory.isEmpty()) {
            fd.setDirectory(directory.first());
        }

        if (fd.exec() == QFileDialog::Accepted) {
            if(userAccount_ != nullptr) {
                adjustImage(fd.selectedFiles().first(), home_path + "/.config/nde/" + userAccount_->userName() + ".png");
                iconPath = home_path + "/.config/nde/" + userAccount_->userName() + ".png";
            }
            else
            {
                iconPath = fd.selectedFiles().first();
            }
            
        }
        else {
            QTimer r;
            r.singleShot(200,[this](){
                this->avatarClick_=false;
            });
            return;
        }
    }
    else if(ui->avatarListWidget->count() == listNum + 2 && index == listNum) {
        if(userAccount_ != nullptr) {
            iconPath = QString(IMAGES_LOCAL_PATH + userAccount_->userName());
        }
        else if(userAccount_ == nullptr){
            iconPath = QString(IMAGES_LOCAL_PATH + UserInstance::getIns()->getCurrentUserName());
        }
    }

    //先返回，再处理图片路径变化
    Q_EMIT backPage();
    Q_EMIT avatarPathChange(iconPath);
}

void AvatarSetting::createIconItem(const QString path) {
    QPixmap img;
    img.load(path);
    QPixmap pixmap = img.scaled(90, 90, Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
    QSize size(90, 90);
    QBitmap mask(size);
    QPainter painter(&mask);
    painter.setRenderHint(QPainter::Antialiasing);
    painter.setRenderHint(QPainter::SmoothPixmapTransform);
    painter.fillRect(0, 0, size.width(), size.height(), Qt::white);
    painter.setBrush(QColor(0, 0, 0));
    painter.drawRoundedRect(0, 0, size.width(), size.height(), 90, 90);
    QPixmap image = pixmap;
    image.setMask(mask);
    QListWidgetItem *item = new QListWidgetItem();
    QLabel *imageLabel = new QLabel;
    imageLabel->setPixmap(image);
    item->setSizeHint(QSize(90,90));
    ui->avatarListWidget->addItem(item);
    ui->avatarListWidget->setSizeIncrement(90,90);
    ui->avatarListWidget->setItemWidget(item, imageLabel);

}

void AvatarSetting::adjustImage(QString sourceUrl, QString targetPath)
{
    QPixmap pixMask = QPixmap(sourceUrl);
    QSize size(pixMask.width(), pixMask.height());
    QSize size2(100, 100);
    QBitmap mask(size);
    QPainter painter(&mask);
    painter.setRenderHint(QPainter::SmoothPixmapTransform);
    painter.setRenderHint(QPainter::Antialiasing);
    painter.setRenderHint(QPainter::TextAntialiasing);
    painter.translate(0, 0);
    painter.fillRect(0, 0, size.width(), size.height(), Qt::white);
    painter.setBrush(QColor(0, 0, 0));
    painter.drawEllipse(0, 0, size.width(), size.height());
    pixMask.setMask(mask);
    QPixmap image = pixMask.scaled(size2);
    image.save(targetPath);
}

void AvatarSetting::setUserAccount(UserAccount *user) {
    userAccount_ = user;
    ui->avatarListWidget->clear();
    updateWidget();
}
