#include "avatarwidget.h"

#include <QDebug>
#include <QUrl>
#include <QPainterPath>
#include <QPaintEvent>
#include <QVBoxLayout>
#include <QApplication>
#include <QRect>
#include <QColor>
#include <QBitmap>

AvatarWidget::AvatarWidget(QWidget *parent)
    : QLabel(parent)
    , m_hover(false)
    , m_deleable(false)
    , m_selected(false)
    , m_painter(nullptr)
{
    QVBoxLayout *mainLayout = new QVBoxLayout;
    mainLayout->setMargin(0);
    mainLayout->setSpacing(0);

    setLayout(mainLayout);
    setFixedSize(PIX_SIZE, PIX_SIZE);
    setObjectName("AvatarWidget");
}

AvatarWidget::~AvatarWidget()
{
}

AvatarWidget::AvatarWidget(const QString &avatar, QWidget *parent)
    : AvatarWidget(parent)
{
    setAvatarPath(avatar);
}

void AvatarWidget::setSelected(const bool selected)
{
    m_selected = selected;
    update();
}

void AvatarWidget::setDeletable(const bool deletable)
{
    m_deleable = deletable;
    update();
}

const QString AvatarWidget::avatarPath() const
{
    return m_avatarPath;
}

void AvatarWidget::setAvatarPath(const QString &avatar)
{
    m_avatarPath = avatar;
    if (!QPixmap(m_avatarPath).isNull()) {
        drawAvatar();
    }
    setAccessibleName(m_avatarPath);
    update();
}

void AvatarWidget::mouseReleaseEvent(QMouseEvent *e)
{
    if (rect().contains(e->pos()))
        Q_EMIT clicked(avatarPath());

    QWidget::mouseReleaseEvent(e);
}

void AvatarWidget::paintEvent(QPaintEvent *e)
{
    QPainterPath painterPath;
    painterPath.addEllipse(QRect(0, 0, width(), height()));
    m_painter = new QPainter(this);
    m_painter->setRenderHint(QPainter::Antialiasing, true);
    m_painter->setClipPath(painterPath);
    m_painter->drawPixmap(this->rect(), m_avatar);

    if (m_selected) {
        setAccessibleDescription("selectedIcon");
        QPen pen(Qt::transparent);
        pen.setWidth(4);
        pen.setColor(Qt::white);

        m_painter->setPen(pen);
        m_painter->setBrush(Qt::transparent);
        m_painter->drawEllipse(rect());
    }

//    if(m_hover)
//    {
//        m_painter->setPen(Qt::NoPen);
//        // 高度
//        int w = this->rect().width();
//        int h = this->rect().height();
//        // 矩形
//        QRect rect(0, h - h / 3, w, h - h / 3);
//        // 反走样
//        m_painter->setRenderHint(QPainter::Antialiasing, true);
//        QLinearGradient linear(QPoint(0, h - h / 4), QPoint(0, h));
//        linear.setColorAt(0.5, QColor(0, 0, 0, 150));

//        // 设置显示模式
//        linear.setSpread(QGradient::PadSpread);
//        m_painter->setBrush(linear);
//        m_painter->drawRect(rect);
//    }

    QPen pen(Qt::transparent);
    pen.setWidth(2);
    pen.setColor(Qt::white);
    m_painter->setPen(pen);
    //把直径平均分成10份
    int portion = this->rect().width() / 10;
    //圆中心点坐标
    QPoint cpt = this->rect().center();
    m_painter->drawText(QPoint(cpt.x() - portion - 6 , cpt.y() + portion * 4 - portion / 2 + 4), tr("Edit"));

    QWidget::paintEvent(e);
    delete m_painter;
}

void AvatarWidget::enterEvent(QEvent *)
{
    m_hover = true;
    update();
}

void AvatarWidget::leaveEvent(QEvent *)
{
    m_hover = false;
    update();
}

void AvatarWidget::resizeEvent(QResizeEvent *event)
{
    QWidget::resizeEvent(event);
    drawAvatar();
    update();
}

void AvatarWidget::drawAvatar() {
    QPixmap img;
    img.load(m_avatarPath);
    QPixmap pixmap = img.scaled(PIX_SIZE, PIX_SIZE, Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
    QSize size(PIX_SIZE, PIX_SIZE);
    QBitmap mask(size);
    QPainter painter(&mask);
    painter.setRenderHint(QPainter::Antialiasing);
    painter.setRenderHint(QPainter::SmoothPixmapTransform);
    painter.fillRect(0, 0, size.width(), size.height(), Qt::white);
    painter.setBrush(QColor(0, 0, 0));
    painter.drawRoundedRect(0, 0, size.width(), size.height(), 90, 90);
    m_avatar = pixmap;
    m_avatar.setMask(mask);
}
