#include "changepassword.h"
#include "ui_changepassword.h"
#include <QFile>
#include <KAuth/KAuthAction>
#include <KAuth/KAuthExecuteJob>
#include "userinstance.h"

ChangePassword::ChangePassword(QWidget *parent) :
    Page(parent),
    ui(new Ui::ChangePassword),
    userAccount_(nullptr),
    messageW_(nullptr)
{
    ui->setupUi(this);

    QFile f(":/qss/changepassword.qss");
    if(f.open(QIODevice::ReadOnly))
    {
        QString style = f.readAll().replace("\n", "");
        this->setStyleSheet(style);
    }
    ui->newPasswordLineEdit->setEchoMode(QLineEdit::Password);
    ui->repeatPasswordLineEdit->setEchoMode(QLineEdit::Password);

    connect(ui->newPasswordLineEdit, &QLineEdit::textChanged, this, &ChangePassword::onClosePasswordMessage);
    connect(ui->repeatPasswordLineEdit, &QLineEdit::textChanged, this, &ChangePassword::onCloseRepeatPasswordMessage);

    messageW_ = new MessageWidget(this);
    messageW_->setDirection(MessageWidget::Direction::up);
    messageW_->setStartPos(20);
    messageW_->hide();
}

ChangePassword::~ChangePassword()
{
    delete messageW_;
    delete ui;
}

void ChangePassword::backBtnClicked()
{
    ui->newPasswordLineEdit->setText("");
    ui->repeatPasswordLineEdit->setText("");
}

void ChangePassword::applyBtnClicked()
{
    QString newPassword = ui->newPasswordLineEdit->text();
    QString repeatPassword = ui->repeatPasswordLineEdit->text();

    if(newPassword.isEmpty()) {
        messageW_->showWidget(ui->newPasswordLineEdit->mapTo(this, QPoint(0, 0)).x() + 20,
                              ui->newPasswordLineEdit->mapTo(this, QPoint(0, 0)).y() + ui->newPasswordLineEdit->height() - 12,
                              tr("The new password cannot be empty"));
        return;
    }

    if(newPassword.contains(" ")) {
        messageW_->showWidget(ui->newPasswordLineEdit->mapTo(this, QPoint(0, 0)).x() + 20,
                              ui->newPasswordLineEdit->mapTo(this, QPoint(0, 0)).y() + ui->newPasswordLineEdit->height() - 10,
                              tr("Password cannot have spaces"));

        return;
    }

    if(newPassword.size() > MAXINPUTSIZE) {
        messageW_->showWidget(ui->newPasswordLineEdit->mapTo(this, QPoint(0, 0)).x() + 20,
                              ui->newPasswordLineEdit->mapTo(this, QPoint(0, 0)).y() + ui->newPasswordLineEdit->height() - 10,
                              tr("Password must be shorter than 16 characters"));
        return;
    }

    if(repeatPassword.isEmpty()) {
        messageW_->showWidget(ui->repeatPasswordLineEdit->mapTo(this, QPoint(0, 0)).x() + 20,
                              ui->repeatPasswordLineEdit->mapTo(this, QPoint(0, 0)).y() + ui->repeatPasswordLineEdit->height() - 12,
                              tr("The repeat password cannot be empty"));
        return;
    }

    if(repeatPassword.contains(" ")) {
        messageW_->showWidget(ui->repeatPasswordLineEdit->mapTo(this, QPoint(0, 0)).x() + 20,
                              ui->repeatPasswordLineEdit->mapTo(this, QPoint(0, 0)).y() + ui->repeatPasswordLineEdit->height() - 10,
                              tr("Password cannot have spaces"));
        return;
    }

    if(repeatPassword.size() > MAXINPUTSIZE) {
        messageW_->showWidget(ui->repeatPasswordLineEdit->mapTo(this, QPoint(0, 0)).x() + 20,
                              ui->repeatPasswordLineEdit->mapTo(this, QPoint(0, 0)).y() + ui->repeatPasswordLineEdit->height() - 10,
                              tr("Repeat password must be shorter than 16 characters"));
        return;
    }

    if(newPassword == repeatPassword) {
        if(UserInstance::getIns()->isAdministratorUser()) {
            KAuth::Action action("org.nde.auth.helper.setpassword");
            if (!action.isValid()) {
                return ;
            }
            action.setHelperId(QStringLiteral("org.nde.auth.helper"));
            action.addArgument(QStringLiteral("userName"), userAccount_->userName());
            action.addArgument(QStringLiteral("password"), newPassword);

            KAuth::ExecuteJob *job = action.execute();
            if(job->exec()) {
                ui->newPasswordLineEdit->setText("");
                ui->repeatPasswordLineEdit->setText("");
                Q_EMIT backPage();
            }
        }
        else {
            AccountsManager *manager = UserInstance::getIns()->getAccountsManager();
            if(manager->editAUserPasswd(userAccount_->userName(), true, newPassword)) {
                ui->newPasswordLineEdit->setText("");
                ui->repeatPasswordLineEdit->setText("");
                Q_EMIT backPage();
            }
        }

    }
    else {
        messageW_->showWidget(ui->repeatPasswordLineEdit->mapTo(this, QPoint(0, 0)).x() + 20,
                              ui->repeatPasswordLineEdit->mapTo(this, QPoint(0, 0)).y() + ui->repeatPasswordLineEdit->height() - 10,
                              tr("The two password inputs are not the same"));
        return;
    }
}

void ChangePassword::refreshPage()
{

}

void ChangePassword::mousePressEvent(QMouseEvent *)
{
    messageW_->close ();
}

void ChangePassword::resizeEvent(QResizeEvent* event)
{
    QWidget::resizeEvent(event);
    messageW_->close ();
}

void ChangePassword::onClosePasswordMessage() {
    QString passText = ui->newPasswordLineEdit->text();
    int curCount = passText.count();
    if(curCount > MAXINPUTSIZE) {
        ui->newPasswordLineEdit->setMaxLength(MAXINPUTSIZE + 1);
        messageW_->showWidget(ui->newPasswordLineEdit->mapTo(this, QPoint(0, 0)).x() + 20,
                              ui->newPasswordLineEdit->mapTo(this, QPoint(0, 0)).y() + ui->newPasswordLineEdit->height() - 10,
                              tr("Password must be shorter than 16 characters"));
    }
    else {
        messageW_->close ();
    }
}

void ChangePassword::onCloseRepeatPasswordMessage() {
    QString passText = ui->repeatPasswordLineEdit->text();
    int curCount = passText.count();
    if(curCount > MAXINPUTSIZE) {
        ui->repeatPasswordLineEdit->setMaxLength(MAXINPUTSIZE + 1);
        messageW_->showWidget(ui->repeatPasswordLineEdit->mapTo(this, QPoint(0, 0)).x() + 20,
                              ui->repeatPasswordLineEdit->mapTo(this, QPoint(0, 0)).y() + ui->repeatPasswordLineEdit->height() - 10,
                              tr("Repeat password must be shorter than 16 characters"));
    }
    else {
        messageW_->close (); // 关闭提示窗口
    }
}

