#include "changeusername.h"
#include "ui_changeusername.h"
#include <QFile>
#include <KAuth/KAuthAction>
#include <KAuth/KAuthExecuteJob>
#include <QTimer>
#include "userinstance.h"

ChangeUserName::ChangeUserName(QWidget *parent) :
    Page(parent),
    ui(new Ui::ChangeUserName),
    userAccount_(nullptr),
    messageW_(nullptr)
{
    ui->setupUi(this);

    QFile f(":/qss/changeusername.qss");
    if(f.open(QIODevice::ReadOnly))
    {
        QString style = f.readAll().replace("\n", "");
        this->setStyleSheet(style);
    }

    connect(ui->userNameLineEdit, &QLineEdit::textChanged, this, &ChangeUserName::onCloseNameMessage);

    messageW_ = new MessageWidget(this);
    messageW_->setDirection(MessageWidget::Direction::up);
    messageW_->setStartPos(20);
    messageW_->hide();
}

ChangeUserName::~ChangeUserName()
{
    delete messageW_;
    delete ui;
}

void ChangeUserName::backBtnClicked()
{
     ui->userNameLineEdit->setText("");
}

void ChangeUserName::applyBtnClicked()
{
    QString userName = ui->userNameLineEdit->text();
    AccountsManager *manager = new AccountsManager();
    userName = userName.simplified();   // 去掉前后空格
    if(userName.isEmpty()) {
        messageW_->showWidget(ui->userNameLineEdit->mapTo(this, QPoint(0, 0)).x() + 20,
                              ui->userNameLineEdit->mapTo(this, QPoint(0, 0)).y() + ui->userNameLineEdit->height() - 10,
                              tr("The user name cannot be empty"));
        return;
    }

    if(userName.startsWith("-")) {
        messageW_->showWidget(ui->userNameLineEdit->mapTo(this, QPoint(0, 0)).x() + 20,
                              ui->userNameLineEdit->mapTo(this, QPoint(0, 0)).y() + ui->userNameLineEdit->height() - 10,
                              tr("Name cannot start with '-' characher"));
        return;
    }

    if(userName.startsWith(".")) {
        messageW_->showWidget(ui->userNameLineEdit->mapTo(this, QPoint(0, 0)).x() + 20,
                              ui->userNameLineEdit->mapTo(this, QPoint(0, 0)).y() + ui->userNameLineEdit->height() - 10,
                              tr("Name cannot start with '.' characher"));
        return;
    }

    if(userName == "." || userName == "..") {
        messageW_->showWidget(ui->userNameLineEdit->mapTo(this, QPoint(0, 0)).x() + 20,
                              ui->userNameLineEdit->mapTo(this, QPoint(0, 0)).y() + ui->userNameLineEdit->height() - 10,
                              tr("Name '%1' is not allowed").arg(userName));
        return;
    }

    if(messageW_->isDigitStr(userName)) {
        messageW_->showWidget(ui->userNameLineEdit->mapTo(this, QPoint(0, 0)).x() + 20,
                              ui->userNameLineEdit->mapTo(this, QPoint(0, 0)).y() + ui->userNameLineEdit->height() - 10,
                              tr("Fully numeric name is not allowed"));
        return;
    }

    if(userName == "$") {
        messageW_->showWidget(ui->userNameLineEdit->mapTo(this, QPoint(0, 0)).x() + 20,
                              ui->userNameLineEdit->mapTo(this, QPoint(0, 0)).y() + ui->userNameLineEdit->height() - 10,
                              tr("Name '$' is not allowed"));
        return;
    }

    if(userName.contains(" ", Qt::CaseSensitive)) {
        messageW_->showWidget(ui->userNameLineEdit->mapTo(this, QPoint(0, 0)).x() + 20,
                              ui->userNameLineEdit->mapTo(this, QPoint(0, 0)).y() + ui->userNameLineEdit->height() - 10,
                              tr("Name cannot contain spaces"));
        return;
    }

    QRegExp rx(PORTABLE_FS_CHARS);
    int pos = 0;
    QString matchStr = "";

    while((pos = rx.indexIn(userName, pos)) != -1) {
        matchStr += userName[pos] + ",";
        pos += rx.matchedLength();
    }

    if(!matchStr.isEmpty()) {
        matchStr = matchStr.left(matchStr.length() - 1);
        messageW_->showWidget(ui->userNameLineEdit->mapTo(this, QPoint(0, 0)).x() + 20,
                              ui->userNameLineEdit->mapTo(this, QPoint(0, 0)).y() + ui->userNameLineEdit->height() - 10,
                              tr("Name cannot contain character: '%1'").arg(matchStr));
        return;
    }

    QRegExp rxend(PORTABLE_END_CHARS);
    if(rxend.exactMatch(userName.right(1))) {
        messageW_->showWidget(ui->userNameLineEdit->mapTo(this, QPoint(0, 0)).x() + 20,
                              ui->userNameLineEdit->mapTo(this, QPoint(0, 0)).y() + ui->userNameLineEdit->height() - 10,
                              tr("End of name cannot contain special character: '%1'").arg(userName.right(1)));
        return;
    }

    if(userName.size() > MAXINPUTSIZE) {
        messageW_->showWidget(ui->userNameLineEdit->mapTo(this, QPoint(0, 0)).x() + 20,
                              ui->userNameLineEdit->mapTo(this, QPoint(0, 0)).y() + ui->userNameLineEdit->height() - 10,
                              tr("Name must be shorter than 16 characters"));
        return;
    }

    QStringList listDir;
    listDir << "root" << "home" << "daemon" << "system" << "etc" << "boot" << "lib" << "media" << "opt" << "sbin" << "sys"
            << "tmp" << "var" << "bin"<< "dev" << "lib64" << "mnt" << "proc" << "run" << "srv" << "usr";
    if(listDir.contains(userName)) {
        messageW_->showWidget(ui->userNameLineEdit->mapTo(this, QPoint(0, 0)).x() + 20,
                              ui->userNameLineEdit->mapTo(this, QPoint(0, 0)).y() + ui->userNameLineEdit->height() - 12,
                              tr("User name is reserved for system: %1").arg(userName));
        return;
    }

    if(manager->findUserByName(userName)) {
        messageW_->showWidget(ui->userNameLineEdit->mapTo(this, QPoint(0, 0)).x() + 20,
                              ui->userNameLineEdit->mapTo(this, QPoint(0, 0)).y() + ui->userNameLineEdit->height() - 12,
                              tr("The user name already exists"));
        delete manager;
        manager = nullptr;
        return;
    }
    else {
        delete manager;
        manager = nullptr;
    }

    if(UserInstance::getIns()->isUserLogining(userAccount_->userName())) {
        messageW_->showWidget(ui->userNameLineEdit->mapTo(this, QPoint(0, 0)).x() + 20,
                              ui->userNameLineEdit->mapTo(this, QPoint(0, 0)).y() + ui->userNameLineEdit->height() - 12,
                              tr("The user is logging in and cannot modify the user name"));
        return;
    }
    QString oldName = userAccount_->userName();
    if(userName != oldName) {
        bool isAutoLogin = userAccount_->automaticLogin();
        if(isAutoLogin) {
            cancelAutoLogin(oldName, false);
            Q_EMIT autoLoginCancel();
        }

        if(UserInstance::getIns()->isAdministratorUser()) {
            KAuth::Action action("org.nde.auth.helper.setusername");
            if (!action.isValid()) {
                return ;
            }
            action.setHelperId(QStringLiteral("org.nde.auth.helper"));
            action.addArgument(QStringLiteral("userName"), oldName);
            action.addArgument(QStringLiteral("newUserName"), userName);
            KAuth::ExecuteJob *job = action.execute();
            if(job->exec()) {
                ui->userNameLineEdit->setText("");
                Q_EMIT changeAccountName(userName);
                Q_EMIT backPage();
            }
            else {
                messageW_->showWidget(ui->userNameLineEdit->mapTo(this, QPoint(0, 0)).x() + 20,
                                      ui->userNameLineEdit->mapTo(this, QPoint(0, 0)).y() + ui->userNameLineEdit->height() - 12,
                                      tr("Change user name failed"));
                return;
            }
        }
        else {
            AccountsManager *manager = new AccountsManager();
            if(manager->editAUserName(oldName, userName)) {
                avatarRename(oldName, userName);
                ui->userNameLineEdit->setText("");
                Q_EMIT changeAccountName(userName);

                Q_EMIT backPage();
            }
            else {
                messageW_->showWidget(ui->userNameLineEdit->mapTo(this, QPoint(0, 0)).x() + 20,
                                      ui->userNameLineEdit->mapTo(this, QPoint(0, 0)).y() + ui->userNameLineEdit->height() - 12,
                                      tr("Change user name failed"));
                return;
            }
        }
    }
}

void ChangeUserName::refreshPage()
{
    //
}

void ChangeUserName::mousePressEvent(QMouseEvent *)
{
    messageW_->close ();
}

void ChangeUserName::resizeEvent(QResizeEvent* event)
{
    QWidget::resizeEvent(event);
    messageW_->close ();
}

void ChangeUserName::avatarRename(QString oldName, QString newName)
{
    KAuth::Action action("org.nde.auth.helper.changeavatarname");
    if (!action.isValid()) {
        return ;
    }
    action.setHelperId(QStringLiteral("org.nde.auth.helper"));
    action.addArgument(QStringLiteral("oldName"), oldName);
    action.addArgument(QStringLiteral("newName"), newName);
    KAuth::ExecuteJob *job = action.execute();
    job->exec();
}

void ChangeUserName::onCloseNameMessage() {
    QString passText = ui->userNameLineEdit->text();
    int curCount = passText.count();
    if(curCount > MAXINPUTSIZE) {
        ui->userNameLineEdit->setMaxLength(MAXINPUTSIZE + 1);
        messageW_->showWidget(ui->userNameLineEdit->mapTo(this, QPoint(0, 0)).x() + 20,
                              ui->userNameLineEdit->mapTo(this, QPoint(0, 0)).y() + ui->userNameLineEdit->height() - 10,
                              tr("Name must be shorter than 16 characters"));
    }
    else {
        messageW_->close ();
    }
}

void ChangeUserName::cancelAutoLogin(QString userName, bool isCheck) {
    KAuth::Action action("org.nde.auth.helper.setautologin");
    if (!action.isValid()) {
        return ;
    }
    action.setHelperId(QStringLiteral("org.nde.auth.helper"));

    action.addArgument(QStringLiteral("userName"), userName);
    action.addArgument(QStringLiteral("autoLogin"), isCheck);
    KAuth::ExecuteJob *job = action.execute();
    job->exec();
}
