#include "createuser.h"
#include "ui_createuser.h"
#include <QFile>
#include <QRegExp>
#include <QTimer>
#include "userinstance.h"

#define PAGE_BUTTOM_BTN_CREATE (Page::PAGE_BBTN_USER+1)

CreateUser::CreateUser(QWidget *parent) :
    Page(parent),
    ui(new Ui::CreateUser),
    messageW_(nullptr),
    avatar_(nullptr)
{
    ui->setupUi(this);

    QFile f(":/qss/createuser.qss");
    if(f.open(QIODevice::ReadOnly))
    {
        QString style = f.readAll().replace("\n", "");
        this->setStyleSheet(style);
    }

    // 头像图标
    avatar_ = new AvatarWidget();
    ui->horizontalLayout_2->addWidget(avatar_, 0, Qt::AlignTop | Qt::AlignHCenter);
    avatar_->setAvatarPath(":/images/avatar.png");

    ui->password->setEchoMode(QLineEdit::Password);
    ui->repeatPassword->setEchoMode(QLineEdit::Password);

    QStringList typeList;
    typeList << tr("Admin Account") << tr("Standard Account");
    ui->typeComboBox->clear();
    ui->typeComboBox->addItems(typeList);

    messageW_ = new MessageWidget(this);
    messageW_->setDirection(MessageWidget::Direction::up);
    messageW_->setStartPos(20);
    messageW_->hide();

    connect(ui->userName, &QLineEdit::textChanged, this, &CreateUser::onCloseNameMessage);
    connect(ui->password, &QLineEdit::textChanged, this, &CreateUser::onClosePasswordMessage);
    connect(ui->repeatPassword, &QLineEdit::textChanged, this, &CreateUser::onCloseRepeatPasswordMessage);
    connect(avatar_, &AvatarWidget::clicked, this, &CreateUser::onEditAvatar);
}

CreateUser::~CreateUser()
{
    delete messageW_;
    delete ui;
    delete avatar_;
}

void CreateUser::backBtnClicked()
{
    ui->userName->setText("");
    ui->password->setText("");
    ui->repeatPassword->setText("");
//    avatar_->setAvatarPath(":/images/avatar.png");
}

QList<QPair<int, QString> > CreateUser::buttomBtnsPair()
{
    QList<QPair<int, QString> > pairs;
    pairs<<QPair<int, QString>(PAGE_BUTTOM_BTN_CREATE, tr("Create User"));
    return pairs;
}

QList<int> CreateUser::buttomBtnsVisible()
{
    QList<int> keys;
    keys << PAGE_BUTTOM_BTN_CREATE;
    return keys;
}

void CreateUser::buttomBtnClicked(const int key)
{
    if(key == PAGE_BUTTOM_BTN_CREATE)
    {
        QString userName = ui->userName->text();
        QString password = ui->password->text();
        QString repeatPassword = ui->repeatPassword->text();
        int typeIdex = ui->typeComboBox->currentIndex();
        UserAccount::AccountType type = UserAccount::AccountType::AdministratorAccountType;
        userName = userName.simplified();   // 去掉前后空格
        if(userName.isEmpty()) {
            messageW_->showWidget(ui->userName->mapTo(this, QPoint(0, 0)).x() + 20,
                                  ui->userName->mapTo(this, QPoint(0, 0)).y() + ui->userName->height() - 10,
                                  tr("The user name cannot be empty"));
            return;
        }

        if(userName.startsWith("-")) {
            messageW_->showWidget(ui->userName->mapTo(this, QPoint(0, 0)).x() + 20,
                                  ui->userName->mapTo(this, QPoint(0, 0)).y() + ui->userName->height() - 10,
                                  tr("Name cannot start with '-' characher"));
            return;
        }

        if(userName.startsWith(".")) {
            messageW_->showWidget(ui->userName->mapTo(this, QPoint(0, 0)).x() + 20,
                                  ui->userName->mapTo(this, QPoint(0, 0)).y() + ui->userName->height() - 10,
                                  tr("Name cannot start with '.' characher"));
            return;
        }

        if(userName == "." || userName == "..") {
            messageW_->showWidget(ui->userName->mapTo(this, QPoint(0, 0)).x() + 20,
                                  ui->userName->mapTo(this, QPoint(0, 0)).y() + ui->userName->height() - 10,
                                  tr("Name '%1' is not allowed").arg(userName));
            return;
        }

        if(messageW_->isDigitStr(userName)) {
            messageW_->showWidget(ui->userName->mapTo(this, QPoint(0, 0)).x() + 20,
                                  ui->userName->mapTo(this, QPoint(0, 0)).y() + ui->userName->height() - 10,
                                  tr("Fully numeric name is not allowed"));
            return;
        }

        if(userName == "$") {
            messageW_->showWidget(ui->userName->mapTo(this, QPoint(0, 0)).x() + 20,
                                  ui->userName->mapTo(this, QPoint(0, 0)).y() + ui->userName->height() - 10,
                                  tr("Name '$' is not allowed"));
            return;
        }

        if(userName.contains(" ", Qt::CaseSensitive)) {
            messageW_->showWidget(ui->userName->mapTo(this, QPoint(0, 0)).x() + 20,
                                  ui->userName->mapTo(this, QPoint(0, 0)).y() + ui->userName->height() - 10,
                                  tr("Name cannot contain spaces"));
            return;
        }

        QRegExp rx(PORTABLE_FS_CHARS);
        int pos = 0;
        QString matchStr = "";

        while((pos = rx.indexIn(userName, pos)) != -1) {
            matchStr += userName[pos] + ",";
            pos += rx.matchedLength();
        }

        if(!matchStr.isEmpty()) {
            matchStr = matchStr.left(matchStr.length() - 1);
            messageW_->showWidget(ui->userName->mapTo(this, QPoint(0, 0)).x() + 20,
                                  ui->userName->mapTo(this, QPoint(0, 0)).y() + ui->userName->height() - 10,
                                  tr("Name cannot contain character: '%1'").arg(matchStr));
            return;
        }

        QRegExp rxend(PORTABLE_END_CHARS);
        if(rxend.exactMatch(userName.right(1))) {
            messageW_->showWidget(ui->userName->mapTo(this, QPoint(0, 0)).x() + 20,
                                  ui->userName->mapTo(this, QPoint(0, 0)).y() + ui->userName->height() - 10,
                                  tr("End of name cannot contain special character: '%1'").arg(userName.right(1)));
            return;
        }

        if(userName.size() > MAXINPUTSIZE) {
            messageW_->showWidget(ui->userName->mapTo(this, QPoint(0, 0)).x() + 20,
                                  ui->userName->mapTo(this, QPoint(0, 0)).y() + ui->userName->height() - 10,
                                  tr("Name must be shorter than 16 characters"));
            return;
        }

        QStringList listDir;
        listDir << "root" << "home" << "daemon" << "system" << "etc" << "boot" << "lib" << "media" << "opt" << "sbin" << "sys"
                << "tmp" << "var" << "bin"<< "dev" << "lib64" << "mnt" << "proc" << "run" << "srv" << "usr";
        if(listDir.contains(userName)) {
            messageW_->showWidget(ui->userName->mapTo(this, QPoint(0, 0)).x() + 20,
                                  ui->userName->mapTo(this, QPoint(0, 0)).y() + ui->userName->height() - 12,
                                  tr("User name is reserved for system: %1").arg(userName));
            return;
        }

        if(UserInstance::getIns()->getAccountsManager()->findUserByName(userName)) {
            messageW_->showWidget(ui->userName->mapTo(this, QPoint(0, 0)).x() + 20,
                                  ui->userName->mapTo(this, QPoint(0, 0)).y() + ui->userName->height() - 12,
                                  tr("The user name already exists"));
            return;
        }

        if(password.isEmpty()) {
            messageW_->showWidget(ui->password->mapTo(this, QPoint(0, 0)).x() + 20,
                                  ui->password->mapTo(this, QPoint(0, 0)).y() + ui->password->height() - 10,
                                  tr("The new password cannot be empty"));
            return;
        }

        if(password.contains(" ")) {
            messageW_->showWidget(ui->password->mapTo(this, QPoint(0, 0)).x() + 20,
                                  ui->password->mapTo(this, QPoint(0, 0)).y() + ui->password->height() - 10,
                                  tr("Password cannot have spaces"));
            return;
        }

        if(password.size() > MAXINPUTSIZE) {
            messageW_->showWidget(ui->password->mapTo(this, QPoint(0, 0)).x() + 20,
                                  ui->password->mapTo(this, QPoint(0, 0)).y() + ui->password->height() - 10,
                                  tr("Password must be shorter than 16 characters"));
            return;
        }

        if(repeatPassword.isEmpty()) {
            messageW_->showWidget(ui->repeatPassword->mapTo(this, QPoint(0, 0)).x() + 20,
                                  ui->repeatPassword->mapTo(this, QPoint(0, 0)).y() + ui->repeatPassword->height() - 10,
                                  tr("The repeat password cannot be empty"));
            return;
        }

        if(repeatPassword.contains(" ")) {
            messageW_->showWidget(ui->repeatPassword->mapTo(this, QPoint(0, 0)).x() + 20,
                                  ui->repeatPassword->mapTo(this, QPoint(0, 0)).y() + ui->repeatPassword->height() - 10,
                                  tr("Password cannot have spaces"));
            return;
        }

        if(repeatPassword.size() > MAXINPUTSIZE) {
            messageW_->showWidget(ui->repeatPassword->mapTo(this, QPoint(0, 0)).x() + 20,
                                  ui->repeatPassword->mapTo(this, QPoint(0, 0)).y() + ui->repeatPassword->height() - 10,
                                  tr("Repeat password must be shorter than 16 characters"));
            return;
        }

        if(password != repeatPassword) {
            messageW_->showWidget(ui->repeatPassword->mapTo(this, QPoint(0, 0)).x() + 20,
                                  ui->repeatPassword->mapTo(this, QPoint(0, 0)).y() + ui->repeatPassword->height() - 10,
                                  tr("The two password inputs are not the same"));
            return;
        }

        switch(typeIdex)
        {
        case 0:
            type = UserAccount::AccountType::AdministratorAccountType;
            break;
        case 1:
            type = UserAccount::AccountType::StandardAccountType;
            break;
        }
        if(UserInstance::getIns()->isAdministratorUser()) {
            KAuth::Action action("org.nde.auth.helper.createuser");
            if (!action.isValid()) {
                return ;
            }
            action.setHelperId(QStringLiteral("org.nde.auth.helper"));

            action.addArgument(QStringLiteral("userName"), userName);
            action.addArgument(QStringLiteral("password"), password);
            action.addArgument(QStringLiteral("fullName"), "");
            action.addArgument(QStringLiteral("accountType"), type);
            action.addArgument(QStringLiteral("iconPath"), iconPath_);

            KAuth::ExecuteJob *job = action.execute();
            if(job->exec() && job->data()["isSuccess"].toBool())
            {
                iconPath_ = "";
                Q_EMIT backPage();
            }
            else {
                messageW_->showWidget(ui->userName->mapTo(this, QPoint(0, 0)).x() + 20,
                                      ui->userName->mapTo(this, QPoint(0, 0)).y() + ui->userName->height() - 10,
                                      tr("Failed to create user"));
                return;
            }
        }
        else {
            if(UserInstance::getIns()->getAccountsManager()->createAUser(userName, "", type, password, iconPath_, false))
            {
                iconPath_ = "";
                Q_EMIT backPage();
            }
            else
            {
                messageW_->showWidget(ui->userName->mapTo(this, QPoint(0, 0)).x() + 20,
                                      ui->userName->mapTo(this, QPoint(0, 0)).y() + ui->userName->height() - 10,
                                      tr("Failed to create user"));
                return;
            }
        }
    }
}

void CreateUser::refreshPage()
{
    avatar_->setAvatarPath(":/images/avatar.png");
}

void CreateUser::updateUserIcon(const QString path)
{
    iconPath_ = path;
    avatar_->setAvatarPath(path);
}

void CreateUser::mousePressEvent(QMouseEvent *)
{
    messageW_->close ();
}

void CreateUser::resizeEvent(QResizeEvent* event)
{
    QWidget::resizeEvent(event);
    messageW_->close ();
}

void CreateUser::onCloseNameMessage() {
    QString passText = ui->userName->text();
    int curCount = passText.count();
    if(curCount > MAXINPUTSIZE) {
        ui->password->setMaxLength(MAXINPUTSIZE + 1);
        messageW_->showWidget(ui->userName->mapTo(this, QPoint(0, 0)).x() + 20,
                              ui->userName->mapTo(this, QPoint(0, 0)).y() + ui->userName->height() - 10,
                              tr("Name must be shorter than 16 characters"));
    }
    else {
        messageW_->close ();
    }
}

void CreateUser::onClosePasswordMessage() {
    QString passText = ui->password->text();
    int curCount = passText.count();
    if(curCount > MAXINPUTSIZE) { //大于输入最大值就不允许输入
        ui->password->setMaxLength(MAXINPUTSIZE + 1);
        messageW_->showWidget(ui->password->mapTo(this, QPoint(0, 0)).x() + 20,
                              ui->password->mapTo(this, QPoint(0, 0)).y() + ui->password->height() - 10,
                              tr("Password must be shorter than 16 characters"));
    }
    else {
        messageW_->close ();
    }
}

void CreateUser::onCloseRepeatPasswordMessage() {
    QString passText = ui->repeatPassword->text();
    int curCount = passText.count();
    if(curCount > MAXINPUTSIZE) {
        ui->repeatPassword->setMaxLength(MAXINPUTSIZE + 1);
        messageW_->showWidget(ui->repeatPassword->mapTo(this, QPoint(0, 0)).x() + 20,
                              ui->repeatPassword->mapTo(this, QPoint(0, 0)).y() + ui->repeatPassword->height() - 10,
                              tr("Repeat password must be shorter than 16 characters"));
    }
    else {
        messageW_->close ();
    }
}

void CreateUser::onEditAvatar()
{
    Q_EMIT createUserAccountAvatar(nullptr);
}
