#include "edituser.h"
#include "ui_edituser.h"
#include <QFile>
#include <KAuth/KAuthAction>
#include <KAuth/KAuthExecuteJob>
#include "userinstance.h"
#include "../baseui/clickedframe.h"

#define PAGE_BUTTOM_BTN_DEL (Page::PAGE_BBTN_USER+1)

EditUser::EditUser(QWidget *parent) :
    Page(parent),
    ui(new Ui::EditUser),
    toggleLogin_(nullptr),
    toggleDeleteHome_(nullptr),
    layout_(nullptr),
    layoutDel_(nullptr),
    isDeleteHome_(false),
    avatar_(nullptr),
    messageW_(nullptr)
{
    ui->setupUi(this);

    QFile f(":/qss/edituser.qss");
    if(f.open(QIODevice::ReadOnly))
    {
        QString style = f.readAll().replace("\n", "");
        this->setStyleSheet(style);
    }

    avatar_ = new AvatarWidget();
    ui->horizontalLayout_2->addWidget(avatar_, 0, Qt::AlignTop | Qt::AlignHCenter);
    connect(avatar_, &AvatarWidget::clicked, this, &EditUser::onEditAvatar);

    // 用户类型选项
    QStringList typeList;
    typeList << tr("Admin Account") << tr("Standard Account");
    ui->typeComboBox->clear();
    ui->typeComboBox->addItems(typeList);
    connect(ui->typeComboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(onUserTypeChanged(int)));

    // 自动登录选项
    layout_ = new QHBoxLayout();
    layout_->setContentsMargins(0,0,0,0);
    layout_->setSpacing(0);
    ui->autoLoginWidget->setLayout(layout_);
    toggleLogin_ = new NdeWidget::QMaterialToggle(ui->autoLoginWidget);
    connect(toggleLogin_, &NdeWidget::QMaterialToggle::toggled, this, &EditUser::onToggleUserAutoLoginChanged);
    layout_->addWidget(toggleLogin_);

    // 是否删除用户目录
    layoutDel_ = new QHBoxLayout();
    layoutDel_->setContentsMargins(0,0,0,0);
    layoutDel_->setSpacing(0);
    ui->deleteWidget->setLayout(layoutDel_);
    toggleDeleteHome_ = new NdeWidget::QMaterialToggle(ui->deleteWidget);
    toggleDeleteHome_->setChecked(false);
    connect(toggleDeleteHome_, &NdeWidget::QMaterialToggle::toggled, this, &EditUser::onToggleDeleteHomeChecked);
    layoutDel_->addWidget(toggleDeleteHome_);

    ui->changeNameFrame->setTitle(tr("Change User Name"));
    connect(ui->changeNameFrame,&ClickedFrame::clicked,this,&EditUser::onChangeUserNameClick);
    ui->changePasswordFrame->setTitle(tr("Change Password"));
    connect(ui->changePasswordFrame,&ClickedFrame::clicked,this,&EditUser::onChangePasswordClick);

    messageW_ = new MessageWidget(this);
    messageW_->setDirection(MessageWidget::Direction::up);
    messageW_->setStartPos(20);
    messageW_->hide();
}

EditUser::~EditUser()
{
    delete ui;
    delete avatar_;
//    delete messageW_;
    //    messageW_ = nullptr;
}

void EditUser::backBtnClicked()
{

}

QList<QPair<int, QString> > EditUser::buttomBtnsPair()
{
    QList<QPair<int, QString> > pairs;
    pairs<<QPair<int, QString>(PAGE_BUTTOM_BTN_DEL, tr("Delete User"));
    return pairs;
}

QList<int> EditUser::buttomBtnsRed()
{
    QList<int> keys;
    keys << PAGE_BUTTOM_BTN_DEL;
    return keys;
}

QList<int> EditUser::buttomBtnsVisible()
{
    QList<int> keys;
    keys << PAGE_BUTTOM_BTN_DEL;
    return keys;
}

void EditUser::buttomBtnClicked(const int key)
{
    if(key == PAGE_BUTTOM_BTN_DEL)
    {
        if(userAccount_->automaticLogin()) { // 如果该用户设置了自动登录，则先取消自动登录再删除
            onToggleUserAutoLoginChanged(false);
        }

        if(UserInstance::getIns()->isAdministratorUser()) {
            KAuth::Action action("org.nde.auth.helper.deleteuser");
            if (!action.isValid()) {
                return ;
            }
            action.setHelperId(QStringLiteral("org.nde.auth.helper"));

            action.addArgument(QStringLiteral("userName"), userAccount_->userName());
            action.addArgument(QStringLiteral("deleteHome"), isDeleteHome_);
            KAuth::ExecuteJob *job = action.execute();
            if(job->exec())
            {
                Q_EMIT backPage();
            }
        }
        else {
            UserInstance::getIns()->getAccountsManager()->deleteUser(userAccount_, isDeleteHome_);
            if(!UserInstance::getIns()->getAccountsManager()->findAUserByName(userAccount_->userName()))
            {
                Q_EMIT backPage();
            }
        }
    }
}

void EditUser::refreshPage()
{
    //
}

void EditUser::updateUI() {

    if(userAccount_ == nullptr) return;

    ui->userNameLabel->setText(userAccount_->userName());

    // 头像图标
    QString iconPath = UserInstance::getIns()->getAccountsManager()->findUserIconFile(userAccount_->userName());
    if(iconPath.isEmpty() || iconPath == QString("/home/%1/.face").arg(userAccount_->userName()) || !QFile::exists(iconPath)) {
        iconPath = ":/images/avatar.png";
    }

    avatar_->setAvatarPath(iconPath);

    // 用户类型选项
    ui->typeComboBox->blockSignals(true);
    switch(userAccount_->accountType())
    {
    case UserAccount::AccountType::StandardAccountType:
        ui->typeComboBox->setCurrentIndex(1);
        break;
    case UserAccount::AccountType::AdministratorAccountType:
        ui->typeComboBox->setCurrentIndex(0);
        break;
    }
    ui->typeComboBox->blockSignals(false);

    // 自动登录选项
    toggleLogin_->blockSignals(true);
    toggleLogin_->setChecked(userAccount_->automaticLogin());
    toggleLogin_->blockSignals(false);

    // 是否删除用户目录
    toggleDeleteHome_->setChecked(false);

    // 判断该用户是否为登录状态，如果是则禁止修改用户名和删除用户
    if(UserInstance::getIns()->isUserLogining(userAccount_->userName()))
    {
        ui->changeNameFrame->setEnabled(false);
        Q_EMIT buttomBtnVisibleChanged(PAGE_BUTTOM_BTN_DEL, false);
    }
    //若未登录，则判断系统最后一个管理员用户
    else if(userAccount_->accountType() == UserAccount::AccountType::AdministratorAccountType && UserInstance::getIns()->isLastManager())
    {
        ui->changeNameFrame->setEnabled(true);
        Q_EMIT buttomBtnVisibleChanged(PAGE_BUTTOM_BTN_DEL, false);
    }
    else {
        ui->changeNameFrame->setEnabled(true);
        Q_EMIT buttomBtnVisibleChanged(PAGE_BUTTOM_BTN_DEL, true);
    }
}

void EditUser::mousePressEvent(QMouseEvent *)
{
    messageW_->close ();
}

void EditUser::resizeEvent(QResizeEvent* event)
{
    QWidget::resizeEvent(event);
    messageW_->close ();
}

void EditUser::onChangeUserNameClick()
{
    //登录的用户不能修改用户名
    if(UserInstance::getIns()->isUserLogining(userAccount_->userName())) return;

    Q_EMIT changeUserAccountName(userAccount_);
}

void EditUser::onChangePasswordClick()
{
    Q_EMIT changeUserAccountPasswd(userAccount_);
}

void EditUser::onToggleUserAutoLoginChanged(bool checked) {
    QString userName = userAccount_->userName();
    KAuth::Action action("org.nde.auth.helper.setautologin");
    if (!action.isValid()) {
        return ;
    }
    action.setHelperId(QStringLiteral("org.nde.auth.helper"));

    action.addArgument(QStringLiteral("userName"), userName);
    action.addArgument(QStringLiteral("autoLogin"), checked);
    KAuth::ExecuteJob *job = action.execute();
    job->exec();
    Q_EMIT changeAutoLogin();
}

void EditUser::onUserTypeChanged(int index) {
    if(userTypeClick_){
        return ;
    }
    userTypeClick_ = true;
    if(index == 1 && UserInstance::getIns()->isLastManager())
    {
        ui->typeComboBox->setCurrentIndex(0);
        messageW_->showWidget(ui->typeComboBox->mapTo(this, QPoint(0, 0)).x() + 20,
                              ui->typeComboBox->mapTo(this, QPoint(0, 0)).y() + ui->typeComboBox->height() - 10,
                              tr("The system must retain an administrator"));
    }
    else {
        if(UserInstance::getIns()->isAdministratorUser()) {
            KAuth::Action action("org.nde.auth.helper.setaccounttype");
            if (!action.isValid()) {
                return ;
            }
            action.setHelperId(QStringLiteral("org.nde.auth.helper"));

            action.addArgument(QStringLiteral("userName"), userAccount_->userName());
            action.addArgument(QStringLiteral("accountType"), index);

            KAuth::ExecuteJob *job = action.execute();
            job->exec();
        }
        else {
            UserAccount::AccountType type = UserAccount::AccountType::StandardAccountType;
            switch(index)
            {
            case 0:
                type = UserAccount::AccountType::AdministratorAccountType;
                break;
            case 1:
                type = UserAccount::AccountType::StandardAccountType;
                break;
            }
            if(!UserInstance::getIns()->getAccountsManager()->editAUserType(userAccount_->userName(), type)) {
                switch(userAccount_->accountType())
                {
                case UserAccount::AccountType::StandardAccountType:
                    ui->typeComboBox->setCurrentIndex(1);
                    break;
                case UserAccount::AccountType::AdministratorAccountType:
                    ui->typeComboBox->setCurrentIndex(0);
                    break;
                }
            }
        }
    }

    QTimer r;
    r.singleShot(200,[this](){
        this->userTypeClick_=false;
    });
}

void EditUser::onToggleDeleteHomeChecked(bool checked) {
    if(checked) {
        isDeleteHome_ = true;
    }
    else {
        isDeleteHome_ = false;
    }
}

void EditUser::onEditAvatar() {
    Q_EMIT changeUserAccountAvatar(userAccount_);
}

void EditUser::updateAccountAvatar(QString path)
{
    if(UserInstance::getIns()->isAdministratorUser()) {
        KAuth::Action action("org.nde.auth.helper.setavatar");
        if (!action.isValid()) {
            return ;
        }
        action.setHelperId(QStringLiteral("org.nde.auth.helper"));

        action.addArgument(QStringLiteral("userName"), userAccount_->userName());
        action.addArgument(QStringLiteral("iconPath"), path);

        KAuth::ExecuteJob *job = action.execute();
        if(job->exec())
        {
            avatar_->setAvatarPath(path);
        }
    }
    else {
        if(UserInstance::getIns()->getAccountsManager()->editAUserIcon(userAccount_->userName(), path))
            avatar_->setAvatarPath(path);
    }
}

void EditUser::changeName(QString name)
{
    ui->userNameLabel->setText(name);
}
