#ifndef EDITUSER_H
#define EDITUSER_H

#include <QWidget>
#include <QHBoxLayout>
#include <QtAccountsService/AccountsManager>
#include <QtAccountsService/UserAccount>
#include <designer/ndewidgetmaterialtoggle.h>
#include "avatarwidget.h"
#include "changepassword.h"
#include "changeusername.h"
#include "avatarsetting.h"
#include "../baseui/messagewidget.h"
#include "../baseui/page.h"

using namespace QtAccountsService;

namespace Ui {
class EditUser;
}

class EditUser : public Page
{
    Q_OBJECT
public:
    explicit EditUser(QWidget *parent = nullptr);
    ~EditUser();

public:
    //页面唯一键
    static QString  pageKey() {return "user_edituser";}

    //页面标题
    virtual QString pageTitle() {return tr("Edit User");}

    //Back按钮按下后的接口，主要用于back之前应该处理的一些操作
    virtual void backBtnClicked();

    //该页面底部的按钮QPair<key, text>
    virtual QList<QPair<int, QString> > buttomBtnsPair();
    //红色的底部按钮,底部按钮默认为蓝色
    virtual QList<int> buttomBtnsRed();
    virtual QList<int> buttomBtnsVisible();
    //底部按钮按下事件响应的接口
    virtual void buttomBtnClicked(const int key);

    //每次进入该界面需要做的操作
    virtual void refreshPage();

public:
    void setUserAccount(UserAccount *userAccount) {
        userAccount_ = userAccount;
        updateUI();
    }

    void updateUI();
    void updateAccountAvatar(QString path);
    void changeName(QString name);
Q_SIGNALS:
    void changeUserAccountName(UserAccount *userAccount);
    void changeUserAccountPasswd(UserAccount *userAccount);
    void changeUserAccountAvatar(UserAccount *userAccount);
    void changeAutoLogin();

private Q_SLOTS:
    void onUserTypeChanged(int index);
    void onToggleUserAutoLoginChanged(bool checked);
    void onChangeUserNameClick();
    void onChangePasswordClick();
    void onToggleDeleteHomeChecked(bool checked);
    void onEditAvatar();

protected:
    void mousePressEvent (QMouseEvent *);
    void resizeEvent(QResizeEvent* event);

private:
    Ui::EditUser *ui;
    UserAccount *userAccount_;
    NdeWidget::QMaterialToggle * toggleLogin_;
    NdeWidget::QMaterialToggle * toggleDeleteHome_;
    QHBoxLayout * layout_;
    QHBoxLayout * layoutDel_;
    bool isDeleteHome_;
    AvatarWidget *avatar_;
    MessageWidget *messageW_;
    bool userTypeClick_{false};
};

#endif // EDITUSER_H
