#include "users.h"
#include "ui_users.h"
#include <QFile>
#include <QString>
#include <KAuth/KAuthAction>
#include <KAuth/KAuthExecuteJob>

#include "userinstance.h"
#include "../baseui/common.h"

Users::Users(QWidget *parent) :
    Page(parent),
    ui(new Ui::Users),
    layout_(nullptr),
    avatar_(nullptr),
    messageW_(nullptr)
{
    ui->setupUi(this);

    QFile f(":/qss/users.qss");
    if(f.open(QIODevice::ReadOnly))
    {
        QString style = f.readAll().replace("\n", "");
        this->setStyleSheet(style);
    }

    messageW_ = new MessageWidget(this);
    messageW_->setDirection(MessageWidget::Direction::up);
    messageW_->setStartPos(20);
    messageW_->hide();

    initUI();

    // 其他用户
    loadOtherUser();
}

Users::~Users()
{
    delete ui;
    delete toggleLogin_;
    delete layout_;
    delete avatar_;
    delete messageW_;
    messageW_ = nullptr;
}

void Users::refreshPage()
{

}

void Users::initUI()
{
    QString iconPath = nullptr;
    UserAccount * currentUser = UserInstance::getIns()->getCurrentUser();
    if(currentUser == nullptr) {
        iconPath = ":/images/avatar.png";
    }
    else {
        iconPath = UserInstance::getIns()->getAccountsManager()->findUserIconFile(currentUser->userName());
        if(iconPath.isEmpty() || iconPath == QString("/home/%1/.face").arg(currentUser->userName()) || !QFileInfo(iconPath).exists()) {
            iconPath = ":/images/avatar.png";
        }
    }

    //用户头像
    avatar_ = new AvatarWidget();
    ui->horizontalLayout_2->addWidget(avatar_, 0, Qt::AlignTop | Qt::AlignHCenter);
    avatar_->setAvatarPath(iconPath);
    connect(avatar_, &AvatarWidget::clicked, this, &Users::onEditAvatar);

    //用户名
    if(currentUser == nullptr) {
        ui->currentUser->setText(UserInstance::getIns()->getCurrentUserName());
    }
    else {
        ui->currentUser->setText(currentUser->userName());
    }

    //用户名
    ui->changeUserNameFrame->setTitle(tr("Change User Name"));
    ui->changeUserNameFrame->setEnabled(false);

    //密码
    ui->changePasswordFrame->setTitle(tr("Change Password"));
    connect(ui->changePasswordFrame,SIGNAL(clicked()),this,SLOT(onChangePasswordClick()));

    // 用户类型选项
    QStringList typeList;
    typeList << tr("Admin Account") << tr("Standard Account");
    ui->typeComboBox->clear();
    ui->typeComboBox->addItems(typeList);
    if(currentUser == nullptr) {
        ui->frame_usertype->setEnabled(false);
    }
    else
    {
        switch(currentUser->accountType())
        {
        case UserAccount::AccountType::StandardAccountType:
            ui->typeComboBox->setCurrentIndex(1);
            break;
        case UserAccount::AccountType::AdministratorAccountType:
            ui->typeComboBox->setCurrentIndex(0);
            break;
        }
        connect(ui->typeComboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(onUserTypeChanged(int)));
    }

    // 自动登录选项
    layout_ = new QHBoxLayout();
    layout_->setContentsMargins(0,0,0,0);
    layout_->setSpacing(0);
    ui->autoLoginWidget->setLayout(layout_);
    toggleLogin_ = new NdeWidget::QMaterialToggle(ui->autoLoginWidget);
    layout_->addWidget(toggleLogin_);
    if(currentUser == nullptr) {
        ui->frame_autologin->setEnabled(false);
        toggleLogin_->setChecked(false);
    }
    else
    {
        toggleLogin_->setChecked(currentUser->automaticLogin());
        connect(toggleLogin_, &NdeWidget::QMaterialToggle::toggled, this, &Users::onToggleUserAutoLoginChanged);
    }

    // add User
    connect(ui->createUserBtn, &QPushButton::clicked,this,&Users::onCreateUserClick);
    if(currentUser == nullptr)
    {
        ui->createUserFrame->setEnabled(false);
    }
}

void Users::loadOtherUser()
{
    UserAccountList otherUserList = UserInstance::getIns()->getOtherUserList();

    for(int i = 0; i < otherUserList.size(); ++i) {
        QString name = (otherUserList.at(i))->userName();
        OtherUserFrame *otherUserFrame = new OtherUserFrame(otherUserList.at(i));
        otherUserFrame->setObjectName(otherUserList.at(i)->userName());
        connect(otherUserFrame, &OtherUserFrame::changeUsersIndex, this, &Users::editUserAccount);
        ui->layout_list->addWidget(otherUserFrame);
    }
}

void Users::clearOtherUser()
{
    while(ui->layout_list->count())
    {
        QLayoutItem * item = ui->layout_list->takeAt(0);
        if(item->widget()) item->widget()->deleteLater();
        delete item;
    }
}

void Users::updateUserList() {
    clearOtherUser();
    loadOtherUser();
}

void Users::mousePressEvent(QMouseEvent *)
{
    messageW_->close ();
}

void Users::resizeEvent(QResizeEvent* event)
{
    QWidget::resizeEvent(event);
    messageW_->close ();
}

void Users::onUserTypeChanged(int index) {
    if(userTypeClick_){
        return ;
    }
    userTypeClick_ = true;
    UserAccount * currentUser = UserInstance::getIns()->getCurrentUser();
    if(index == 1 && UserInstance::getIns()->isLastManager())
    {
        ui->typeComboBox->setCurrentIndex(0);
        messageW_->showWidget(ui->typeComboBox->mapTo(this, QPoint(0, 0)).x() + 20,
                              ui->typeComboBox->mapTo(this, QPoint(0, 0)).y() + ui->typeComboBox->height() - 10,
                              tr("The system must retain an administrator"));
    }
    else {
        if(UserInstance::getIns()->isAdministratorUser()) {
            KAuth::Action action("org.nde.auth.helper.setaccounttype");
            if (!action.isValid()) {
                return ;
            }
            action.setHelperId(QStringLiteral("org.nde.auth.helper"));

            action.addArgument(QStringLiteral("userName"), currentUser->userName());
            action.addArgument(QStringLiteral("accountType"), index);

            KAuth::ExecuteJob *job = action.execute();
            job->exec();
        }
        else {
            UserAccount::AccountType type = UserAccount::AccountType::StandardAccountType;
            switch(index)
            {
            case 0:
                type = UserAccount::AccountType::AdministratorAccountType;
                break;
            case 1:
                type = UserAccount::AccountType::StandardAccountType;
                break;
            }
            if(!UserInstance::getIns()->getAccountsManager()->editAUserType(currentUser->userName(), type)) {
                switch(currentUser->accountType())
                {
                case UserAccount::AccountType::StandardAccountType:
                    ui->typeComboBox->setCurrentIndex(1);
                    break;
                case UserAccount::AccountType::AdministratorAccountType:
                    ui->typeComboBox->setCurrentIndex(0);
                    break;
                }
            }
        }
    }

    QTimer r;
    r.singleShot(200,[this](){
        this->userTypeClick_=false;
    });
}

void Users::onToggleUserAutoLoginChanged(bool checked) {
    UserAccount * currentUser = UserInstance::getIns()->getCurrentUser();
    //    if(isAdministratorUser()) {
    KAuth::Action action("org.nde.auth.helper.setautologin");
    if (!action.isValid()) {
        return ;
    }
    action.setHelperId(QStringLiteral("org.nde.auth.helper"));

    action.addArgument(QStringLiteral("userName"), currentUser->userName());
    action.addArgument(QStringLiteral("autoLogin"), checked);
    KAuth::ExecuteJob *job = action.execute();
    job->exec();
    //    }
    //    else {
    //        UserInstance::getIns()->getAccountsManager()->editAUserAutoLogin(currentUser_->userName(), currentUser_->userName(), checked);
    //    }
}

void Users::onCreateUserClick() {
    Q_EMIT nextPage(CreateUser::pageKey());
}

void Users::updateAutoLogin() {
    UserAccount * currentUser = UserInstance::getIns()->getCurrentUser();
    if(currentUser != nullptr)
    {
        toggleLogin_->setChecked(currentUser->automaticLogin());
    }
}

void Users::updateUserIcon(const QString path)
{
    QString iconPath = nullptr;
    UserAccount * currentUser = UserInstance::getIns()->getCurrentUser();
    if(currentUser == nullptr) {
        iconPath = ":/images/avatar.png";
    }
    else {
        iconPath = path;
        if(iconPath.isEmpty() || iconPath == QString("/home/%1/.face").arg(currentUser->userName()) || !QFileInfo(iconPath).exists()) {
            iconPath = ":/images/avatar.png";
        }
        UserInstance::getIns()->getAccountsManager()->editAUserIcon(currentUser->userName(), path);
    }
    avatar_->setAvatarPath(iconPath);
}

void Users::onEditAvatar()
{
    UserAccount * currentUser = UserInstance::getIns()->getCurrentUser();
    Q_EMIT editUserAccountAvatar(currentUser);
}

void Users::onChangePasswordClick()
{
    UserAccount * currentUser = UserInstance::getIns()->getCurrentUser();
    Q_EMIT editUserAccountPasswd(currentUser);
}
