#include "userstack.h"
#include "userinstance.h"

#include "userinstance.h"
#include "changepassword.h"
#include "changeusername.h"
#include "createuser.h"
#include "users.h"
#include "avatarsetting.h"
#include "../baseui/common.h"

UserStack::UserStack(QWidget *parent) : StackPage(parent)
{
    this->setFirstPage(Users::pageKey());
}

Page *UserStack::createPage(const QString key)
{
    if(key == Users::pageKey())
    {
        Users * page = new Users(this);
        connect(page, &Users::editUserAccountPasswd, this, &UserStack::slotChangeUserAccountPasswd);
        connect(page, &Users::editUserAccount, this, &UserStack::slotEditUserAccount);
        connect(page, &Users::editUserAccountAvatar, this, &UserStack::slotEditUserAccountAvatar);
        connect(UserInstance::getIns()->getAccountsManager(), &AccountsManager::userAdded,
                this, &UserStack::slotUserListChange);
        connect(UserInstance::getIns()->getAccountsManager(), &AccountsManager::userDeleted,
                this, &UserStack::slotUserListChange);
        return page;
    }
    else if (key == CreateUser::pageKey())
    {
        CreateUser * page = new CreateUser(this);
        connect(page, &CreateUser::createUserAccountAvatar, this, &UserStack::slotEditUserAccountAvatar);
        return page;
    }
    else if (key == EditUser::pageKey())
    {
        EditUser * page = new EditUser(this);
        connect(page, &EditUser::changeUserAccountName, this, &UserStack::slotChangeUserAccountName);
        connect(page, &EditUser::changeUserAccountPasswd, this, &UserStack::slotChangeUserAccountPasswd);
        connect(page, &EditUser::changeUserAccountAvatar, this, &UserStack::slotEditUserAccountAvatar);
        connect(page, &EditUser::changeAutoLogin, this, &UserStack::slotChangeAutoLoing);
        return page;
    }
    else if (key == AvatarSetting::pageKey())
    {
        AvatarSetting * page = new AvatarSetting(this);
        connect(page , &AvatarSetting::avatarPathChange, this, &UserStack::slotAvaterPathChange);
        return page;
    }
    else if (key == ChangeUserName::pageKey())
    {
        ChangeUserName * page = new ChangeUserName(this);
        connect(page, &ChangeUserName::changeAccountName, this, &UserStack::slotUserChange);
        return page;
    }
    else if (key == ChangePassword::pageKey())
    {
        ChangePassword * page = new ChangePassword(this);
        return page;
    }
    return nullptr;
}

void UserStack::slotUserChange(QString name)
{
    Page *ePage = this->getPage(EditUser::pageKey());
    EditUser * editPage = qobject_cast<EditUser*>(ePage);
    if(editPage)
    {
        editPage->changeName(name);
    }
}
void UserStack::slotUserListChange()
{
    Page *page = this->getPage(Users::pageKey());
    Users * tmpPage = qobject_cast<Users*>(page);
    if(tmpPage)
    {
        tmpPage->updateUserList();
    }
}

void UserStack::slotChangeUserAccountName(QtAccountsService::UserAccount *account)
{
    this->setNextPage(ChangeUserName::pageKey());
    Page *page = this->getPage(ChangeUserName::pageKey());
    ChangeUserName * tmpPage = qobject_cast<ChangeUserName*>(page);
    if(tmpPage)
    {
        tmpPage->setUserAccount(account);
    }
}

void UserStack::slotChangeUserAccountPasswd(UserAccount *account)
{
    this->setNextPage(ChangePassword::pageKey());
    Page *page = this->getPage(ChangePassword::pageKey());
    ChangePassword * tmpPage = qobject_cast<ChangePassword*>(page);
    if(tmpPage)
    {
        tmpPage->setUserAccount(account);
    }
}

void UserStack::slotEditUserAccount(UserAccount *account)
{
    this->setNextPage(EditUser::pageKey());
    Page *page = this->getPage(EditUser::pageKey());
    EditUser * tmpPage = qobject_cast<EditUser*>(page);
    if(tmpPage)
    {
        tmpPage->setUserAccount(account);
    }
}

void UserStack::slotEditUserAccountAvatar(UserAccount *account)
{
    this->setNextPage(AvatarSetting::pageKey());
    Page *page = this->getPage(AvatarSetting::pageKey());
    AvatarSetting * tmpPage = qobject_cast<AvatarSetting*>(page);
    if(tmpPage)
    {
        tmpPage->setUserAccount(account);
    }
}

void UserStack::slotAvaterPathChange(QString path)
{
    Page *tp = this->getTopPage();
    Users * usersPage  = qobject_cast<Users*>(tp);
    if(usersPage)
    {
        usersPage->updateUserIcon(path);
    }
    EditUser * editPage  = qobject_cast<EditUser*>(tp);
    if(editPage)
    {
        editPage->updateAccountAvatar(path);
    }
    CreateUser * createPage  = qobject_cast<CreateUser*>(tp);
    if(createPage)
    {
        createPage->updateUserIcon(path);
    }
}

void UserStack::slotChangeAutoLoing()
{
    Page *page = this->getPage(Users::pageKey());
    Users * tmpPage = qobject_cast<Users*>(page);
    if(tmpPage)
    {
        tmpPage->updateAutoLogin();
    }
}
