#ifndef VOICE_H
#define VOICE_H

#include <QWidget>
#include <designer/ndewidgetmaterialtoggle.h>
#include <pulse/pulseaudio.h>
#include <pulse/glib-mainloop.h>
#include "soundcarditem.h"
#include "../baseui/page.h"

namespace Ui {
class Voice;
}

class Voice : public Page
{
    Q_OBJECT
public:
    explicit Voice(QWidget *parent = nullptr);
    ~Voice();

public:
    //页面唯一键
    static QString pageKey() {return "voice";}

    //页面标题
    virtual QString pageTitle() {return tr("Voice");}

    //该页面底部的按钮QPair<key, text>
    virtual QList<QPair<int, QString> > buttomBtnsPair(){
        return QList<QPair<int, QString> >();
    }

    //刷新界面显示接口
    virtual void refreshPage();

public:
    pa_threaded_mainloop *get_mainloop() const {return m_mainLoop;}
    void receiveServer();
    void receiveCards();
    void receiveSinks();
    void receiveSources();
    void requestServerInfoUpdate();
    void requestSinkInfoUpdate(uint32_t idx);
    void requestSourceInfoUpdate(uint32_t idx);
    void requestCardInfoUpdate(uint32_t idx);
    void updateToggleSpeaker();
    void updateToggleMicro();
    void handleOutputVolumeWheelEvent(QWheelEvent *event);
    void handleInputVolumeWheelEvent(QWheelEvent *event);
    void saveCardInfoData(const pa_card_info *info);
    void removeCardByIndex(uint32_t index);
    void save_sink_to_PortList(unsigned int index, pa_sink_port_info *port);
    void clean_sink_PortList();
    void save_source_to_PortList(unsigned int index, pa_source_port_info *port);
    void clean_source_PortList();
    void setDefaultSink(QString name)
    {
        default_sink = name;
    }
    void setDefaultSource(QString name)
    {
        default_source = name;
    }

Q_SIGNALS:
    void contextStateChanged(pa_context_state_t state);
    void serverInfoChanged();
    void sinkInfoChanged(uint32_t idx);
    void sourceInfoChanged(uint32_t idx);
    void cardInfoChanged(uint32_t idx);
    void setSpeakerMute(bool mute);
    void setMicroMute(bool mute);
    void addCardItem(int index);
public Q_SLOTS:
    void setContextState(pa_context_state_t state);
    void receiveServerInfo();
    void receiveSinkInfo(uint32_t idx);
    void receiveSourceInfo(uint32_t idx);
    void receiveCardInfo(uint32_t idx);
    void setSpeakerSwitch(bool mute);
    void setMicroSwitch(bool mute);
    void onAddCardItem(int index);

protected:
    bool eventFilter(QObject * watched, QEvent * event) override;

private:
    void InitOutputUI();
    void InitInputUI();
    void setupSubscription();
    static void contextSubscriptionCallback(pa_context *context, pa_subscription_event_type_t t, uint32_t idx, void *userdata);
    static void serverInfoCallback(pa_context *context, const pa_server_info*info, void *userdata);
    static void cardInfoCallback(pa_context *context, const pa_card_info *info, int isLast, void *userdata);
    static void sinkInfoCallback(pa_context *context, const pa_sink_info *info, int isLast, void *userdata);
    static void sourceInfoCallback(pa_context *context, const pa_source_info *info, int isLast, void *userdata);

private Q_SLOTS:
    void onToggleSpeakerChecked(bool check);
    void onToggleMicroChecked(bool check);
    void onOutPutVolumeChanged(int value);
    void onInPutVolumeChanged(int value);
    void onBalanceChanged(int value);
    void handleContextStateChanged();
    void connectContext();
    void handleOutPutComboBoxIndexChanged(int index);
    void handleInPutComboBoxIndexChanged(int index);
    void handleCardItemChanged(int index, QString name);

private:
    Ui::Voice *ui;
    NdeWidget::QMaterialToggle * speaker_switch;
    NdeWidget::QMaterialToggle * microphone_switch;
    QTimer m_reconnectionTimer;
    pa_threaded_mainloop *m_mainLoop;
    pa_mainloop_api *m_mainLoopApi;
    pa_context* m_context;
    QString default_sink;
    QString default_source;
    pa_cvolume m_sink_volume;
    pa_cvolume m_source_volume;
    pa_channel_map m_channelMap;
    pa_context_state_t m_contextState;
    bool m_ready;
    uint32_t m_sink_index;
    uint32_t m_source_index;
    int active_sink_port;
    int active_source_port;
    unsigned int card_count;
    QMap<unsigned int, pa_sink_port_info> m_sink_PortList;
    QMap<unsigned int, pa_source_port_info> m_source_PortList;
    QMap<unsigned int, SoundCardItem *> m_CardList;
    QMap<unsigned int, pa_card_info*> m_CardInfoList;
    QMap<unsigned int, unsigned int> m_CardIndexlist;
};

#endif // VOICE_H
